/**
 * @(#) OhElement; base class of element models.
 *
 */

package ohymosj;

import ohymosj.*;
import java.util.*;
import java.io.*;

/**
 * <japanese>
 * {^vff\NXłB<p>
 *
 * ۃNXłB<p>
 *
 * vff́Ap^, ԗ, M[q, M[qf[^oɋLA
 * p^EԗʁEM[q̓̓f[^pĐvZsȂAԗʂXVA
 * M[qo̓f[^𑗐M܂B
 * vZƂ̂AevffɋʂȍƂƋʂłȂƂƂ𕪗A
 * ʕ̂ݒoEKî{^vffłB
 * ̃NX́A{^vff\NX`Ă܂B<p>
 *
 * vffLSM[q, SM[q́A
 * _receivePortArray, _sendPortArray ɓo^܂B
 * {^vffł́A̓o^葱ɊւKisȂA
 * vff̍쐬Ƃ̊ȑfi߂ƂƂɁA
 * evffɓo^葱Ɋւ铯̐M^Ă܂B
 * ܂Aevf̑M[q̐ڑ͕nfsȂ܂A
 * ̍ƂeՂɂ邽߁AvffL鑗M[qɊւ錟@\`܂B<p>
 *
 * ʏ́Avff̏ԂXV邽߂ɕKvȃf[^͎M[qoĎ擾悤ɐ݌v܂A
 * ނ𓾂Ȃꍇ́AuڒʐMvɂĕKvȃf[^擾dg݂[U`ł悤ɁA
 * ڒʐMvf̔zL邽߂̃oϐ _objectArray pӂĂ܂B
 * </japanese>
 *
 * <p>
 *
 * OhElement;
 *
 * Abstract class for element models. <p>
 *
 * An element model
 *
 * <ul>
 *  <li> has data members for storing parameters, state variables, receive ports, send port,
 *  <li> calculates the time change of the state variables using input data obtained through the receive ports, and
 *  <li> sends output data through the send ports.
 * </ul>
 *
 * In hydrolgic simulations, various element models are needed because various processes are involved. <p>
 *
 * Class OhElement expresses the common part of those hydrologic element models.
 *
 * @author <a href="mailto:satoh@wr.kuciv.kyoto-u.ac.jp">satoh</a>
 * @version $Revision: 1.145 $ $Date: 2004/03/08 17:03:02 $
 */
public abstract class OhElement extends OhObject
{
  // @begin {fields} -----------------------------------------------------
  
  /**
   * <japanese>
   * <a href="OhElement.html#initialWork()">OhElement.initialWork()</a> 
   * I(true)ۂ(false)\tOłB
   * </japanese>
   * <p>
   * Whether the initial work has been completed or not.
   *
   */
  protected boolean _initialWorkDoneP;

  /**
   * <japanese>
   * <a href="OhElement.html#canYouShareInfo()">OhElement.canYouShareInfo()</a>Share_info 
   * I(true)ۂ(false)\tOłB
   * </japanese>
   * <p>
   * Whether the "shareInfo" has been completed or not.
   *
   */
  protected boolean _shareInfoDoneP;

  /**
   * <japanese>
   * Ɨp̐`XgłB
   * _receivePortArray, _sendPortArray ̍쐬Ɏgp܂B
   * </japanese>
   * <p>
   * A list for storing Object.
   * It is used as an object for work at the time of creation of "_receivePortArray" and "_sendPortArray".
   *
   */
  protected OhLinkedList _objectList; 

  /**
   * <japanese>
   * vfLM[q̔złB
   * </japanese>
   * <p>
   * Array of "OhReceivePort".
   *
   */
  protected Object[] _receivePortArray;

  /**
   * <japanese>
   * _receivePortArray ̗vfłB
   * </japanese>
   * <p>
   * Number of receive ports.
   *
   */
  protected int _numberOfReceivePorts;

  /**
   * <japanese>
   * vfL鑗M[q̔złB
   * </japanese>
   * <p>
   * Array of "OhSendPort".
   *
   */
  protected Object[] _sendPortArray;

  /**
   * <japanese>
   * _sendPortArray ̗vfłB
   * </japanese>
   * <p>
   * Number of send ports.
   *
   */
  protected int _numberOfSendPorts;

  /**
   * <japanese>
   * ڒʐMvf̔złB
   * </japanese>
   * <p>
   * Array of "OhObject".
   *
   */
  protected Object[] _objectArray;

  /**
   * <japanese>
   * _objectArray ̗vfłB
   * </japanese>
   * <p>
   * Number of objects.
   *
   */
  protected int _numberOfObjects;

  /**
   * <japanese>
   * f[^łB
   * </japanese>
   * <p>
   * Data string.
   *
   */
  protected String _dataString;

  /**
   * <japanese>
   * ݎ(vZςݎ)łB
   * </japanese>
   * <p>
   * Current(Calculated time) time.
   * "OhTimePoint" is defined in "OhTimePoint.java".
   *
   */
  protected OhTimePoint _currentTime;

  /**
   * <japanese>
   * vZPXebv̎(^CXebv)[sec]łB
   * </japanese>
   * <p>
   * Time step.[sec]
   *
   */
  protected long _timeStep;


  /**
   * <japanese>
   * vZɂďԂm肵Ă鎞łB
   * </japanese>
   * <p>
   * Time which the state has decided in repeated calculation.
   *
   */
  protected OhTimePoint _fixedTime;

  /**
   * <japanese>
   * vZڕW[sec]łB<p>
   *
   * iӁ)
   * vZvfł́AvZJn _fixedTime, _targetTime  _currentTime ƈvKv܂B
   * <a href=OhElement.html#setInitialState(OhBufferedReader)">OhElement.setInitialState(OhBufferedReader)</a>
   *  _currentTime ݒ肳ꂽA
   * <a href=OhElement.html#setCur2Fix2Tar()">OhElement.setCur2Fix2Tar()</a> ĂԂA
   *  _targetTime = _fixedTime = _currentTime; ƂĂB
   * </japanese>
   * <p>
   * Repeated calculation target time. [sec] <p>
   * 
   * It is necessary to make "_fixedTime" and "_targetTime" in agreement with "_currentTime"
   * with the element which carries out repeated calculation at the time of a calculation start. <p>
   *
   * After initial setting of the "_currentTime" is carried out by "setInitialState()",
   * call "void cur2fix2tar()" which the bottom defines or give as direct
   * "_targetTime = _fixedTime = _currentTime;".
   *
   */
  protected OhTimePoint _targetTime;

  /**
   * <japanese>
   * vZ悤Ƃ̂ɌvZłȂ̗vfɂƂɁA
   * _calReqP = true ɐݒ肳悤ɂ܂B
   * āAOhElement  _calReqP = true ̂Ƃ͌vZ悤ɓw͂܂B
   * _calReqP ͂̃NXŎn߂ĒǉꂽoϐłB
   * ̃NX̃IuWFNgƂɂ _calReqP  false ɏ܂B
   * </japanese>
   * <p>
   * Flag of request of calculation.
   * Initial value is NO.
   * When a cause incalculable although the lower-stream-of-a-river side tended to calculate is in this element,
   * it is made to be set as "_calReqP = true".
   * And, "OhElement" tries to calculate hard at the time of "_calReqP = true".
   *
   */
  protected boolean _calReqP;
  
  // @end {instance variable or static variable} -------------------------
  
  
  // @begin {abstract methods} -------------------------------------------

  // 1. creation and reference of "_receivePortArray", "_sendPortArray" and "_objectArray"
  
  /**
   * <japanese>
   * hNX̑SM[qo^܂B<p>
   *
   * ̃\bh́AhNX̑SM[q
   * <a href="OhElement.html#register(java.lang.Object)">OhElement.register(Object)</a>
   * <a href="OhElement.html#register(java.lang.Object, int)">OhElement.register(Object, int)</a>
   * ɂo^܂B<p>
   *
   * o^ĂȂ[q͐ڑłȂ̂őMɗpł܂B<p>
   *
   * Ⴆ΁AhNXŎM[q abc `ꍇA
   *
   * <pre>
   *  public void registerReceivePort() {
   *    register((Object) abc);
   *  }
   * </pre>
   *
   * Ƃ΁Aabc M[qƂēo^ł܂B
   * </japanese>
   * <p>
   * Declares method registerReceivePorts() as abstract method.
   * In classes derived from class Element, its definition should be given. <p>
   *
   * "registerReceivePorts()" registers all the receiving ports of an inherited class,
   * and all sending ports by
   * <a href="OhElement.html#register(java.lang.Object)">OhElement.register(Object)</a>
   * <a href="OhElement.html#register(java.lang.Object, int)">OhElement.register(Object, int)</a>. <p>
   *
   * The port which is not registered is not applicable to sending and receiving,
   * since it is not connectable. <br>
   * For example, when an inherited class defines the receiving port "abc",
   * if it is performed as follows, "abc" can be registered as a receiving terminal.
   *
   * <pre>
   *   void registerReceivePort() {
   *     register((Object)abc);
   *   }
   * </pre>
   *
   *
   * @exception OhError if an error occurs
   */
  public abstract void registerReceivePorts() throws OhError;


  /**
   * <japanese>
   * hNX̑SM[qo^܂B<p>
   *
   * ̃\bh́AhNX̑SM[q
   * <a href="OhElement.html#register(java.lang.Object)">OhElement.register(Object)</a>
   * <a href="OhElement.html#register(java.lang.Object, int)">OhElement.register(Object, int)</a>
   * ɂo^܂B<p>
   *
   * o^ĂȂ[q͐ڑłȂ̂őMɗpł܂B<p>
   *
   * Ⴆ΁AhNXőM[q abc `ꍇA
   *
   * <pre>
   *  public void registerSendPort() {
   *    register((Object) abc);
   *  }
   * </pre>
   *
   * Ƃ΁Aabc 𑗐M[qƂēo^ł܂B
   * </japanese>
   * <p>
   *
   * Declares method "registerSendPorts() as abstract method.
   * In classes derived from class Element, its definition should be given. <p>
   *
   * "registerSendPorts()" registers all the receiving ports of an inherited class,
   * and all sending ports by
   * <a href="OhElement.html#register(java.lang.Object)">OhElement.register(Object)</a>
   * <a href="OhElement.html#register(java.lang.Object, int)">OhElement.register(Object, int)</a>. <p>
   *
   * The port which is not registered is not applicable to sending and receiving,
   * since it is not connectable.
   * For example, when an inherited class defines the receiving port "abc",
   * if it is performed as follows, "abc" can be registered as a receiving terminal.
   *
   * <pre>
   *   void registerSendPort() {
   *     register((Object)abc);
   *   }
   * </pre>
   *
   * @exception OhError if an error occurs
   */
  public abstract void registerSendPorts() throws OhError;

  // 2. connection function

  // 3. numerical initialization
  
  /**
   * <japanese>
   * p^ݒ肷郁\bhAehNXɉĒ`܂B
   * ̓Xg[ aBr Ŏw肳t@Cf[^ǂ݂Ƃăp^ݒ肵܂B<p>
   * </japanese>
   * <p>
   *
   * Declares method "setParameter()" as abstract method.
   * In classes derived from class Element, its definition should be given. <p>
   *
   * Parameters are read in the input stream "aBr".
   * It is necessary to create an element model according to each inherited class.
   * Data is read in the file specified by the input stream "aBr",
   * and the amount of initial states is set up. <p>
   *
   * Please have the program and consistency which save the last state at a file.
   *
   * @param aBr a <code>OhBufferedReader</code> value
   * @exception OhError if an error occurs
   */
  public abstract void setParameter(OhBufferedReader aBr) throws OhError;

  /**
   * <japanese>
   * ԗʂݒ肷郁\bhAehNXɉĒ`܂B
   * ̓Xg[ aBr Ŏw肳t@Cf[^ǂ݂Ƃďԗʂݒ肵܂B<p>
   *
   * ŏIԂt@CɕۑvOƂ܂悤ɂĂB
   * </japanese>
   * <p>
   * Declares method "setInitialState()" as abstract method.
   * In classes derived from class Element, its definition should be given. <p>
   *
   * The method which sets up the amount of initial states is defined according to each inherited class.
   *
   * @param aBr a <code>OhBufferedReader</code> value
   * @exception OhError if an error occurs
   */
  public abstract void setInitialState(OhBufferedReader aBr) throws OhError; 

  // 4. time about a simulation

  // 5. execution of calculation

  /**
   * <japanese>
   * ̏o͂sȂ܂B<p>
   *
   * ԗʂ߂ɁA_currentTime ł̏o͂`܂B
   * M[qɎƏo͒l݂܂B
   * _currnetTime ł̏o͂vZłꍇ true Ԃ܂B
   * Kvȓ̓f[^ĂȂȂǂāA
   * _currentTime ̏o͂vZłȂꍇ false Ԃ܂B<p>
   *
   * vfɂẮAsetInitialState() ̒
   * ̏o͂悤ɂs悢m܂B
   * Ƃ΁Alݒ̂߂ɗ^镶ɂāA
   * ̏o͂̂ς肷ꍇȂǂłB
   * ꍇ́AinitialOutput() ͉Ȃ悤ɒ`āA
   * setInitialState() ŏ̏o͂悤ɒ`Ă\܂B
   * </japanese>
   * <p>
   *
   * An initial output.
   * In classes derived from class Element, its definition should be given. <p>
   *
   * The output in "_currentTime" is defined immediately after deciding the amount of initial states.
   * Time and an output value are written in a sending port.
   * "true" is returned when the output in "_currentTime" is able to be calculated.
   * "false" is returned when the output of "_currentTime" is not able to be calculated
   * by required input data not being obtained.
   *
   * @return a <code>boolean</code> value
   * @exception OhError if an error occurs
   */
  public abstract boolean initialOutput() throws OhError;

  /**
   * <japanese>
   * ^CXebv̌vZsȂ܂B<p>
   *
   * ehNXŃfɉ _timeStep [sec] vZA
   * ߂lƂĕԂ\bh`ĉB<p>
   * 
   * ܂A_timeStep ǂ̂悤ȏŁAǂ̂悤Ȓlɂ邩`܂B<p>
   * 
   * Ⴆ΁A_timeStep l 3600 [sec] ɐݒ肷ꍇA
   *
   * <pre>
   *  public long calculateTimeStep() {
   *    return 3600;
   *  }
   * </pre>
   *
   * ƒ`ĉB<p>
   * 
   * ܂AL̂悤Ƀoϐ _k ̒lɂ _timeStep ύX邱Ƃł܂B<p>
   *
   * <pre>
   *  public long calculateTimeStep() {
   *    return (_k > 0) ? 360 : 60;
   *  }
   *
   *  public long calculateTimeStep() {
   *    return 360 * pow(10, _k);
   *  }
   * </pre>
   *
   * _timeStep ́Awork() ̒Őݒ肳̂ŁA
   *
   * <pre>
   *  public long calculateTimeStep() {
   *    _timeStep = 1;
   *    return 1;
   *  }
   * </pre>
   *
   * ̂悤ɁÃ\bh̒ŕύXȂłB<p>
   * 
   * ۃ\bhȂ̂ŁAhNXŕK`ĉB<p>
   *
   * ߂l: ̌vZ̃^CXebvB[sec]
   * </japanese>
   * <p>
   *
   * Calculation of a time step.
   * In classes derived from class Element, its definition should be given. <p>
   *
   * Please calculate "_timeStep" according to a model in each inherited class,
   * and define the method which returns it as a return value.
   * That is, it defines under what conditions "_timeStep" is made into what value.
   *
   * @return a <code>long</code> value
   * @exception OhError if an error occurs
   */
  public abstract long calculateTimeStep() throws OhError;
  
  /**
   * <japanese>
   * vZJn̉Es̔f܂B<p>
   *
   * ܂A_currentTime  _terminalTime ȏ̏ꍇAvZς݂Ɣf false Ԃ܂B<p>
   *
   * ɁASĂ̏LM[q getNecessaryTimeFrom(), getNecessaryTimeTo() Ԃ
   * f[^pbN擾\ł邱ƂmF܂B
   * 擾\ȏꍇAvZs\Ɣf true Ԃ܂B
   * 擾s\ȏꍇAvZssƔf false Ԃ܂B<p>
   *
   * f[^擾s\łƕԎĂM[qɑ΂ČvZvo܂B
   * ̓Iɂ́AM[q calReq() bZ[W𑗂܂B
   * <pre>
   * [߂l]
   *  Etrue --> vZs\̏ꍇB
   *  Efalse -> vZss\̏ꍇA܂͌vZς݂̏ꍇB
   * </pre>
   * </japanese>
   * <p>
   *
   * Judgment whether calculation can be started or not.
   * In classes derived from class Element, its definition should be given. <p>
   *
   * First, when "_currentTime" is more than "_terminalTime",
   * it is judged that calculation has finished, and "false" is returned. <p>
   *
   * Next, it checks that all possession receiving ports can acquire the data pack
   * between "getNecessaryTimeFrom" and "getNecessaryTimeTo".
   * It is judged that calculation execution is possible the case of being acquirable,
   * "true" is returned.
   * It is judged that calculation execution is impossible the case of being unacquirable,
   * "false" is returned. <p>
   *
   * A calculation request is advanced to the receiving port
   * which will have replied that data is unacquirable.
   * In particular, a calculation request message is sent to a receiving port.
   *  
   * <pre>
   * [RETURN]
   *  Etrue --> When calculation execution is possible.
   *  Efalse -> When calculation execution is impossible.
   *              When calculation has finished.
   * </pre>
   *
   * @return a <code>boolean</code> value
   * @exception OhCannotGetDataError if an error occurs
   * @exception OhError if an error occurs
   */
  public abstract boolean canYouCalculate() throws OhCannotGetDataError, OhError;

  /**
   * <japanese>
   * PXebv̌vZ葱`܂B<p>
   *
   * e[UƎ̗vff쐬ہAPXebv̌vZ葱`ĉB
   *
   * ʓIȗvfł́A
   *
   * <ul>
   *  <li> ŏAeԗʂɂ _currentTime ɂli[ĂB
   *  <li> ł _currentTime + _timeStep ɂԗ, o͒lvZB
   *  <li> ̃\bhIAԗʂ _currentTime + _timeStep ɂlɍXVĂB
   * </ul>
   *
   * ̂R_𖞑悤ɁAL (1)  (4) ɋLqĉB
   *
   * <pre>
   *  (1) M[q̓́B
   *  (2) vZB
   *  (3) M[qւ̏óB
   *  (4) ԗʂ̍XVB
   * </pre>
   *
   * (2) ł́Ap[^A_currentTime ɂԗʁAM[q̓͂pA
   * _currentTime + _timeStep ɂԗʁAMׂo͒lvZ鐔f̂PXebv
   * ̓Iɒ`ĉB<p>
   *
   * ̃\bh̒ _currentTime, _timeStep ύXȂŉB
   * KvȂ _timeStep Xɍׂԋ敪ɕČvZł܂A
   * ͍̎ƗpϐpA_currentTime, _timeStep ύXȂ悤ɒӂĉB<p>
   *
   * vZɕKvȑSĂ̐ĺAf[^oyяLM[q̓͂ƂĎ擾ł܂B
   * ]Ĉ͕svłB<p>
   *
   * \ʂvZłꍇ́Atrue Ԃ܂B
   * calculate() ̓ŌvZԃXebvvZA
   * ̌vZԃXebṽf[^擾łȂȂǂāA
   * vZi߂邱ƂłȂꍇ́Afalse Ԃ܂B<p>
   *
   * ۃ\bhȂ̂ŁAhNXŕK`ĉB
   *
   * <pre>
   * [ACCIDENT]
   *  Ẽ\bh̒ _currentTime, _timeStep ύXꂽꍇB
   * </pre>
   * </japanese>
   * <p>
   *
   * Calculation procedure for 1 step.
   * In classes derived from class Element, its definition should be given.
   *
   * <ul>
   *  <li>At first, in each amount of states, the value at "_currentTime" is stored.
   *  <li>Here, the amount of states and output value in "_currentTime + _timeStep" are calculated.
   *  <li>Next, the amount of states is updated by the value in a certain time.
   * </ul>
   *
   * In a general element,
   * Please describe following (1) to (4) in order to fulfill these 3 points.
   *
   * <pre>
   *  (1) the input from a receiving port.
   *  (2) numerical computation.
   *  (3) the output to a sending port.
   *  (4) updating of the amount of states.
   * </pre>
   *
   * At (2), by using parameters, the amount of states in "_currentTime" , and the input from a receiving ports,
   * please define concretely one step of a numerical model
   * which calculate the amount of states and the output value
   * which should be transmitted in "_currentTime + _timeStep". <p>
   *
   * In this method, please do not change "_currentTime" and "_timeStep". <p>
   *
   * All numerical values required for calculation are acquirable
   * as instance variable and input from a possession receiving port.
   * Therefore, arguments are unnecessary. <p>
   *
   * The case where it is calculable as planned,
   * "true" is returned.
   * When a calculation time step is calculated inside "calculate()",
   * the data for the calculation time step cannot be acquired and calculation is not able to be advanced,
   * "false" is returned.
   *
   * @return a <code>boolean</code> value
   * @exception OhCannotGetDataError if an error occurs
   * @exception OhError if an error occurs
   */
  public abstract boolean calculate() throws OhCannotGetDataError, OhError;

  /**
   * <japanese>
   * _currentime 
   * <a href="OhTotalSystem.html#_terminalTime">OhTotalSystem._terminalTime</a>
   * Ԃ̌vZASY`܂B<p>
   *
   * calculate(), calculateTimeStep() ȂǂpāA_terminalTime ܂ł̌vZs܂B<p>
   * 
   * ܂A
   * <a href="OhElement.html#canYouCalculate()">OhElement.canYouCalculate()</a>
   * ̖߂l false Ȃ΁AvZJnsƔfAfalse Ԃ܂B<p>
   * 
   * ߂l true ̏ꍇ́A
   * true łA
   * <a href="OhElement.html#calculate()">OhElement.calculate()</a>,
   * _currentTime.add(_timeStep) JԂs܂B<p>
   * 
   * ŌɁA
   * <a href="OhElement.html#setDataPackArray()">OhElement.setDataPackArray()</a>,
   * sAtrue Ԃ܂B
   *
   * <pre> 
   * [߂l]
   *  Etrue --> vZsꍇB
   *  Efalse -> vZs\̏ꍇA܂͌vZς݂̏ꍇB
   * </pre>
   * </japanese>
   * <p>
   *
   * Calculate between "_currentTime" and "_terminalTime".
   * In classes derived from class Element, its definition should be given. <p>
   *
   * Calculation by "_terminalTime" is performed using "calculate()", "calculateTimeStep()", etc. <p>
   *
   * First, if return value of "canYouCalculate()" becomes "false",
   * it will be judged that a calculation start is impossible,
   * "false" is returned. <p>
   *
   * When the return value is "true",
   * "calculate()" and "_currentTime += _timeStep" are repeated and performed,
   * as long as the return value of "canYouCalculate" is "true".  <p>
   *
   * Finally, "setDataPackArray()" and "reportToSender()" are performed and "true" is returned.
   *
   * <pre>
   * [RETURN]
   *  Etrue --> When calculation was performed.
   *  Efalse -> When calculation is impossible.
   *              When calculation has finished.
   * </pre>
   *
   * @return a <code>boolean</code> value
   * @exception OhCannotGetDataError if an error occurs
   * @exception OhError if an error occurs
   */
  public abstract boolean work() throws OhCannotGetDataError, OhError;

  // 6. information output function

  // 7. the function to save the last state at a file

  /**
   * <japanese>
   * vf̍ŏIԂo̓Xg[ os Ŏw肳t@CɏލƂ郁\bhłB
   * [U`vffł́Ã\bhK`Ȃ΂Ȃ܂B<p>
   *
   * ŏIԂގdl͎̂悤ɂĂB
   * <pre>
   * (data_string)
   * </pre>
   * </japanese>
   * <p>
   * This is the method which does the work which writes the last state of an element
   * in the file specified by the output stream "aBr". <p>
   *
   * In classes derived from class Element, its definition should be given.
   *
   * @param aPr a <code>PrintWriter</code> value
   */
  public abstract void saveTerminalState(PrintWriter aPr);
  
  // @end {abstract methods} ---------------------------------------------


  // @begin {initialization} ---------------------------------------------

  /**
   * <japanese>
   * \vf̃NXt@Cǂ݂ƂĂ肠镔nfŗp
   * RXgN^̕⏕\bhłB
   * ̃RXgN^ō쐬ƂɌĂ΂邱Ƃz肵Ă܂B
   * Ⴆ΁AclassName, objectName, objectNumber t@CǂݎĂƂA
   * ΏۂƂt@C̃Xg[ br ƂƂA
   *
   * <pre>
   *  (OhElement)elm = (OhElement)Class.forName("ohymosj." + className).newInstance();
   *  elm.init(objectName, objectNum, br);
   * </pre>
   *
   * ̂悤ɂč\vf蓖Ă܂B<p>
   *
   * \vf̃NXt@Cǂ݂ƂĂ肠镔nf
   * p悤ɂ邽߂ɂ́A
   * ̃\bh`ĂKv܂B<p>
   *
   * init(aObjName, aObjNum, aBr) `̍\vfĂ΂ꂽƂɂ́A
   * O
   * <a href="OhError.html">OhError</a>
   * ܂B
   * </japanese>
   * <p>
   * Initialization with the information which is given through a file.
   *
   * @param aObjName a <code>String</code> value
   * @param aObjNum an <code>int</code> value
   * @param aBr an <code>OhBufferedReader</code> value
   * @return a <code>boolean</code> value
   * @exception OhError if an error occurs
   */
  public boolean init(String aObjName, int aObjNum, OhBufferedReader aBr) throws OhError {
    if (true)
      throw new OhError("This init() method is not defined.");

    return true;
  }


  /**
   * <japanese>
   * IuWFNgAIuWFNgԍAڒʐMvf̔zAf[^܂B
   * ڒʐMvf̔zɊւẮA
   * aNumOfObjs ̗vfzɊmۂ܂B
   * </japanese>
   *
   * @param aObjName a <code>String</code> value
   * @param aObjNum an <code>int</code> value
   * @param aNumOfObjs an <code>int</code> value
   * @param aDataStr a <code>String</code> value
   * @return a <code>boolean</code> value
   * @exception OhError if an error occurs
   */
  public boolean init(String aObjName, int aObjNum, int aNumOfObjs, String aDataStr) throws OhError {
    return ((super.init(aObjName, aObjNum) == true) && (canYouSetObjectArray(aNumOfObjs) == true)
      && (canYouGetDataString(aDataStr) == true)) ? true : false;
  }

  /**
   * <japanese>
   * IuWFNgAIuWFNgԍAڒʐMvf̔z܂B
   * ڒʐMvf̔zɊւẮA
   * aNumOfObjs ̗vfzɊmۂ܂B
   * </japanese>
   *
   * @param aObjName a <code>String</code> value
   * @param aObjNum an <code>int</code> value
   * @param aNumOfObjs an <code>int</code> value
   * @return a <code>boolean</code> value
   * @exception OhError if an error occurs
   */
  public boolean init(String aObjName, int aObjNum, int aNumOfObjs                 ) throws OhError {
    return ((super.init(aObjName, aObjNum) == true) && (canYouSetObjectArray(aNumOfObjs) == true)
      && (canYouGetDataString(null) == true)) ? true : false;
  }

  /**
   * <japanese>
   * IuWFNgAIuWFNgԍAf[^܂B
   * </japanese>
   *
   * @param aObjName a <code>String</code> value
   * @param aObjNum an <code>int</code> value
   * @param aDataStr a <code>String</code> value
   * @return a <code>boolean</code> value
   * @exception OhError if an error occurs
   */
  public boolean init(String aObjName, int aObjNum,                 String aDataStr) throws OhError {
    return ((super.init(aObjName, aObjNum) == true) && (canYouSetObjectArray(0) == true)
      && (canYouGetDataString(aDataStr) == true)) ? true : false;
  }

  /**
   * <japanese>
   * IuWFNgAIuWFNgԍ܂B
   * </japanese>
   *
   * @param aObjName a <code>String</code> value
   * @param aObjNum an <code>int</code> value
   * @return a <code>boolean</code> value
   * @exception OhError if an error occurs
   */
  public boolean init(String aObjName, int aObjNum                                 ) throws OhError {
    return ((super.init(aObjName, aObjNum) == true) && (canYouSetObjectArray(0) == true)
      && (canYouGetDataString(null) == true)) ? true : false;
  }

  // @end {initialization} -----------------------------------------------
  
  
  // @begin {constructors} -----------------------------------------------
  
  /**
   * <japanese>
   * ftHgRXgN^łB
   *
   * ftHgRXgN^
   *
   * <p>
   * <dl>
   *  <li> _className         : "OhElement"
   *  <li> _objectName        : null
   *  <li> _objectNumber      : -1
   * </dl>
   * <p>
   *
   * ɐݒ肵܂B
   * </jananese>
   *
   */
  public OhElement() {
    super();
    _initialWorkDoneP     = false;
    _shareInfoDoneP       = false;
    _objectList           = null;
    _receivePortArray     = null;
    _numberOfReceivePorts = -1;
    _sendPortArray        = null;
    _numberOfSendPorts    = -1;
    _objectArray          = null;
    _numberOfObjects      = -1;
    _dataString           = null;
    _currentTime          = new OhTimePoint();
    _timeStep             = -1L;
    _fixedTime            = new OhTimePoint();
    _targetTime           = new OhTimePoint();
    _calReqP              = false;

    _className            = "OhElement";
  }


  /**
   * <japanese>
   * ̃RXgN^̓oϐȉ̂悤ɐݒ肵܂B<p>
   *
   * <dl>
   *  <li> _className         : "OhElement"
   *  <li> _objectName        : aObjName
   *  <li> _objectNumber      : aObjNum
   *  <li> _objectArray       : Ɋm(̔z̗vf aNumOfObjs )
   *  <li> _dataString        : aStr
   * </dl>
   * </jananese>
   *
   * @param aObjName a <code>String</code> value
   * @param aObjNum an <code>int</code> value
   * @param aNumOfObjs an <code>int</code> value
   * @param aStr a <code>String</code> value
   * @exception OhError if an error occurs
   */
  public OhElement(String aObjName, int aObjNum, int aNumOfObjs, String aStr) throws OhError {
    super();
    _initialWorkDoneP     = false;
    _shareInfoDoneP       = false;
    _objectList           = null;
    _receivePortArray     = null;
    _numberOfReceivePorts = -1;
    _sendPortArray        = null;
    _numberOfSendPorts    = -1;
    _objectArray          = null;
    _numberOfObjects      = -1;
    _dataString           = null;
    _currentTime          = new OhTimePoint();
    _timeStep             = -1L;
    _fixedTime            = new OhTimePoint();
    _targetTime           = new OhTimePoint();
    _calReqP              = false;

    _className            = "OhElement";

    init(aObjName, aObjNum, aNumOfObjs, aStr);
  }


  /**
   * <japanese>
   * ̃RXgN^̓oϐȉ̂悤ɐݒ肵܂B<p>
   *
   * <dl>
   *  <li> _className         : "OhElement"
   *  <li> _objectName        : aObjName
   *  <li> _objectNumber      : aObjNum
   *  <li> _dataString        : aStr
   * </dl>
   * </jananese>
   *
   * @param aObjName a <code>String</code> value
   * @param aObjNum an <code>int</code> value
   * @param aStr a <code>String</code> value
   * @exception OhError if an error occurs
   */
  public OhElement(String aObjName, int aObjNum, String aStr) throws OhError {
    super();
    _initialWorkDoneP     = false;
    _shareInfoDoneP       = false;
    _objectList           = null;
    _receivePortArray     = null;
    _numberOfReceivePorts = -1;
    _sendPortArray        = null;
    _numberOfSendPorts    = -1;
    _objectArray          = null;
    _numberOfObjects      = -1;
    _dataString           = null;
    _currentTime          = new OhTimePoint();
    _timeStep             = -1L;
    _fixedTime            = new OhTimePoint();
    _targetTime           = new OhTimePoint();
    _calReqP              = false;

    _className            = "OhElement";

    init(aObjName, aObjNum, aStr);
  }

  
  // @end {constructors} -------------------------------------------------

  
  // @begin {methods} ----------------------------------------------------

  // 1. creation and reference of "_receivePortArray", "_sendPortArray" and "_objectArray"

  /**
   * <japanese>
   * _objectList  Object ^IuWFNg擾܂B<p>
   *
   * ̃\bh́ANX, IuWFNg, IuWFNgԍw肵āA
   * _objectList  Object ^IuWFNg擾܂B<p>
   *
   * w肳ꂽIuWFNgo^ĂȂꍇ null Ԃ܂B<p>
   *
   * _objectList  null ̏ꍇ́AG[bZ[W\Anull Ԃ܂B
   *
   * <pre>
   * [ARGUMENT]
   *  EaClassName - 擾IuWFNg̃NX
   *  EaObjName --- 擾IuWFNg̃IuWFNg
   *  EaObjNum ---- 擾IuWFNg̃IuWFNgԍ
   * [RETURN]
   *  EObject ----- Object ^ɃLXgꂽwIuWFNg
   *  Enull ------- wIuWFNg𔭌łȂꍇ
   *                 _objectList ݂Ȃꍇ
   * [ACCIDENT]
   *  EObject ^ȊO̕ϐ object_list Ɏ^Ăꍇ
   * </pre>
   * </japanese>
   * <p>
   * Finds an object with class name 'aClassName', object name 'aObjName', and
   * object number 'aObjNum' from "_objectList"
   * and returns the object.
   * When such an object could not be found from "_objectList",
   * "null" will be returned.
   *
   * @param aClassName a <code>String</code> value
   * @param aObjName a <code>String</code> value
   * @param aObjNum an <code>int</code> value
   * @return an <code>Object</code> value
   * @exception OhError if an error occurs
   */
  public Object getFromList(String aClassName, String aObjName, int aObjNum) throws OhError {    //; (*1)
    if (_objectList == null) {                 //; The list was already arranged.
      StringWriter buffer = new StringWriter();
      PrintWriter  out    = new PrintWriter(buffer);
      out.print("\tAsked > [" + aClassName + "] : '" + aObjName + "' : " + aObjNum + " :" + NEWLINE);
      oPrint(out, "\t");
      String str = buffer.toString();
      out.close();
      throw new OhError(str);
    }


    for (int i = 0; i < _objectList.size(); i++)
      if (((OhObject)_objectList.get(i)).objCompare(aClassName, aObjName, aObjNum) == true)
        return _objectList.get(i);    // The appointed object was discovered.
    
    return null;     // The appointed object was not registered.
  }



  
  /**
   * <japanese>
   * _objectList  Object ^IuWFNg擾܂B
   * </japanese>
   *
   * Finds an object with object name 'aObjName', and object number 'aObjNum' from "_objectList"
   * and returns the object.
   * When such an object could not be found from "_objectList",
   * "null" will be returned.
   *
   * @param aObjName a <code>String</code> value
   * @param aObjNum an <code>int</code> value
   * @return an <code>Object</code> value
   * @exception OhError if an error occurs
   */
  public Object getFromList(String aObjName, int aObjNum) throws OhError {      //; (*2)
    if (_objectList == null) {                 // The list was already arranged.
      StringWriter buffer = new StringWriter();
      PrintWriter  out    = new PrintWriter(buffer);
      out.print("\tAsked > " + aObjName + " : " + aObjNum + " :" + NEWLINE);
      oPrint(out, "\t");
      String str = buffer.toString();
      out.close();
      throw new OhError(str);
    }
    
    for (int i = 0; i < _objectList.size(); i++)
      if (((OhObject)_objectList.get(i)).objCompare(aObjName, aObjNum) == true)
        return _objectList.get(i);    // The appointed object was discovered.

    return null;     // The appointed object was not registered.
  }





  /**
   * <japanese>
   * Object ^IuWFNg̓o^ _owner ̐ݒsȂ܂B<p>
   *
   *  Object ^IuWFNg "aObj"  _objectList ɓo^܂B
   * ɁÃ\bhĂяoIuWFNgithisj "aObj"  _owner ƂȂ悤ɐݒ肳܂B<p>
   *
   * o^ɂȀꍇ̓G[bZ[Wo͌ false Ԃ܂B
   *
   * <ul>
   *  <li> aObj  null ̏ꍇB
   *  <li> object_list  NULL ̏ꍇB
   *  <li> NXAIuWFNgAIuWFNgԍS aObj ƈvIuWFNg
   *       _objectList ɓo^ĂꍇB
   *  <li>aObj  _owner ݒς݂̏ꍇB
   * </ul>
   *
   * ܂Ȁꍇُ͈𔭐܂B
   * 
   * <ul>
   *  <li> aObj  OhObject ^ȊO̕ϐi[ĂꍇB
   *  <li> _objectList  OhObject ^ȊO̕ϐ^ĂꍇB
   * </ul>
   *
   * <pre>
   * [ARGUMENT]
   *  EaObj --- Object ^ɃLXgꂽo^IuWFNgB
   * [RETURN]
   *  Etrue --> ɓo^ꂽꍇB
   *  Efalse -> aObj  null ̏ꍇB
   *             object_list  NULL ̏ꍇBiz񉻍ς݁Hj
   *             aObj ƁA NXA IuWFNgA IuWFNgԍ<b>S</b>vIuWFNg
   *             _objectList ɓo^ĂꍇB
   *             aObj  _owner ݒς݂̏ꍇBiOhObject.setOwner()j
   *
   * [ACCIDENT]
   *  EaObj  OhObject ^ȊO̕ϐi[ĂꍇB
   *  E_objectList  OhObject ^ȊO̕ϐ^ĂꍇB
   * </pre>
   * </japanese>
   * <p>
   * Registers an object 'aObj' to the data member '_objectList'.
   *
   * @param aObj an <code>Object</code> value
   * @return a <code>boolean</code> value
   * @exception OhError if an error occurs
   */
  public boolean register(Object aObj) throws OhError {                //; (*3)
    if (aObj == null) {                        // The argument is inaccurate.
      StringWriter buffer = new StringWriter();
      PrintWriter  out    = new PrintWriter(buffer);
      oPrint(out, "\t");
      String str = buffer.toString();
      out.close();
      throw new IllegalArgumentException(str);
    }

    if (_objectList == null) {                 // The list was already arranged.
      StringWriter buffer = new StringWriter();
      PrintWriter  out    = new PrintWriter(buffer);
      oPrint(out, "\t");
      String str = buffer.toString();
      out.close();
      throw new OhError(str);
    }

    // Comparison of class name, object name, and object number.
    // When variables other than "Object" type are stored in "aObj", abnormalities occur.
    //
    String className = ((OhObject)aObj).getClassName();
    String objName   = ((OhObject)aObj).getObjectName();
    int    objNum    = ((OhObject)aObj).getObjectNumber();

    if (objName == null || objNum == -1) {
      StringWriter buffer = new StringWriter();
      PrintWriter  out    = new PrintWriter(buffer);
      ((OhObject)aObj).oPrint(out, "    registered > ");
      oPrint(out, "\t");
      String str = buffer.toString();
      out.close();
      throw new OhError(str);
    }

    if (getFromList(className, objName, objNum) != null) {
      StringWriter buffer = new StringWriter();
      PrintWriter  out    = new PrintWriter(buffer);
      ((OhObject)aObj).oPrint(out, "    registered > ");
      oPrint(out, "\t");
      String str = buffer.toString();
      out.close();
      throw new OhError(str);
    }

    ((OhObject)aObj).setOwner((Object)this);    //; aObj  _owner  ̃\bhĂяo this ɐݒ肷B

    _objectList.add(aObj);

    return true;
  }



  

  /**
   * <japanese>
   * z񉻂 _objectList ԂA
   * ɁA aNumOfObjs izĵPԖڂɗvf _numberOfObjects i[܂B<p>
   *
   * _objectList  null wĂꍇAG[bZ[W\܂B<p>
   *
   * ܂A aNumOfObjs[0]  0 ȏ̏ꍇ́Aɔz쐬ς݂ƔfA
   * G[bZ[W\܂B
   * z񉻌A_numOfObjs[0]  0 ȏɐݒ肳܂B
   * ]āAzݒOɂ _numOfObjs[0]  0 ɏĂĉB<p>
   *
   * _numberOfReceivePort, _numberOfSendPort ́ARXgN^ -1 ɏ܂B<p>
   *
   * <pre>
   * [in] aNumOfObjs[0] - _objectList ̗vfi_numberOfObjectsj
   *
   * [RETURN] _objectList z񉻂 Object ^z
   * </pre>
   * </japanese>
   * <p>
   * Creates an array of objects which are included in '_objectList'
   * and stores it in '_aObjArray' and the number of elements in 'aNumOfObjs'.
   *
   * @param aNumOfObjs an <code>int[]</code> value
   * @return an <code>Object[]</code> value
   * @exception OhError if an error occurs
   */
  public Object[] makeObjectArray(int[] aNumOfObjs) throws OhError {    //; (*4)
    if (_objectList == null) {
      StringWriter buffer = new StringWriter();
      PrintWriter  out    = new PrintWriter(buffer);
      oPrint(out, "\t");
      String str = buffer.toString();
      out.close();
      throw new OhError(str);
    }

    if (aNumOfObjs[0] >= 0) {
      StringWriter buffer = new StringWriter();
      PrintWriter  out    = new PrintWriter(buffer);
      oPrint(out, "\t");
      String str = buffer.toString();
      out.close();
      throw new OhError(str);
    }

    Object[] aObjArray;
    aObjArray = _objectList.toArray(new Object[_objectList.size()]);
    
    aNumOfObjs[0] = _objectList.size();

    return aObjArray;
  }


  

  /**
   * <japanese>
   * _objectArray 쐬܂B
   *
   * ̓Iɂ́A
   * aNumOfObjs (arg) ̃IuWFNgL邽߂̗̈q[vɊmۂ܂B
   *
   * _objectArray ͒ڒʐM鑊L邽߂Ɏg܂B<p>
   *
   * ܂A
   *
   * <pre>
   *  EaNumOfObjs (arg) ̏ꍇB
   *  E_numberOfObjects  0 ȏ̏ꍇBi쐬ς݁j
   * </pre>
   *
   * ̏ꍇ̓G[bZ[Wo͌ false Ԃ܂B
   *
   * <pre>
   * [ARGUMENT]
   *  EaNumOfObjs -> _objectArray ɋLIuWFNgB
   * [RETURN]
   *  Etrue --> _objectArray mۂłꍇB
   *  Efalse -> aNumOfObjs ̏ꍇA܂ _numberOfObjects  0 ȏ̏ꍇBi쐬ς݁j
   * </pre>
   * </japanese>
   * <p>
   * Acquire the memory to store "aNumOfObjs" objects.
   *
   * @param aNumOfObjs an <code>int</code> value
   * @return a <code>boolean</code> value
   * @exception OhError if an error occurs
   */
  public boolean canYouSetObjectArray(int aNumOfObjs) throws OhError {    //; (*5)
    if (aNumOfObjs < 0) {
      StringWriter buffer = new StringWriter();
      PrintWriter  out    = new PrintWriter(buffer);
      oPrint(out, "    ");
      String str = buffer.toString();
      out.close();
      throw new IllegalArgumentException(str);
    }

    if ((_numberOfObjects >= 0) || (_objectArray != null)) {
      StringWriter buffer = new StringWriter();
      PrintWriter  out    = new PrintWriter(buffer);
      oPrint(out, "    ");
      String str = buffer.toString();
      out.close();
      throw new OhError(str);
    }

    try {
      _objectArray = new Object[aNumOfObjs];
    }
    catch (ArrayIndexOutOfBoundsException e) {
      StringWriter buffer = new StringWriter();
      PrintWriter  out    = new PrintWriter(buffer);
      oPrint(out, "\t");
      String str = buffer.toString();
      out.close();
      throw new ArrayIndexOutOfBoundsException(str);
    }

    for (int i = 0; i < aNumOfObjs; i++)
      _objectArray[i] = null;

    _numberOfObjects = aNumOfObjs;

    return true;
  }



  
  /**
   * <japanese>
   * Object ^IuWFNg̓o^ _owner ̐ݒsȂ܂B
   * </japanese>
   * <p>
   * Registers the object "aObj" with which the current object communicates directly
   * to object_array[num].
   *
   * @param aObj an <code>Object</code> value
   * @param aNum an <code>int</code> value
   * @return a <code>boolean</code> value
   * @exception OhError if an error occurs
   */
  public boolean register(Object aObj, int aNum) throws OhError {    //; (*6)
    if ((aNum < 0) || (aNum >= _numberOfObjects)) {
      StringWriter buffer = new StringWriter();
      PrintWriter  out    = new PrintWriter(buffer);
      out.print("\tnum (arg) = " + aNum + ", _numberOfObjects = " + _numberOfObjects + NEWLINE);
      oPrint(out, "\t");
      String str = buffer.toString();
      out.close();
      throw new IllegalArgumentException(str);
    }

    if (aObj == null) {
      StringWriter buffer = new StringWriter();
      PrintWriter  out    = new PrintWriter(buffer);
      oPrint(out, "\t");
      String str = buffer.toString();
      out.close();
      throw new IllegalArgumentException(str);
    }

    if (_objectArray[aNum] != null) {
      StringWriter buffer = new StringWriter();
      PrintWriter  out    = new PrintWriter(buffer);
      out.print("\tDoubly set _objectArray[" + aNum + "]." + NEWLINE);
      ((OhObject)_objectArray[aNum]).oPrint(out, "    first > ");
      ((OhObject)aObj).oPrint(out, "    next > ");
      String str = buffer.toString();
      out.close();
      throw new OhError(str);
    }

    _objectArray[aNum] = aObj;

    return true;
  }

  
  /**
   * <japanese>
   * Object ^IuWFNgLpz쐬܂B
   * ܂A_receivePortArray, _sendPortArray, _objectArray 쐬܂B<p>
   *
   * ܂A_objectList  null ł邱ƂmF܂B
   * null łȂꍇA_objectList gpƔfăG[bZ[W\Areturn ܂B<p>
   *
   * ɁAȉ̍Ƃs܂B
   *
   * <ol>
   *  <li> OhLinkedList IuWFNgq[vɍ쐬 _objectList ɐݒ肷B
   *  <li> OhElement.registerReceivePort(), OhElenment.makeObjectArray() p
   *       _receivePortArray 쐬B
   *  <li> OhLinkedList.clear() ɂ _objectList B
   *  <li> OhElement.registerSendPort(), OhElement.makeObjectArray() p
   *       _sendPortArray 쐬B
   * </ol>
   *
   * Ō _objectList  null ɐݒ肵܂B<p>
   *
   * </japanese>
   * <p>
   * Makes "_receivePortArray" and "_sendPortArray".
   *
   * @return a <code>boolean</code> value
   * @exception OhError if an error occurs
   */
  public boolean construct() throws OhError {    //; (*7)
    if (_objectList != null) {
      StringWriter buffer = new StringWriter();
      PrintWriter  out    = new PrintWriter(buffer);
      oPrint(out, "    ");
      String str = buffer.toString();
      out.close();
      throw new OhError(str);
    }

    _objectList = new OhLinkedList();

    // --- Makes _receivePortArray. ---

    registerReceivePorts();    // Abstract method call. Throws "OhError".

    int[] numberOfReceivePortsArray = {_numberOfReceivePorts};
    
    _receivePortArray     = makeObjectArray(numberOfReceivePortsArray);    //; (*4)
    _numberOfReceivePorts = numberOfReceivePortsArray[0];

    _objectList.clear();

    // --- Makes _sendPortArray. ---

    registerSendPorts();    // Abstract method call.

    int[] numberOfSendPortsArray = {_numberOfSendPorts};
    
    _sendPortArray     = makeObjectArray(numberOfSendPortsArray);    
    _numberOfSendPorts = numberOfSendPortsArray[0];

    _objectList = null;

    return true;
  }



  /**
   * <japanese>
   * Object ^IuWFNg擾܂B<p>
   *
   * getReceivePort(), getSendPort(), getObject() ŗp⏕\bh̖SB
   * </japanese>
   * <p>
   * Finds an object with class name 'aClassName', object name 'aObjName', and object number 'aObjNum'
   * from 'aObjArray'
   * and returns the object.
   *
   *
   * @param aClassName a <code>String</code> value
   * @param aObjName a <code>String</code> value
   * @param aObjNum an <code>int</code> value
   * @param aObjArray an <code>Object[]</code> value
   * @param aNumOfObjs an <code>int</code> value
   * @return an <code>Object</code> value
   */
  public Object getFromArray(String aClassName, String aObjName, int aObjNum,
                             Object[] aObjArray, int aNumOfObjs) {           //; (*8)
    if (aNumOfObjs < 0) {                     // The list is not arranged.
      StringWriter buffer = new StringWriter();
      PrintWriter  out    = new PrintWriter(buffer);
      oPrint(out, "\t");
      String str = buffer.toString();
      out.close();
      throw new IllegalArgumentException(str);
    }
    
    for (int k = 0; k < aNumOfObjs; k++)
      if (((OhObject)aObjArray[k]).objCompare(aClassName, aObjName, aObjNum) == true)
        return aObjArray[k];    // The appointed object was discovered.
    
    return null;
  }
  

  
  /**
   * <japanese>
   * Object ^IuWFNg擾܂B<p>
   *
   * getReceivePort(), getSendPort(), getObject() ŗp⏕\bh̖SB
   * </japanese>
   * <p>
   * Finds an object with object name 'aObjName', and object number 'aObjNum'
   * from 'aObjArray'
   * and returns the object.
   *
   *
   * @param aObjName a <code>String</code> value
   * @param aObjNum an <code>int</code> value
   * @param aObjArray an <code>Object[]</code> value
   * @param aNumOfObjs an <code>int</code> value
   * @return an <code>Object</code> value
   */
  public Object getFromArray(String aObjName, int aObjNum, Object[] aObjArray, int aNumOfObjs) {  //; (*9)
    if (aNumOfObjs < 0) {                      // The list is not arranged.
      StringWriter buffer = new StringWriter();
      PrintWriter  out    = new PrintWriter(buffer);
      oPrint(out, "\t");
      String str = buffer.toString();
      out.close();
      throw new IllegalArgumentException(str);
    }
    
    for (int k = 0; k < aNumOfObjs; k++)
      if (((OhObject)aObjArray[k]).objCompare(aObjName, aObjNum) == true)
        return aObjArray[k];    // The appointed object was discovered.

    return null;
  }



  
  /**
   * <japanese>
   * _receivePortArray M[q擾܂B<p>
   *
   * NXAIuWFNgAIuWFNgԍw肷邱ƂɂA
   * _receivePortArray M[q擾܂B
   * w肳ꂽIuWFNgo^ĂȂꍇ null Ԃ܂B
   * </japanese>
   * <p>
   * By appointing class name, object name, and object number,
   * acquires a receiving port from "_receivePortArray". <p>
   *
   * When the appointed object is not registered,
   * "null" is returned.
   *
   *
   * @param aClassName a <code>String</code> value
   * @param aObjName a <code>String</code> value
   * @param aObjNum an <code>int</code> value
   * @return an <code>Object</code> value
   * @exception OhError if an error occurs
   */
  public Object getReceivePort(String aClassName, String aObjName, int aObjNum) throws OhError {     //; (*10-1)
    Object p = getFromArray(aClassName, aObjName, aObjNum, _receivePortArray, _numberOfReceivePorts);

    if (p == null) {                           // The appointed object was not registered.
      StringWriter buffer = new StringWriter();
      PrintWriter  out    = new PrintWriter(buffer);
      out.print("\tAsked > \'" + aObjName + "\' : " + aObjNum + " :" + NEWLINE);
      oPrint(out, "\t");
      String str = buffer.toString();
      out.close();
      throw new OhError(str);
    }
    return p;
  }

  
  /**
   * <japanese>
   * _receivePortArray M[q擾܂B<p>
   *
   * NXAIuWFNgw肷邱ƂɂA
   * _receivePortArray M[q擾܂B
   * w肳ꂽIuWFNgo^ĂȂꍇ null Ԃ܂B
   * </japanese>
   * <p>
   * By appointing class name and object name,
   * acquires a receiving port from "_receivePortArray".
   * At this time, the object number is set to 0. <p>
   *
   * When the appointed object is not registered,
   * "null" is returned.
   *
   * @param aClassName a <code>String</code> value
   * @param aObjName a <code>String</code> value
   * @return an <code>Object</code> value
   * @exception OhError if an error occurs
   */
  public Object getReceivePort(String aClassName, String aObjName) throws OhError {           //; (*10-2)
    Object p = getFromArray(aClassName, aObjName, 0, _receivePortArray, _numberOfReceivePorts);

    if (p == null) {                           // The appointed object was not registered.
      StringWriter buffer = new StringWriter();
      PrintWriter  out    = new PrintWriter(buffer);
      out.print("\tAsked > \'" + aObjName + "\' : 0 :" + NEWLINE);
      oPrint(out, "\t");
      String str = buffer.toString();
      out.close();
      throw new OhError(str);
    }
    return p;
  }

  

  /**
   * <japanese>
   * _sendPortArray 瑗M[q擾܂B<p>
   *
   * NXAIuWFNgAIuWFNgԍw肷邱ƂɂA
   * _sendPortArray 瑗M[q擾܂B
   * w肳ꂽIuWFNgo^ĂȂꍇ null Ԃ܂B
   * </japanese>
   * <p>
   * By appointing class name, object name, and object number,
   * acquires a sending port from "_sendPortArray". <p>
   *
   * When the appointed object is not registered,
   * "null" is returned.
   *
   *
   * @param aClassName a <code>String</code> value
   * @param aObjName a <code>String</code> value
   * @param aObjNum an <code>int</code> value
   * @return an <code>Object</code> value
   * @exception OhError if an error occurs
   */
  public Object getSendPort(String aClassName, String aObjName, int aObjNum) throws OhError {  //; (*11-1)
    Object p = getFromArray(aClassName, aObjName, aObjNum, _sendPortArray, _numberOfSendPorts);

    if (p == null) {                           // The appointed object was not registered.
      StringWriter buffer = new StringWriter();
      PrintWriter  out    = new PrintWriter(buffer);
      out.print("\tAsked > \'" + aObjName + "\' : " + aObjNum + " :" + NEWLINE);
      oPrint(out, "\t");
      String str = buffer.toString();
      out.close();
      throw new OhError(str);
    }
    return p;
  }


  /**
   * <japanese>
   * _sendPortArray 瑗M[q擾܂B<p>
   *
   * NXAIuWFNgw肷邱ƂɂA
   * _sendPortArray 瑗M[q擾܂B
   * w肳ꂽIuWFNgo^ĂȂꍇ null Ԃ܂B
   * </japanese>
   * <p>
   * By appointing class name and object name, acquires a sending port from "_sendPortArray".
   * At this time, the object number set to 0. <p>
   *
   * When the appointed object is not registered, "null" is returned.
   *
   *
   * @param aClassName a <code>String</code> value
   * @param aObjName a <code>String</code> value
   * @return an <code>Object</code> value
   * @exception OhError if an error occurs
   */
  public Object getSendPort(String aClassName, String aObjName) throws OhError {        //; (*11-2)
    Object p = getFromArray(aClassName, aObjName, 0, _sendPortArray, _numberOfSendPorts);

    if (p == null) {  //; The appointed object was not registered.
      StringWriter buffer = new StringWriter();
      PrintWriter  out    = new PrintWriter(buffer);
      out.print("\tAsked > \'" + aObjName + "\' : 0 :" + NEWLINE);
      oPrint(out, "\t");
      String str = buffer.toString();
      out.close();
      throw new OhError(str);
    }
    return p;
  }



  /**
   * <japanese>
   * _receivePortArray M[q擾܂B<p>
   *
   * IuWFNgAIuWFNgԍw肷邱ƂɂA
   * _receivePortArray M[q擾܂B
   * w肳ꂽIuWFNgo^ĂȂꍇ null Ԃ܂B
   * </japanese>
   * <p>
   * By appointing object name and object number,
   * acquires a receiving port from "_receivePortArray". <p>
   *
   * When the appointed object is not registered,
   * "null" is returned.
   *
   *
   * @param aObjName a <code>String</code> value
   * @param aObjNum an <code>int</code> value
   * @return an <code>Object</code> value
   * @exception OhError if an error occurs
   */
  public Object getReceivePort(String aObjName, int aObjNum) throws OhError {           //; (*12-1)
    Object p = getFromArray(aObjName, aObjNum, _receivePortArray, _numberOfReceivePorts);

    if (p == null) {                           // The appointed object was not registered.
      StringWriter buffer = new StringWriter();
      PrintWriter  out    = new PrintWriter(buffer);
      out.print("\tAsked > \'" + aObjName + "\' : " + aObjNum + " :" + NEWLINE);
      oPrint(out, "\t");
      String str = buffer.toString();
      out.close();
      throw new OhError(str);
    }
    return p;
  }


  /**
   * <japanese>
   * _receivePortArray M[q擾܂B<p>
   *
   * IuWFNgw肷邱ƂɂA
   * _receivePortArray M[q擾܂B
   * w肳ꂽIuWFNgo^ĂȂꍇ null Ԃ܂B
   * </japanese>
   * <p>
   * By appointing object name,
   * acquires a receiving port from "_receivePortArray".
   * At this time, the object number is set to 0. <p>
   *
   * When the appointed object is not registered,
   * "null" is returned.
   *
   *
   * @param aObjName a <code>String</code> value
   * @return an <code>Object</code> value
   * @exception OhError if an error occurs
   */
  public Object getReceivePort(String aObjName) throws OhError {                  //; (*12-2)
    Object p = getFromArray(aObjName, 0, _receivePortArray, _numberOfReceivePorts);

    if (p == null) {                           // The appointed object was not registered.
      StringWriter buffer = new StringWriter();
      PrintWriter  out    = new PrintWriter(buffer);
      out.print("\tAsked > \'" + aObjName + "\' : 0 :" + NEWLINE);
      oPrint(out, "\t");
      String str = buffer.toString();
      out.close();
      throw new OhError(str);
    }
    return p;
  }


  /**
   * <japanese>
   * _sendPortArray 瑗M[q擾܂B<p>
   *
   * IuWFNgAIuWFNgԍw肷邱ƂɂA
   * _sendPortArray 瑗M[q擾܂B
   * w肳ꂽIuWFNgo^ĂȂꍇ null Ԃ܂B
   * </japanese>
   * <p>
   * By appointing object name and object number,
   * acquires a sending port from "_sendPortArray". <p>
   *
   * When the appointed object is not registered,
   * "null" is returned.
   *
   *
   * @param aObjName a <code>String</code> value
   * @param aObjNum an <code>int</code> value
   * @return an <code>Object</code> value
   * @exception OhError if an error occurs
   */
  public Object getSendPort(String aObjName, int aObjNum) throws OhError {        //; (*13-1)
    Object p = getFromArray(aObjName, aObjNum, _sendPortArray, _numberOfSendPorts);

    if (p == null) {                           // The appointed object was not registered.
      StringWriter buffer = new StringWriter();
      PrintWriter  out    = new PrintWriter(buffer);
      out.print("\tAsked > \'" + aObjName + "\' : " + aObjNum + " :" + NEWLINE);
      oPrint(out, "\t");
      String str = buffer.toString();
      out.close();
      throw new OhError(str);
    }
    return p;
  }


  /**
   * <japanese>
   * _sendPortArray 瑗M[q擾܂B<p>
   *
   * IuWFNgw肷邱ƂɂA
   * _sendPortArray 瑗M[q擾܂B
   * w肳ꂽIuWFNgo^ĂȂꍇ null Ԃ܂B
   * </japanese>
   * <p>
   * By appointing object name,
   * acquires a sending port from "_sendPortArray".
   * At this time, the object number is set to 0. <p>
   *
   * When the appointed object is not registered,
   * "null" is returned.
   *
   *
   * @param aObjName a <code>String</code> value
   * @return an <code>Object</code> value
   * @exception OhError if an error occurs
   */
  public Object getSendPort(String aObjName) throws OhError {               //; (*13-2)
    Object p = getFromArray(aObjName, 0, _sendPortArray, _numberOfSendPorts);

    if (p == null) {                           // The appointed object was not registered.
      StringWriter buffer = new StringWriter();
      PrintWriter  out    = new PrintWriter(buffer);
      out.print("\tAsked > \'" + aObjName + "\' : 0 :" + NEWLINE);
      oPrint(out, "\t");
      String str = buffer.toString();
      out.close();
      throw new OhError(str);
    }
    return p;
  }



  /**
   * <japanese>
   * IuWFNg擾܂B<p>
   *
   * w肳ꂽIuWFNgo^ĂȂꍇ null Ԃ܂B
   * </japanese>
   * <p>
   * acquires a object. <p>
   *
   * When the appointed object is not registered,
   * "null" is returned.
   *
   *
   * @param aClassName a <code>String</code> value
   * @param aObjName a <code>String</code> value
   * @param aObjNum an <code>int</code> value
   * @return an <code>Object</code> value
   * @exception OhError if an error occurs
   */
  public Object getObject(String aClassName, String aObjName, int aObjNum) throws OhError {  //; (*14-1)
    Object obj = getFromArray(aClassName, aObjName, aObjNum, _objectArray, _numberOfObjects);

    if (obj == null) {                         // The appointed object was not registered.
      StringWriter buffer = new StringWriter();
      PrintWriter  out    = new PrintWriter(buffer);
      out.print("\tAsked > \'" + aObjName + "\' : " + aObjNum + " :" + NEWLINE);
      oPrint(out, "\t");
      String str = buffer.toString();
      out.close();
      throw new OhError(str);
    }
    return obj;
  }


  /**
   * <japanese>
   * IuWFNg擾܂B<p>
   *
   * w肳ꂽIuWFNgo^ĂȂꍇ null Ԃ܂B
   * </japanese>
   * <p>
   * acquires a object.
   * At this time, the object number is set to 0. <p>
   *
   * When the appointed object is not registered,
   * "null" is returned.
   *
   *
   * @param aClassName a <code>String</code> value
   * @param aObjName a <code>String</code> value
   * @return an <code>Object</code> value
   * @exception OhError if an error occurs
   */
  public Object getObject(String aClassName, String aObjName) throws OhError {        //; (*14-2)
    Object obj = getFromArray(aClassName, aObjName, 0, _objectArray, _numberOfObjects);

    if (obj == null) {                         // The appointed object was not registered.
      StringWriter buffer = new StringWriter();
      PrintWriter  out    = new PrintWriter(buffer);
      out.print("\tAsked > \'" + aObjName + "\' : 0 :" + NEWLINE);
      oPrint(out, "\t");
      String str = buffer.toString();
      out.close();
      throw new OhError(str);
    }
    return obj;
  }



  /**
   * <japanese>
   * \IuWFNg(\vf)擾܂B<p>
   *
   * NXAIuWFNgAIuWFNgԍw肵āA_objectArray \vf擾܂B
   * OhElement NX͍ŏPʂłvfȂ̂ŁA\vf͎Ă܂B
   * ł͕KG[sȂ悤ɂĂ܂B
   * hNXł
   * <a href="OhSubSystem.html">OhSubSystem</a>,
   * <a href="OhTotalSystem.html">OhTotalSystem</a>
   * ŃI[o[Ch܂B
   *  OhElement NXpĐVɃNX郆[UȂĂ悤ɁA
   * ۃ\bhɂĂ܂B
   * </japanese>
   * <p>
   * Acquires a composition object.
   *
   *
   * @param aClassName a <code>String</code> value
   * @param aObjName a <code>String</code> value
   * @param aObjNum an <code>int</code> value
   * @return an <code>Object</code> value
   * @exception OhError if an error occurs
   * @exception OhError if an error occurs
   */
  public Object getComponent(String aClassName, String aObjName, int aObjNum) throws OhError {
    if (true) {
      StringWriter buffer = new StringWriter();
      PrintWriter  out    = new PrintWriter(buffer);
      out.print("\taClassName (arg) = " + aClassName +
        "aObjName (arg) = " + aObjName +
        "aObjNum (arg) = " + aObjNum + NEWLINE);
      
      oPrint(out, "\t");
      String str = buffer.toString();
      out.close();  
      throw new OhError(str);
    }
    
  return null;
  }


  /**
   * <japanese>
   * \IuWFNg(\vf)擾܂B<p>
   *
   * NXAIuWFNgw肵āA_objectArray \vf擾܂B
   * IuWFNgԍ 0 ɂȂ܂B
   * OhElement NX͍ŏPʂłvfȂ̂ŁA\vf͎Ă܂B
   * ł͕KG[sȂ悤ɂĂ܂B
   * hNXł
   * <a href="OhSubSystem.html">OhSubSystem</a>,
   * <a href="OhTotalSystem.html">OhTotalSystem</a>
   * ŃI[o[Ch܂B
   *  OhElement NXpĐVɃNX郆[UȂĂ悤ɁA
   * ۃ\bhɂĂ܂B
   * </japanese>
   * <p>
   * Acquires a composition object.
   * At this time, the object number is set to 0.
   *
   *
   * @param aClassName a <code>String</code> value
   * @param aObjName a <code>String</code> value
   * @return an <code>Object</code> value
   * @exception OhError if an error occurs
   */
  public Object getComponent(String aClassName, String aObjName) throws OhError {
    if (true) {
      StringWriter buffer = new StringWriter();
      PrintWriter  out    = new PrintWriter(buffer);
      out.print("\taClassName (arg) = " + aClassName +
        "aObjName (arg) = " + aObjName +
        "aObjNum (arg) = 0" + NEWLINE);
      
      oPrint(out, "\t");
      String str = buffer.toString();
      out.close();  
      throw new OhError(str);
    }
    
    return null;
  }


  /**
   * <japanese>
   * \IuWFNg(\vf)擾܂B<p>
   *
   * IuWFNgAIuWFNgԍw肵āA_objectArray \vf擾܂B
   * OhElement NX͍ŏPʂłvfȂ̂ŁA\vf͎Ă܂B
   * ł͕KG[sȂ悤ɂĂ܂B
   * hNXł
   * <a href="OhSubSystem.html">OhSubSystem</a>,
   * <a href="OhTotalSystem.html">OhTotalSystem</a>
   * ŃI[o[Ch܂B
   *  OhElement NXpĐVɃNX郆[UȂĂ悤ɁA
   * ۃ\bhɂĂ܂B
   * </japanese>
   * <p>
   * Acquires a composition object.
   *
   *
   * @param aObjName a <code>String</code> value
   * @param aObjNum an <code>int</code> value
   * @return an <code>Object</code> value
   * @exception OhError if an error occurs
   */
  public Object getComponent(String aObjName, int aObjNum) throws OhError{
    if (true) {
      StringWriter buffer = new StringWriter();
      PrintWriter  out    = new PrintWriter(buffer);
      out.print("aObjName (arg) = " + aObjName + "aObjNum (arg) = " + aObjNum + NEWLINE);
      oPrint(out, "\t");
      String str = buffer.toString();
      out.close();  
      throw new Error(str);
    }

    return null;
  }


  /**
   * <japanese>
   * \IuWFNg(\vf)擾܂B<p>
   *
   * IuWFNgw肵āA_objectArray \vf擾܂B
   * OhElement NX͍ŏPʂłvfȂ̂ŁA\vf͎Ă܂B
   * ł͕KG[sȂ悤ɂĂ܂B
   * hNXł
   * <a href="OhSubSystem.html">OhSubSystem</a>,
   * <a href="OhTotalSystem.html">OhTotalSystem</a>
   * ŃI[o[Ch܂B
   *  OhElement NXpĐVɃNX郆[UȂĂ悤ɁA
   * ۃ\bhɂĂ܂B
   * </japanese>
   * <p>
   * Acquires a composition object.
   * At this time, the object number is set to 0.
   *
   *
   * @param aObjName a <code>String</code> value
   * @return an <code>Object</code> value
   * @exception OhError if an error occurs
   */
  public Object getComponent(String aObjName) throws OhError{
    if (true) {
      StringWriter buffer = new StringWriter();
      PrintWriter  out    = new PrintWriter(buffer);
      out.print("aObjName (arg) = " + aObjName + "aObjNum (arg) = 0" + NEWLINE);
      oPrint(out, "\t");
      String str = buffer.toString();
      out.close();  
      throw new OhError("", str);
    }

    return null;
  }


  // 2. connection function

  /**
   * <japanese>
   * SĂ̏LM[q
   * <a href="OhSendPort.html#canYouSwitch()">OhSendPort.canYouSwitch()</a>
   * s܂B<p>
   *
   * ̓Iɂ́Aȉs܂B
   *
   * <ol>
   *  <li> MM[qz _dataPackArray 쐬B
   *  <li> MM[q̐ _numberOfReceivers ɐݒ肷B
   *  <li> _dataPackArray ̃mۂB
   *  <li> _numberOfDataPacks ̒l -1  0 ɕύXB
   * <ol>
   *
   * ̃\bh̎s͏LM[qڑANł܂B
   * 
   * <pre>
   * [RETURN]
   *  Etrue --> ɎsłꍇB
   *  Efalse -> OhSendPort.canYouSwitch() 
   *             false ԂM[q݂ꍇi OhSendPort.canYouSwitch() sĂꍇjB
   * </pre>
   * </japanese>
   * <p>
   * <a href="OhSendPort.html#canYouSwitch()">OhSendPort.canYouSwitch()</a>
   * is performed with all possession sending ports.
   *
   *
   * @return a <code>boolean</code> value
   * @exception OhError if an error occurs
   */
  public boolean canYouSwitch() throws OhError {       //; (*15)
    boolean[] flag = new boolean[2];   // For judgment.
    int k = 0;

    flag[0] = true;

    // An unexchangeable thing is in receiving ports or relay ports.
    //
    for (k = 0; k < _numberOfReceivePorts; k++)
      if (((OhPort)_receivePortArray[k]).canYouSwitch() != true)
        flag[0] = false;

    if (flag[0] != true) {
      StringWriter buffer = new StringWriter();
      PrintWriter  out    = new PrintWriter(buffer);
      oPrint(out, "\t");
      String str = buffer.toString();
      out.close();  
      throw new OhError(str);
    }

    flag[1] = true;

    
    // An unexchangeable thing is in receiving ports or relay ports.
    //
    for (k = 0; k < _numberOfSendPorts; k++)
      if (((OhPort)_sendPortArray[k]).canYouSwitch() != true)
        flag[1] = false;

    if (flag[1] != true) {
      StringWriter buffer = new StringWriter();
      PrintWriter  out    = new PrintWriter(buffer);
      oPrint(out, "\t");
      String str = buffer.toString();
      out.close();  
      throw new OhError(str);
    }

    return ((flag[0] == true) && (flag[1] == true)) ? true : false;
  }



  /**
   * <japanese>
   * vZƑO̊mFsȂ܂B<p>
   *
   * M[qAM[qf[^pbN𑗎M\ȏԂł邱ƂmF܂B<p>
   *
   * SM[q OhReceivePort.areYouReady ̖߂lAyёSM[q OhSendPort.areYouReady 
   * ߂l true ł邱ƂmF܂B
   * ߂l false ̒[q݂ꍇ false ԂA݂Ȃꍇ true Ԃ܂B<p>
   *
   * ̃\bh̖߂l true ̏ꍇAOhElement ͌vZsł܂B<p>
   *
   * false ̏ꍇ́Af[^pbNMłȂM[qA
   * ܂̓f[^pbN𑗐MłȂM[q݂A
   *  OhElement, M Element ŌvZsł܂B<p>
   *
   * ̏ꍇAڑƂɌ肪邩AOhSendPort  canYouSwitch() sĂȂ\܂B
   *
   * <pre>
   * [RETURN]
   *  Etrue --> S[q areYouReady ̖߂l true ̏ꍇB
   *  Efalse -> ߂l false ̒[q݂ꍇB
   * </pre>
   * </japanese>
   * <p>
   * Checks before calculation work. <p>
   *
   * Checks to see sending ports and receiving ports are in the state
   * which can send and receive the data packs.
   *
   *
   * @return a <code>boolean</code> value
   * @exception OhError if an error occurs
   */
  public boolean areYouReady() throws OhError {  //; (*16)
    boolean flag = true;    // For judgment.
    int k = 0;

    // --- Check "_receivePort". ---

    boolean bufFlag = true;  //; For judgment.
    for (k = 0; k < _numberOfReceivePorts; k++)
      if (((OhPort)_receivePortArray[k]).areYouReady() != true) {
        ((OhPort)_receivePortArray[k]).oPrint(new PrintWriter(System.err, true), "\t");
        bufFlag = false;
      }

    if (bufFlag != true) {
      StringWriter buffer = new StringWriter();
      PrintWriter  out    = new PrintWriter(buffer);
      oPrint(out, "\t");
      String str = buffer.toString();
      out.close();  
      throw new OhError("Receive ports or relay ports are not connected." + str);
    }
    
    // --- Check "_sendPortArray". ---

    bufFlag = true;    //; For judgment.

    for (k = 0; k < _numberOfSendPorts; k++)
      if (((OhPort)_sendPortArray[k]).areYouReady() != true) {
        ((OhPort)_sendPortArray[k]).oPrint(new PrintWriter(System.err, true), "\t");
        bufFlag = false;
      }

    if (bufFlag != true) {
      StringWriter buffer = new StringWriter();
      PrintWriter  out    = new PrintWriter(buffer);
      oPrint(out, "\t");
      String str = buffer.toString();
      out.close();  
      throw new OhError(str);
    }

    // --- Check "_objectArray". ---

    bufFlag = true;    // For judgment.

    for (k = 0; k < _numberOfObjects; k++)
      if (_objectArray[k]== null) {
        StringWriter buffer = new StringWriter();
        PrintWriter  out    = new PrintWriter(buffer);
        out.print("\t_objectArray[" + k + "] has not set." + NEWLINE);
        String str = buffer.toString();
        out.close();  
        throw new OhError(str);
      }

    if (bufFlag != true) {
      StringWriter buffer = new StringWriter();
      PrintWriter  out    = new PrintWriter(buffer);
      oPrint(out, "\t");
      String str = buffer.toString();
      out.close();  
      throw new OhError(str);
    }

    return flag;
  }
    

  // 3. numerical initialization

  /**
   * <japanese>
   * f[^ _dataString  aDataStr ɐݒ肵܂B<p>
   *
   * NX OhElement  String ^̈ init() ŌĂяo܂B
   * </japanese>
   * <p>
   * Gets the data string.
   *
   * @param aDataStr a <code>String</code> value
   * @return a <code>boolean</code> value
   * @exception OhError if an error occurs
   */
  public boolean canYouGetDataString(String aDataStr) throws OhError {    //; (*19)
    if (_dataString != null) {
      StringWriter buffer = new StringWriter();
      PrintWriter  out    = new PrintWriter(buffer);
      oPrint(out, "\t");
      String str = buffer.toString();
      out.close();  
      throw new OhError(str);
    }

    _dataString = aDataStr;

    return true;
  }

  
  /**
   * <japanese>
   * f[^ _dataString  null ɐݒ肵܂B
   * </japanese>
   * <p>
   * Clear the data string.
   *
   */
  public void clearDataString() {    //; (*18)
    _dataString = null;
  }


  /**
   * <japanese>
   * p^ǂݍރ\bhłB
   * `ς݂ŁA[U͐VKɒ`Kv͂܂B
   * </japanese>
   * <p>
   * Gets the parameter.
   *
   *
   * @param aBr an <code>OhBufferedReader</code> value
   * @exception OhError if an error occurs
   */
  public void getParameter(OhBufferedReader aBr) throws OhError {     //; (*22)

    try {
      if (aBr.skipWhites() == -1) {
        OhError e = new OhError("Parameter file error.");
        throw e;
      }
      
      if ((char)aBr.read() != '{') {
        StringWriter buffer = new StringWriter();
        PrintWriter  out    = new PrintWriter(buffer);
        out.print("Now reading parameters for ");
        oPrint(out, " > ");
        String str = buffer.toString();
        out.close();
        throw new OhError("{ should come first", str);
      }
      
      aBr.readLine();

      setParameter(aBr);    // Abstract method call.

      
      if (aBr.skipWhites() == -1) {
        OhError e = new OhError("Parameter file error.");
        throw e;
      }
      
      if ((char)aBr.read() != '}') {
        StringWriter buffer = new StringWriter();
        PrintWriter  out    = new PrintWriter(buffer);
        out.print("Now reading parameters for ");
        oPrint(out, " > ");
        String str = buffer.toString();
        out.close();
        throw new OhError("Should end with {", str);
      }

      aBr.readLine();

    }
    catch (IOException e) {
      StringWriter buffer = new StringWriter();
      PrintWriter  out    = new PrintWriter(buffer);
      oPrint(out, "\t");
      String str = buffer.toString();
      out.close();
      throw new OhError("Error in parameter line.", str);
    }
  }



  /**
   * <japanese>
   * ԗʂǂݍރ\bhłB
   * </japanese>
   * <p>
   * Gets the initial state.
   *
   *
   * @param aBr an <code>OhBufferedReader</code> value
   * @exception OhError if an error occurs
   */
  public void getInitialState(OhBufferedReader aBr) throws OhError {    //; (*23)
    try {
      if (aBr.skipWhites() == -1) {
        OhError e = new OhError("Initial state file error.");
        throw e;
      }
      
      if ((char)aBr.read() != '{') {
        StringWriter buffer = new StringWriter();
        PrintWriter  out    = new PrintWriter(buffer);
        out.print("Now reading initial state for ");
        oPrint(out, " > ");
        String str = buffer.toString();
        out.close();
        throw new OhError("{ should come first", str);
      }
      
      aBr.readLine();
      
      setInitialState(aBr);    // Abstract method call.
      
      if (aBr.skipWhites() == -1) {
        OhError e = new OhError("Initial state file error.");
        throw e;
      }
      
      if ((char)aBr.read() != '}') {
        StringWriter buffer = new StringWriter();
        PrintWriter  out    = new PrintWriter(buffer);
        out.print("Now reading initial state for ");
        oPrint(out, " > ");
        String str = buffer.toString();
        out.close();
        throw new OhError("Should end with {", str);
      }

      aBr.readLine();

    }
    catch (IOException e) {
      StringWriter buffer = new StringWriter();
      PrintWriter  out    = new PrintWriter(buffer);
      oPrint(out, "\t");
      String str = buffer.toString();
      out.close();
      throw new OhError("Error in initial state line.", str);
    }
  }      
 

  /**
   * <japanese>
   * ̃\bh́AڒʐM𗘗păp^Ȃǂ̏L邽߂̃\bhłB
   * lXȗp@l܂A
   * Ⴆ΂IuWFNg̃p^𑼂̃IuWFNgłpꍇȂǂɗpł܂B
   * ꂼꂪp^ setParameter() Őݒ肷̂ł͂ȂA
   * ݒ肵p^lARs[Ă炤A
   * 邢͂̃p^Ă炤AȂǂƂgl܂B
   * LƂsȂꍇ true ԂĉB
   * ƂsȂȂꍇ false ԂĉB
   * ftHgł͒P true Ԃ悤ɒ`Ă܂B
   * </japanese>
   * <p>
   * This is a method for sharing information, such as a parameter,
   * using direct communication.
   *
   * @return a <code>boolean</code> value
   */
  public boolean canYouShareInfo() {
    return true;
  }


  /**
   * <japanese>
   * ̃\bh canYouShareInfo() Ăяo\bhłB
   * ߂l
   * <a href="OperationFlag.html">OperationFlag</a>
   * ^łB<p>
   *
   * doShareInfo() ́A<p>
   *
   * canYouShareInfo() ܂ȂĂȂꍇA
   *
   * <ol>
   *  <li> canYouShareInfo() ĂсA߂l true Ȃ _shareInfoDoneP  true ɐݒ肵A
   *       <a href="OperationFlag.html#E_DONE">OperationFlag.E_DONE</a> Ԃ܂B
   *  <li> canYouShareInfo() ̖߂l false Ȃ 񋤗LƂłȂƂāA
   *       <a href="OperationFlag.html#E_UNDONE">OperationFlag.E_UNDONE</a> Ԃ܂B
   * </ol>
   *
   * canYouShareInfo() łɂłꍇ
   * <a href="OperationFlag.html#E_ALREADY">OperationFlag.E_ALREADY</a> Ԃ܂B
   * </japanese>
   * <p>
   * This is the method which calls 'canYouShareInfo()'.
   *
   *
   * @return an <code>OperationFlag</code> value
   * @exception OhError if an error occurs
   */
  public OperationFlag doShareInfo() throws OhError{    //; (*24)
    if (_shareInfoDoneP == true)
      return OperationFlag.E_ALREADY;
    if (canYouShareInfo() == true)
      return OperationFlag.E_UNDONE;

    _shareInfoDoneP = true;
    return OperationFlag.E_DONE;
  }


  /**
   * <japanese>
   * vfɐڑĂ邷ׂĂ̑M[q̃f[^Ɏ^Ăf[^pbNA
   * Xg[ aPwijŎw肳Ăt@Cɏo܂B
   * ^̎d́AM[q̓̃\bh
   * <a href="OhSendPort.html#saveSendPortStock()">OhSendPort.saveSendPortStock()</a>
   * gčsȂ܂B<p>
   *
   * <a href="OhSubSystem.html">OhSubSystem</a>
   * ł͍Ē`܂B
   * </japanese>
   * <p>
   * The data pack recorded on the data of all the sending ports connected to the element
   * is written out to the file specified by stream 'aPw'.
   * Work of inclusion is performed using method 'saveSendPortStock' of the same name of a sending port. <p>
   *
   * This method is re-defined by 'OhSubSystem'.
   *
   *
   * @param aPw a <code>PrintWriter</code> value
   */
  public void saveSendPortStock(PrintWriter aPw) {  //; (*25)
    if (_numberOfSendPorts <= 0)
      return;

    aPw.print(_className + " " + _objectName + " " + _objectNumber + " (" + NEWLINE);

    for (int k = 0; k < _numberOfSendPorts; k++) {
      aPw.print("(" + k + ") # sending ports number." + NEWLINE);
      ((OhSendPort)_sendPortArray[k]).saveSendPortStock(aPw);
    }

    aPw.print(")" + NEWLINE);
  }


  /**
   * <japanese>
   * f[^ _dataString ǂ݂ƂAvfɐڑĂ鑗M[qɃf[^^܂B
   * </japanese>  
   * <p>
   * The data character sequence '_dataString' is read
   * and data is recorded on the sending port connected to the element.
   *
   *
   * @exception OhError if an error occurs
   */
  public void restoreSendPortStock() throws OhError {    //; (*26)
    int k;
    int m;
    String dStr = _dataString;

    try {
      StringReader     sr = new StringReader(dStr);
      OhBufferedReader br = new OhBufferedReader(sr);

      for (int i = 0; i < _numberOfSendPorts; i++) {

        k = Integer.parseInt(br.readWord());
      
        if (k != i) {
          StringWriter buffer = new StringWriter();
          PrintWriter  out    = new PrintWriter(buffer);
          oPrint(out, "\t");
          String str = buffer.toString();
          out.close();  
          throw new OhError(str);
        }

        m = Integer.parseInt(br.readWord());

        for (int j = 0; j < m; j++) {
          String dataPackStr = br.readWord() + " " + br.readWord() + " " + br.readWord();
          ((OhSendPort)_sendPortArray[i]).sendByString(dataPackStr);
        }
      }
      
    }
    catch (IOException e) {
      e.printStackTrace();
    }
    catch (NumberFormatException e) {
      e.printStackTrace();
    }
  }


  // 4. time about a simulation

  /**
   * <japanese>
   * ݎ _currentTime Ԃ܂B
   * </japanese>
   * <p>
   * Returns '_currentTime'.
   *
   * @return an <code>OhTimePoint</code> value
   */
  public OhTimePoint getCurrentTime() {
    return _currentTime;
  }

  /**
   * <japanese>
   * ^CXebv _timeStep Ԃ܂B
   * </japanese>
   * <p>
   * Returns '_timeStep'.
   *
   * @return a <code>long</code> value
   */
  public long getTimeStep() {
    return _timeStep;
  }

  /**
   * <japanese>
   * Xebv _currentTime Ԃ܂B
   * </japanese>
   *
   * @return an <code>OhTimePoint</code> value
   */
  public OhTimePoint getNextTime() {
    OhTimePoint nextTime;

    OhTimePoint crnTp = new OhTimePoint();
    crnTp.copy(_currentTime);

    nextTime = crnTp.add(_timeStep);

    return nextTime;
  }

  /**
   * <japanese>
   * vZIԂ܂B
   * </japanese>
   *
   * @return an <code>OhTimePoint</code> value
   */
  public OhTimePoint getTerminalTime() {
    return ((OhElement)_owner).getTerminalTime();
  }

  /**
   * <japanese>
   * ԖڕW_owner 擾܂B
   * </japanese>
   * <p>
   * Returns the medium term target time from '_owner'.
   *
   *
   * @return an <code>OhTimePoint</code> value
   */
  public OhTimePoint getMediumTermTargetTime() {
    return ((OhElement)_owner).getMediumTermTargetTime();
  }

  /**
   * <japanese>
   * vZɂďԂm肵Ă鎞 _fixedTime Ԃ܂B
   * </japanese>
   * <p>
   * Returns '_fixedTime'.
   *
   * @return an <code>OhTimePoint</code> value
   */
  public OhTimePoint getFixedTime() {
    return _fixedTime;
  }

  /**
   * <japanese>
   * vZڕW _targetTime Ԃ܂B
   * </japanese>
   * <p>
   * Returns '_targetTime'.
   *
   * @return an <code>OhTimePoint</code> value
   */
  public OhTimePoint getTargetTime() {
    return _targetTime;
  }

  /**
   * <japanese>
   * _targetTime, _fixedTime  _currentTime Ɉv郁\bhłB
   * vfԔvZsȂvffł́A
   * setInitialState()  _currentTime ݒ肳ꂽ
   * ̃\bhĂԕKv܂B
   * </japanese>
   */
  public void setCur2Fix2Tar() {
    OhTimePoint crnTime = new OhTimePoint();
    crnTime.copy(_currentTime);
    _targetTime = crnTime;

    OhTimePoint crnTp = new OhTimePoint();
    crnTp.copy(_currentTime);
    _fixedTime = crnTp;
  }


  /**
   * <japanese>
   * _currentTime XV܂B
   * </japanese>
   *
   * @return a <code>boolean</code> value
   */
  public boolean canYouRenewCurrentTime() {
    _currentTime.add(_timeStep);
    return true;
  }

  
  /**
   * <japanese>
   * _currentTime  _fixedTime ɖ߂܂B
   * </japanese>
   *
   * @exception OhError if an error occurs
   */
  public void rewindCurrentTime() throws OhError {    //; (*27)
    if (_currentTime.compareTo(_fixedTime) <= 0) {
      StringWriter buffer = new StringWriter();
      PrintWriter  out    = new PrintWriter(buffer);
      oPrint(out, " > ");
      String str = buffer.toString();
      out.close();  
      throw new OhError("'_currentTime' not exceeds '_fixedTime.'", str);
    }

    _currentTime.copy(_fixedTime);
  }

  
  /**
   * <japanese>
   * _fixedTime  _currentTime ɐi߂܂B
   * </japanese>
   * <p>
   * Updates '_fixedTime'.
   *
   * @exception OhError if an error occurs
   */
  public void renewFixedTime() throws OhError {   //; (*28)
    if (_currentTime.compareTo(_fixedTime) <= 0) {
      StringWriter buffer = new StringWriter();
      PrintWriter  out    = new PrintWriter(buffer);
      oPrint(out, " > ");
      String str = buffer.toString();
      out.close();  
      throw new OhError("'_fixedTime' exceeds '_currentTime.'", str);
    }

    OhTimePoint crnTime = new OhTimePoint();
    crnTime.copy(_currentTime);
    _fixedTime = crnTime;
  }

  
  /**
   * <japanese>
   * ̃\bh OhElement ł͎g킸A
   * <a href="OhSubSystem.html">OhSubSystem</a> ōĒ`Ďg܂B
   * Ȃ̃NXŒ`ƂƁAOhElement ł\vfɑ΂ĂAOA
   * ̃\bhĂԕKv邩łB
   * ڍׂ́A
   * <a href="OhSubSystem.html#initializeTargetTime()">OhSubSystem.initializeTargetTime()</a>
   * QƂĉB
   * </japanese>
   * <p>
   * This method is not used by OhElement.
   * It uses re-defining by
   * <a href="OhSubSystem.html">OhSubSystem</a>.
   * The reason for having defined this method is that
   * it is necessary to call this method on principle also to the OhElement type composition element.
   *
   */
  public void initializeTargetTime() throws OhError {}

  
  /**
   * <japanese>
   * vZڕW _targetTime XV܂B
   * ɔvZԊԊuƂ܂B
   * <p>
   * </japanese>
   * Updates '_targetTime'.
   *
   *
   * @param aT a <code>long</code> value. Repeated calculation time interval.
   */
  public void renewTargetTime(long aT) {   //; (*29)
    OhTimePoint tarTime = new OhTimePoint();
    tarTime.copy(_targetTime);
    if ((tarTime.add(aT)).compareTo(getTerminalTime())  > 0) {
      if ((OhOption.getDefault().getTimeControlOption() == TimeControl.DO_NOT_GO_BEYOND_TERMINAL_TIME)
        && (_targetTime.compareTo(getTerminalTime()) < 0)) {
        aT = getTerminalTime().subtract(_targetTime);
      }
    }
    _targetTime.add(aT);
  }

  
  /**
   * <japanese>
   * vZڕWvZ܂B
   * </japanese>
   * <p>
   * Calculates repeated calculation target time.
   *
   *
   * @return a <code>long</code> value
   */
  public long calculateIterationStep() throws OhError {  //; (*30)
    StringWriter buffer = new StringWriter();
    PrintWriter  out    = new PrintWriter(buffer);
    oPrint(out, "\t");
    String str = buffer.toString();
    out.close();  
    throw new OhError("'_fixedTime' exceeds '_currentTime.'", str);
  }

  
  /**
   * <japanese>
   * vf̌vZɕKvȃf[^̂AaRp (arg) 󂯎ׂf[^̃f[^L^Ԃ܂B<p>
   *
   * getNecessaryTimeFrom()  getNecessaryTimeTo() ܂ł̃f[^pbN
   * M[q aRp MĂȂ΁Avf͌vZJn邱Ƃł܂B
   * </japanese>
   * <p>
   * Returns the data record time of the data which 'OhElement' should receive from 'aRp' out of data
   * required for next calculation. <p>
   *
   * 'OhElement' cannot start calculation
   * if the data pack from 'getNecessaryTimeFrom' to 'getNecessaryTimeTo' is not received
   * from the receiving terminal 'aRp',
   * 'OhElement' cannot start calculation.
   *
   *
   * @param aRp an <code>OhReceivePort</code> value
   * @return an <code>OhTimePoint</code> value
   */
  public OhTimePoint getNecessaryTimeFrom(OhReceivePort aRp) {    //; (*31)
    return _currentTime;
  }

  
  /**
   * <japanese>
   * vf̌vZɕKvȃf[^̂AaRp (arg) 󂯎ׂf[^̃f[^L^Ԃ܂B<p>
   *
   * getNecessaryTimeFrom()  getNecessaryTimeTo() ܂ł̃f[^pbN
   * M[q aRp MĂȂ΁Avf͌vZJn邱Ƃł܂B
   * </japanese>
   * <p>
   * Returns the data record time of the data which 'OhElement' should receive from 'aRp' out of data
   * required for next calculation. <p>
   *
   * 'OhElement' cannot start calculation
   * if the data pack from 'getNecessaryTimeFrom' to 'getNecessaryTimeTo' is not received
   * from the receiving terminal 'aRp',
   * 'OhElement' cannot start calculation.
   *
   *
   * @param aRp an <code>OhReceivePort</code> value
   * @return an <code>OhTimePoint</code> value
   */
  public OhTimePoint getNecessaryTimeTo(OhReceivePort aRp) {    //; (*32)
    return getNextTime();
  }


  /**
   * <japanese>
   * _currentTime, _timeStep  aTp (arg), 0 ɏ true Ԃ܂B
   * O _timeStep  0 ȏ̏ꍇA
   *
   * <ul>
   *  <li> _currentTime, _timeStep  aTp, 0 ɐݒς݂̏ꍇA
   *      xbZ[Wo͌Atrue Ԃ܂B
   *  <li> ȊȌꍇAG[bZ[Wo͌Afalse Ԃ܂B
   * </ul>
   * </japanese>
   * <p>
   * '_currentTime' and '_timeStep'of <code>OhElement</code> are initialized to 'aTp' and 0,
   * and returns 'true'.
   *
   *
   * @param aTp an <code>OhTimePoint</code> value
   * @return a <code>boolean</code> value
   * @exception OhError if an error occurs
   */
  public boolean isInitialTime(OhTimePoint aTp) throws OhError {    //; (*33)
    if (_timeStep >= 0) {
      if ((_currentTime.compareTo(aTp)) == 0 && (_timeStep == 0)) {
        StringWriter buffer = new StringWriter();
        PrintWriter  out    = new PrintWriter(buffer);
        oPrint(out, "\t");
        String str = buffer.toString();
        out.close();  
        throw new OhError("'_fixedTime' exceeds '_currentTime.'", str);
      }        

      StringWriter buffer = new StringWriter();
      PrintWriter  out    = new PrintWriter(buffer);
      out.print("\t_currentTime = " + _currentTime + ", _timeStep = " + _timeStep + NEWLINE);
      out.print("\taTp (arg) = " + aTp + NEWLINE);
      oPrint(out, "\t");
      String str = buffer.toString();
      out.close();  
      throw new OhError("'_fixedTime' exceeds '_currentTime.'", str);
    }

    _currentTime = aTp;
    _timeStep = 0L;

    return true;
  }
  

  // 5. execution of calculation

  /**
   * <japanese>
   * ƂsȂ܂B<p>
   *
   * ɂāASf[^擾܂B
   * </japanese>
   *
   * @return a <code>boolean</code> value
   * @exception OhError if an error occurs
   */
  public boolean initialWork() throws OhError {          //; (*34)
    if (initialOutput() == false)    //; Throws "OhError".
      return false;

    setDataPackArray();
    
    return true;
  }


  /**
   * <japanese>
   *  initialWork() Iǂ𔻒f܂B<p>
   *
   * yƂ܂ȂĂȂꍇz
   *
   * <ol>
   *  <li> łׂĂ̎M[q̃f[^擾ł΁A
   *       Ƃ\łƔfāAinitialWork() ĂсA
   *       _initialWorkdoneP  true ɐݒ肵
   *       <a href="OperationFlag.html#E_DONE">OperationFlag.E_DONE</a>
   *       Ԃ܂B
   *  <li> ŏȂƂЂƂ̎M[q̃f[^擾łȂƂ́A
   *       Ƃs\łƔf
   *       <a href="OperationFlag.html#E_UNDONE">OperationFlag.E_UNDONE</a>
   *       Ԃ܂B
   * </ol>
   *
   * yƂłɂłꍇz
   *
   * <a href="OperationFlag.html#E_ALREADY">OperationFlag.E_ALREADY</a>
   * Ԃ܂B
   * </japanese>
   *
   * @return an <code>OperationFlag</code> value
   * @exception OhError if an error occurs
   */
  public OperationFlag doInitialWork() throws OhError {    //; (*35)
    if (_initialWorkDoneP == true)
      return OperationFlag.E_ALREADY;
    if (initialWork() != true)       //; Throws "OhError".
      return OperationFlag.E_UNDONE;

    _initialWorkDoneP = true;
    return OperationFlag.E_DONE;
  }
  

  /**
   * <japanese>
   * canYouCalculate() ̌łB<p>
   *
   * getNecessaryTimeFrom(), getNecessaryTimeTo() Œ`鎞Ԃ̊Ԃ
   * f[^擾łΌvZ\ƔfB
   * IԂ̂RłB
   * getNecessaryTimeFrom(), getNecessaryTimeTo() \bh́A
   * [URɍĒ`ł܂AftHgł
   * getNecessaryTimeFrom()  _currentTime ԂA
   * getNecessaryTimeTo  _currentTime + _timeStep ̒lԂ܂B<p>
   *
   * f[^擾s\łƕԎĂM[qɑ΂ČvZvo܂B
   * ̓Iɂ́AM[q calReq() bZ[W𑗂܂B
   *
   * Ƃ΁A
   *
   * <pre>
   *  public boolean canYouCalculate() {
   *    return canYouCalculate0();
   *  }
   * </pre>
   *
   *ƒ`΂悢łB
   * </japanese>
   *
   * @return a <code>boolean</code> value
   * @exception OhCannotGetDataError if an error occurs
   * @exception OhError if an error occurs
   */
  public boolean canYouCalculate0() throws OhCannotGetDataError, OhError {    //; (*36)
    boolean reply = true;
    int k;

    for (k = 0; k < _numberOfReceivePorts; k++) {    // ------------ for loop start.
      OhTimePoint tFrom;
      OhTimePoint tTo;

      Object rp = _receivePortArray[k];
      
      tFrom = getNecessaryTimeFrom((OhReceivePort)rp);    //;  _currentTime ԂB
      tTo   = getNecessaryTimeTo((OhReceivePort)rp);

      if (((OhPort)rp).canYouGetData(tFrom, tTo) != true) {
        ((OhPort)rp).calReq();
        reply = false;
      }
      
    }    // -------------------------------------------------------- for loop end.

    return reply;
  }


  /**
   * <japanese>
   * canYouCalculate() ̌łB<p>
   *
   * getNecessaryTimeFrom() Œ`f[^擾łΌvZ\Ɣf܂B
   * ȉꍇ getNecessaryTimeTo() \bh`Ӗ͂܂Bj<p>
   *
   * f[^擾s\łƕԎĂM[qɑ΂ČvZvo܂B
   * ̓Iɂ́AM[q calReq() bZ[W𑗂܂B<p>
   * </japanese>
   *
   * @return a <code>boolean</code> value
   * @exception OhCannotGetDataError if an error occurs
   * @exception OhError if an error occurs
   */
  public boolean canYouCalculate1() throws OhCannotGetDataError, OhError {  //; (*37)
    boolean reply = true;
    int k = 0;

    for (k = 0; k < _numberOfReceivePorts; k++) {    // ------------ for loop start.
      OhTimePoint tFrom = new OhTimePoint();

      Object rp = _receivePortArray[k];

      tFrom = getNecessaryTimeFrom((OhReceivePort)rp);


      if (((OhPort)rp).canYouGetData(tFrom, tFrom) != true) {
        ((OhPort)rp).calReq();
        reply = false;
      }

    }    // -------------------------------------------------------- for loop end.

    return reply;
  }



  /**
   * <japanese>
   * f[^pbNzXV܂B<p>
   *
   * SĂ̏LM[q
   * <a href="OhSendPort.html#setDataPackArray()">OhSendPort.setDataPackArray()</a>
   * pA
   * Mf[^pbNzXV܂B<p>
   * 
   * ɂAÂf[^pbNz͏AVf[^pbNz񂪍쐬܂B
   * </japanese>
   * <p>
   * Sends data packs. <p>
   *
   * 'OhSendPort setDataPackArray' is used for all possession sending ports,
   * and sending data pack arrangement is updated.
   * Thereby, old data pack arrangement is eliminated and new data pack arrangement is created. <p>
   *
   * But, nothing is carried out in fact.
   *
   */
  public void setDataPackArray() {               //; (*38)
    for (int k = 0; k < _numberOfSendPorts; k++) {
      ((OhPort)_sendPortArray[k]).setDataPackArray();
    }
  }



  /**
   * <japanese>
   * svf[^pbN܂B<p>
   *
   * SĂ̏LM[q
   * <a href="OhReceivePort.html#reduceDataPackStock()">OhReceivePort.reduceDataPackStock()</a>
   * pA
   * MM[qɌvZI񍐂sȂ܂B<p>
   * 
   * ɂAMM[qŕsvf[^pbNAf[^pbNz񂪍XV܂B
   * </japanese>
   * <p>
   * Deletion of an unnecessary datapacks.
   *
   */
  public void reduceDataPackStock() {    //; (*39)
    int k;

    for (k = 0; k < _numberOfReceivePorts; k++)
      ((OhPort)_receivePortArray[k]).reduceDataPackStock();

    for (k = 0; k < _numberOfSendPorts; k++)
      ((OhPort)_sendPortArray[k]).reduceDataPackStock();
  }
    


  /**
   * Returns true.
   *
   * @return a <code>boolean</code> value
   */
  public boolean printStatus() {
    return true;
  }


  /**
   * Records status.
   *
   * @return a <code>boolean</code> value
   * @exception OhError if an error occurs
   */
  public boolean recordStatus() throws OhError {    //; (*41)
    if (_dataString == null) {
      StringWriter buffer = new StringWriter();
      PrintWriter  out    = new PrintWriter(buffer);
      oPrint(out, "\t");
      String str = buffer.toString();
      out.close();  
      throw new OhError(str);
    }

    printStatus();

    return true;
  }


  /**
   * <japanese>
   * work() ̗łB<p>
   *
   * _timeStep 𓾂ĂAcalculate() sB
   * </japanese>
   * <p>
   * 'Work' which performs 'calculate()' after obtaining '_timeStep'.
   *
   *
   * @return a <code>boolean</code> value
   * @exception OhCannotGetDataError if an error occurs
   * @exception OhError if an error occurs
   */
  public boolean work0() throws OhCannotGetDataError, OhError {             //; (*42)
    //; vZvȂA_currenetTime ԖڕWzĂΌvZȂB
    //;
    if (_calReqP == false && _currentTime.compareTo(getMediumTermTargetTime()) >= 0)
      return false;

    boolean calc = false;

    while(true) {

      // Setup of a time step.
      //
      if ((_timeStep = calculateTimeStep()) <= 0) {    // Abstract method call. Throw "OhError".
        if (_timeStep == 0)
          return false;

        StringWriter buffer = new StringWriter();
        PrintWriter  out    = new PrintWriter(buffer);
        out.print("\t_currentTime = " + _currentTime + ", _timeStep = " + _timeStep + NEWLINE);
        oPrint(out, "\t");
        String str = buffer.toString();
        out.close();  
        throw new IllegalArgumentException(str);
      }

      OhTimePoint crnTime = new OhTimePoint();
      crnTime.copy(_currentTime);
      
      if (crnTime.add(_timeStep).compareTo(getTerminalTime()) > 0) {
        if (OhOption.getDefault().getTimeControlOption() == TimeControl.DO_NOT_GO_BEYOND_TERMINAL_TIME) {
          _timeStep = getTerminalTime().subtract(_currentTime);
        }
      }

      // Check before calculation.
      //
      if (canYouCalculate() != true) {    //; Throw "OhCannotGetDataError".
        if (calc == false)
          return false;
        break;          // Finish without calculating.
      }

      // Calculation of 1 step.
      //
      if (calculate() == false) {
        if (calc == false)
          return false;
        break;          // Finish without calculating.
      }

      calc = true;
      _calReqP = false;

      // Update _currentTime.
      //
      if ((_currentTime.add(_timeStep)).compareTo(getMediumTermTargetTime()) >= 0)
        break;
    }
    
    // Process after calculation.
    //
    setDataPackArray();
    reduceDataPackStock();

    return true;
  }


  /**
   * <japanese>
   * work() ̗łB<p>
   *
   * _timeStep ́Acalculate() ̓ŌvZB
   * </japanese>
   * <p>
   * '_timeStep' is calculated inside 'calculate()'.
   *
   *
   * @return a <code>boolean</code> value
   * @exception OhCannotGetDataError if an error occurs
   * @exception OhError if an error occurs
   */
  public boolean work1() throws OhCannotGetDataError, OhError {              //; (*43)
    //;  vZvȂAcurrenet_time ԖڕWzĂΌvZȂB
    //;
    if (_calReqP == false && _currentTime.compareTo(getMediumTermTargetTime()) >= 0)
      return false;

    boolean calc = false;

    while(true) {
      boolean flgDbg;

      // Check before calculation.
      //
      if ((flgDbg = canYouCalculate()) != true) {    //; Throw "OhCannotGetDataError".
        if (calc == false)
          return false;
        break;          // Finish without calculating.
      }

      // Calculation of 1 step.
      //
      if (calculate() != false) {
        if (calc == false)
          return false;
        break;          // Finish without calculating.
      }

      calc = true;
      _calReqP = false;

      // Update _currentTime.
      //
      if (_timeStep <= 0)
        break;

      _currentTime.add(_timeStep);

      if (_currentTime.compareTo(getMediumTermTargetTime()) >= 0)
        break;
    }

    // Process after calculation.
    //
    setDataPackArray();
    reduceDataPackStock();
    
    return true;
  }
  

  /**
   * <japanese>
   * vZvfp work() ̗łB
   * </japanese>
   * <p>
   * For the element which carries out repeated calculation.
   *
   *
   * @return a <code>boolean</code> value
   * @exception OhCannotGetDataError if an error occurs
   * @exception OhError if an error occurs
   */
  public boolean work2() throws OhCannotGetDataError, OhError {                  //; (*44)
    if (_fixedTime.compareTo(getTerminalTime()) >= 0)     // Finished calculation.
      return false;

    if (_currentTime.compareTo(getTerminalTime()) >= 0)   // Waiting other element.
      return false;

    boolean calc = false;

    
    while(true) {

      // Setup of a time step.
      //
      if ((_timeStep = calculateTimeStep()) <= 0) {    // Abstract methos call.
        if (_timeStep == 0)
          return false;

        StringWriter buffer = new StringWriter();
        PrintWriter  out    = new PrintWriter(buffer);
        out.print("\t_currentTime = " + _currentTime + ", _timeStep = " + _timeStep + NEWLINE);
        oPrint(out, "\t");
        String str = buffer.toString();
        out.close();  
        throw new IllegalArgumentException(str);
      }

      // Make not to exceed '_targetTime'.
      //
      OhTimePoint crnTime = new OhTimePoint();
      crnTime.copy(_currentTime);
      if ((crnTime.add(_timeStep)).compareTo(_targetTime) > 0)
        _timeStep = _targetTime.subtract(_currentTime);

      // Check before calculation.
      //
      if (canYouCalculate() != true) {    //; Throw "OhCannotGetDataError".
        if (calc == false)
          return false;
        break;          // Finish without calculating.
      }

      // Calculation of 1 step.
      //
      if (calculate() != false) {
        if (calc == false)
          return false;
        break;          // Finish without calculating.
      }

      calc = true;

      // Update _currentTime.
      //
      if ((_currentTime.add(_timeStep)).compareTo(_targetTime) >= 0)
        break;
    }      

    // Process after calculation.
    //
    return true;
  }


  /**
   * <japanese>
   * vZ _targetTime ɓBǂ𔻒f܂B
   * BĂ trueABĂȂ false Ԃ܂B
   * </japanese>
   * <p>
   * This method judges whether repeated calculation reached '_targetTime'.
   *
   *
   * @return a <code>boolean</code> value
   */
  public boolean doYouReachTargetTime() {       //; (*46)
    if (_currentTime.compareTo(_targetTime) >= 0)
      return true;
    return false;
  }


  /**
   * <japanese>
   * vfԂ̔vZ̎𔻒肵܂B
   * vZsȂvfŕKĒ`ĉB
   *  trueAĂȂ false ԂĉB
   * Ē`Ȃꍇ̓G[bZ[Wo exit ܂B
   * </japanese>  
   * <p>
   *
   * Convergence of the repeated calculation between elements is judged.
   * Please be sure to re-define by the element which performs repeated calculation.
   * In the method, please return 'true' if it converges.
   * When not giving a re-definition, an error message is sent, and it exits.
   *
   * @return a <code>boolean</code> value
   */
  public boolean doYouConverge() throws OhError {    //; (*47)
    StringWriter buffer = new StringWriter();
    PrintWriter  out    = new PrintWriter(buffer);
    oPrint(out, "\t");
    String str = buffer.toString();
    out.close();  
    throw new OhError(str);
  }


  /**
   * <japanese>
   * ڕW܂ł̃f[^̏o͂ƁAf[^pbN̏郁\bhłB
   * </japanese>
   * <p>
   * It is the method which carries out output of the data by repetitive target time,
   * and processing of a data pack.
   *
   */
  public void flushData() throws OhError {  //; (*48)
    sendStockedData();
    
    setDataPackArray();

    reduceDataPackStock();
  }


  /**
   * <japanese>
   * ڕW܂łɂ܂f[^o͂郁\bhłB
   * o͒[q́AႦ
   * <a href="OhSendDouble.html#sendData(OhTimePoint, double)">OhSendDouble.sendData(OhTimePoint, double)</a>
   * Ȃǂ𗘗pĉB
   * vZsȂvfŕKĒ`ĉB
   * Ē`Ȃꍇ̓G[bZ[Wo exit ܂B
   * </japanese>
   * <p>
   * It is the method which outputs the data which accumulated by repetitive target time.
   * For example, please use 'OhSendDouble sendData(OhTimePoint aT, double aValue)'. <p>
   *
   * Please be sure to re-define by the element which performs repeated calculation.
   * When not giving a re-definition, an error message is sent, and it exits.
   *
   */
  public void sendStockedData() throws OhError {    //; (*49)
    StringWriter buffer = new StringWriter();
    PrintWriter  out    = new PrintWriter(buffer);
    oPrint(out, "\t");
    String str = buffer.toString();
    out.close();  
    throw new OhError(str);
  }


  /**
   * <japanese>
   * ̗vfvZ邱ƂɂȂA
   * KvƂȂf[^Mł邩`FbN郁\bh canYouCalculate() s邱ƂŁA
   * ̗vfˑĂ鑼̗vf֌vZv`B܂B<p>
   *
   * \bh canYouCalculate() sŁA
   * ̗vfւ̌vZvƂL^܂B
   * ̗vf work() ł̗vl܂B<p>
   *
   * ̃\bh́A̗vf̑M[qĂ΂܂B
   * nł́Ã\bhĂ΂邱Ƃ͂܂B
   * p[q́AvZv𑗐M[qɓ]邾ŁA
   * p[q̏L҂ł镔nɂ͌vZv𑗂ȂłB<p>
   *
   * ̒`sK؂łꍇ́ApNXōĒ`ĂB 
   * </japanese>
   *
   * @exception OhCannotGetDataError if an error occurs
   * @exception OhError if an error occurs
   */
  public void calReq() throws OhCannotGetDataError, OhError {
    long timeStepBackup = _timeStep;
    _timeStep = calculateTimeStep();    // Abstract method call.
                                        //; throws "OhError".
    canYouCalculate();                  //; Throw "OhCannotGetDataError" and "OhError".
    _timeStep = timeStepBackup;
    _calReqP = true;
  }


  // 6. information output function

  /**
   * <japanese>
   * LIuWFNg̏o͂܂B
   * </japanese>
   * <p>
   * Information output on possession objects.
   *
   * @param aPw a <code>PrintWriter</code> value
   * @param aHead a <code>String</code> value
   * @param aTitle a <code>String</code> value
   * @param aPortArray an <code>Object[]</code> value
   * @param aNumOfObjs an <code>int</code> value
   */
  public void printObjects(PrintWriter aPw, String aHead, String aTitle,    //; (*50)
    Object[] aPortArray, int aNumOfObjs) {
    if (aTitle != null) {
      aPw.print(aHead + aTitle);
      oPrint(aPw, "of : ");
    }
    else
      oPrint(aPw, aHead);

    for (int k = 0; k < aNumOfObjs; k++) {
      aPw.print(aHead);
      ((OhObject)aPortArray[k]).cPrint(aPw, "    ");
    }
  }


  /**
   * <japanese>
   * LM[q̏o͂܂B
   * </japanese>
   * <p>
   * Information output on possession receiving ports.
   *
   */
  public void printReceivePorts(PrintWriter aPw, String aHead) {
    printObjects(aPw, aHead, "receiving ports :", _receivePortArray, _numberOfReceivePorts);
  }

  
  /**
   * <japanese>
   * LM[q̏o͂܂B
   * </japanese>
   * <p>
   * Information output on possession receiving ports.
   *
   */
  public void printReceivePorts(PrintWriter aPw) {
    printObjects(aPw, "    ", "receiving ports :", _receivePortArray, _numberOfReceivePorts);
  }

  
  /**
   * <japanese>
   * LM[q̏̏o
   * </japanese>
   * <p>
   * Information output on possession receiving ports.
   *
   */
  public void printReceivePorts(String aHead) {
    PrintWriter pw = new PrintWriter(System.err, true);
    printObjects(pw, aHead, "receiving ports :", _receivePortArray, _numberOfReceivePorts);
  }

  
  /**
   * <japanese>
   * LM[q̏o͂܂B
   * </japanese>
   * <p>
   * Information output on possession receiving ports.
   *
   */
  public void printReceivePorts() {
    PrintWriter pw = new PrintWriter(System.err, true);
    printObjects(pw, "    ", "receiving ports :", _receivePortArray, _numberOfReceivePorts);
  }


  /**
   * <japanese>
   * LM[q̏o͂܂B
   * </japanese>
   * <p>
   * Information output on possession sending ports.
   *
   */
  public void printSendPorts(PrintWriter aPw, String aHead) {
    printObjects(aPw, aHead, "sending ports :", _sendPortArray, _numberOfSendPorts);
  }

  
  /**
   * <japanese>
   * LM[q̏o͂܂B
   * </japanese>
   * <p>
   * Information output on possession sending ports.
   *
   */
  public void printSendPorts(PrintWriter aPw) {
    printObjects(aPw, "    ", "sending ports :", _sendPortArray, _numberOfSendPorts);
  }

  
  /**
   * <japanese>
   * LM[q̏o͂܂B
   * </japanese>
   * <p>
   * Information output on possession sending ports.
   *
   */
  public void printSendPorts(String aHead) {
    PrintWriter pw = new PrintWriter(System.err, true);
    printObjects(pw, aHead, "sending ports :", _sendPortArray, _numberOfSendPorts);
  }

  
  /**
   * <japanese>
   * LM[q̏o͂܂B
   * </japanese>
   * <p>
   * Information output on possession sending ports.
   *
   */
  public void printSendPorts() {
    PrintWriter pw = new PrintWriter(System.err, true);
    printObjects(pw, "    ", "sending ports :", _sendPortArray, _numberOfSendPorts);
  }


  /**
   * <japanese>
   * ڑo͂܂B
   * </japanese>
   * <p>
   * Information output on connection.
   *
   */
  public void printConnection(PrintWriter aPw, String aHead) {    //; (*51-1)
    String str  = aHead + "    ";
    int k;

    for (k = 0; k < _numberOfReceivePorts; k++) {
      ((OhSenderPort)_receivePortArray[k]).cPrint(aPw, aHead);
      ((OhSenderPort)_receivePortArray[k]).printConnection(aPw, str);
    }

    for (k = 0; k < _numberOfReceivePorts; k++) {
      ((OhSenderPort)_sendPortArray[k]).cPrint(aPw, aHead);
      ((OhSenderPort)_sendPortArray[k]).printConnection(aPw, str);
    }
  }

  
  /**
   * <japanese>
   * ڑ̏o
   * </japanese>
   * <p>
   * Information output on connection.
   * At this time, the header is "    ".
   *
   */
  public void printConnection(PrintWriter aPw) {  //; (*51-2)
    String head = "    ";
    String str  = "        ";
    int k;

    for (k = 0; k < _numberOfReceivePorts; k++) {
      ((OhSenderPort)_receivePortArray[k]).cPrint(aPw, head);
      ((OhSenderPort)_receivePortArray[k]).printConnection(aPw, str);
    }

    for (k = 0; k < _numberOfReceivePorts; k++) {
      ((OhSenderPort)_sendPortArray[k]).cPrint(aPw, head);
      ((OhSenderPort)_sendPortArray[k]).printConnection(aPw, str);
    }
  }

  
  /**
   * <japanese>
   * ڑo͂܂B
   * </japanese>
   * <p>
   * Information output on connection.
   *
   */
  public void printConnection(String aHead) {          //; (*51-3)
    PrintWriter pw = new PrintWriter(System.err, true);
    String str  = aHead + "    ";
    int k;

    for (k = 0; k < _numberOfReceivePorts; k++) {
      ((OhSenderPort)_receivePortArray[k]).cPrint(pw, aHead);
      ((OhSenderPort)_receivePortArray[k]).printConnection(pw, str);
    }

    for (k = 0; k < _numberOfSendPorts; k++) {
      ((OhSenderPort)_sendPortArray[k]).cPrint(pw, aHead);
      ((OhSenderPort)_sendPortArray[k]).printConnection(pw, str);
    }
  }

  
  /**
   * <japanese>
   * ڑo͂܂B
   * </japanese>
   * <p>
   * Information output on connection.
   *
   */
  public void printConnection() {                      //; (*51-4)
    PrintWriter pw = new PrintWriter(System.err, true);
    String head = "    ";
    String str  = "        ";
    int k;

    for (k = 0; k < _numberOfReceivePorts; k++) {
      ((OhSenderPort)_receivePortArray[k]).cPrint(pw, head);
      ((OhSenderPort)_receivePortArray[k]).printConnection(pw, str);
    }

    for (k = 0; k < _numberOfReceivePorts; k++) {
      ((OhSenderPort)_sendPortArray[k]).cPrint(pw, head);
      ((OhSenderPort)_sendPortArray[k]).printConnection(pw, str);
    }
  }


  // 7. the function to save the last state at a file

  /**
   * <japanese>
   * ŏIԂۑ܂B
   * </japanese>
   *
   * @param aPw a <code>PrintWriter</code> value
   */
  public void terminalState(PrintWriter aPw) {                               //; (*52)
    aPw.print(_className + " " + _objectName + " " + _objectNumber + NEWLINE);
    aPw.print("{" + NEWLINE);

    saveTerminalState(aPw);

    aPw.print(NEWLINE);
    aPw.print("}" + NEWLINE);
  }


  // 8. addition

  /**
   * <japanese>
   * f̗vf쐬܂B<p>
   *
   * ɁAf̗vfƂȂ郂f\NX^ƁA
   * ̗vff̃IuWFNg𐶐܂B
   * </japanese>
   *
   * @param aClassName a <code>String</code> value
   * @return an <code>OhElement</code> value
   */
  public static OhElement create(String aClassName) {
    OhElement elm = null;
    try {
      elm = (OhElement)Class.forName("ohymosj." + aClassName).newInstance();
    }
    catch (Exception e) {
      e.printStackTrace();
    }
    return elm;
  }
  
  // @end {methods} ------------------------------------------------------

} // OhElement
