/**
 * @(#) OhScfSubSystem; subsystem created from a structure configuration file.
 *                      Make "OhScfSubSystem.java" from "ohscfsub.h" and "ohscfsub.cpp" by Satoh.
 */

package ohymosj;

import ohymosj.*;
import java.util.*;
import java.io.*;

/**
 * <japanese>
 * \`t@Cgpč\镔n\NXłB<p>
 *
 * ̃NX́A[q\vf̍\t@C "Structure Configuration File" 
 * ǂ݂Ƃč肠镔ñNXłB
 * </japanese>
 * <p>
 * ScfSubSystem;
 *
 * Sub system class configured from a "structure configuration file".
 *
 * @author <a href="mailto:satoh@wr.kuciv.kyoto-u.ac.jp">satoh</a>
 * @version $Revision: 1.25 $ $Date: 2004/03/08 17:03:02 $
 */
public class OhScfSubSystem extends OhSubSystem
{
  // @begin {instance variable or static variable} -----------------------

  /**
   * The array where relay ports for receiveing data are stored.
   *
   */
  protected OhPort[] _receivePorts;

  /**
   * Number of relay ports for receiving data.
   *
   */
  protected int _numOfReceivePorts;

  /**
   * The array where relay ports for sending data are stored.
   *
   */
  protected OhPort[] _sendPorts;

  /**
   * Number of relay ports for sending data.
   *
   */
  protected int _numOfSendPorts;

  /**
   * Usual(Non-iterate) Components.
   *
   */
  protected OhElement[] _elements;

  /**
   * Number of usual components.
   *
   */
  protected int _numOfElements;

  /**
   * Iterator Components.
   *
   */
  protected OhElement[] _iterators;

  /**
   * Number of iterator components.
   *
   */
  protected int _numOfIterators;

  /**
   * "Structure Configuration File" Name.
   *
   */
  protected String _scfFile;

  // @end {instance variable or static variable} -------------------------

  // @begin {initialization} ---------------------------------------------

  /**
   * Initialize the element with the information obtained from the file.
   *
   * <pre>
   * [in] objectname -- object name
   * [in] objnum     -- object number
   * [in] fp         -- file pointer
   * </pre>
   *
   * @param aObjName a <code>String</code> value
   * @param aObjNum an <code>int</code> value
   * @param aBr an <code>OhBufferedReader</code> value
   * @return a <code>boolean</code> value
   * @exception OhError if an error occurs
   */
  public boolean init(String aObjName, int aObjNum, OhBufferedReader aBr) throws OhError {
    
    // OhObject.init(aObjName, aObjNum);
    //
    if (aObjName == null) {
      StringWriter buffer = new StringWriter();
      PrintWriter  out    = new PrintWriter(buffer);
      oPrint(out, "\t");
      String str = buffer.toString();
      out.close();
      throw new IllegalArgumentException(str);
    }

    if (_objectName != null) {
      StringWriter buffer = new StringWriter();
      PrintWriter  out    = new PrintWriter(buffer);
      out.print("\taObjName(arg) = \"" + aObjName + "\", aObjNum(arg) = " + aObjNum + NEWLINE);
      oPrint(out, "\t");
      String str = buffer.toString();
      out.close();
      throw new OhError(str);
    }

    _objectName   = aObjName;
    _objectNumber = aObjNum;


    // Read SCF name from "aBr".
    //
    String fileName = null;
    try {
      aBr.skipWhites();
      fileName = aBr.readWord();
    }
    catch (Exception e) {
      e.printStackTrace();
    }

    _scfFile = fileName;


    // Open the SCF. Comment strings are removed.
    //
    FileReader       fr = null;
    OhBufferedReader br = null;

    try {
      fr = new FileReader(_scfFile);
      br = new OhBufferedReader(fr, true);
    }
    catch (FileNotFoundException e) {
      e.printStackTrace();
    }

    
    // Read Numbers part.
    //
    try {
      br.skipWhites();
      _numOfReceivePorts = Integer.parseInt(br.readWord());
      if (_numOfReceivePorts > 0) {
        _receivePorts = new OhPort[_numOfReceivePorts];
      }
      
      br.skipWhites();
      _numOfElements = Integer.parseInt(br.readWord());
      if (_numOfElements > 0) {
        _elements = new OhElement[_numOfElements];
      }
      
      br.skipWhites();
      _numOfIterators = Integer.parseInt(br.readWord());
      if (_numOfIterators > 0) {
        _iterators = new OhElement[_numOfIterators];
      }

      br.skipWhites();
      _numberOfIteratorSets = Integer.parseInt(br.readWord());
      
      br.skipWhites();
      _numOfSendPorts = Integer.parseInt(br.readWord());
      if (_numOfSendPorts > 0) {
        _sendPorts = new OhPort[_numOfSendPorts];
      }
    }
    catch (Exception e) {
      e.printStackTrace();
    }


    // Read Memory allocation part.
    //
    String header;
    String objectName;
    String className;
    String portType;
    int    objectNumber;

    // Read [ReceivePorts] section.
    //
    try {
      br.skipWhites();
      header = br.readWord();
      
      if (header.equals("[ReceivePorts]")) {
        int id;
        for (int i = 0; i < _numOfReceivePorts; i++) {
          br.skipWhites();
          id = Integer.parseInt(br.readWord());
          br.skipWhites();
          objectName = br.readWord();
          br.skipWhites();
          objectNumber = Integer.parseInt(br.readWord());
          br.skipWhites();
          className = br.readWord();

          if (id != i) {
            StringWriter buffer = new StringWriter();
            PrintWriter  out    = new PrintWriter(buffer);
            out.print("SCF data error in [ReceivePorts] section. The id number " + i
              + " was expected, but it was " + id + "." + NEWLINE);
            String str = buffer.toString();
            out.close();
            throw new OhError(str);
          }

//;       _receivePorts[i] = (OhPort)Class.forName("ohymosj." + className).newInstance();
          _receivePorts[i] = OhPort.create(className);
          _receivePorts[i].init(objectName, objectNumber);
        }
      }
      else {
        String str = "SCF data error: no [ReceivePorts] section." + NEWLINE;
        throw new OhError(str);
      }
    }
    catch (Exception e) {
      e.printStackTrace();
    }

    // Read [Components] section.
    //
    try {
      br.skipWhites();
      header = br.readWord();
      
      if (header.equals("[Components]")) {
        for (int i = 0; i < _numOfElements; i++) {
          int id;
          br.skipWhites();
          id = Integer.parseInt(br.readWord());
          br.skipWhites();
          objectName = br.readWord();
          br.skipWhites();
          objectNumber = Integer.parseInt(br.readWord());
          br.skipWhites();
          className = br.readWord();

          if (id != i) {
            StringWriter buffer = new StringWriter();
            PrintWriter  out    = new PrintWriter(buffer);
            out.print("SCF data error in [Components] section. The id number " + i
              + " was expected, but it was " + id + "." + NEWLINE);
            String str = buffer.toString();
            out.close();
            throw new OhError(str);
          }

//;       _elements[id] = (OhElement)Class.forName("ohymosj." + className).newInstance();
          _elements[id] = OhElement.create(className);
          _elements[id].init(objectName, objectNumber, br);
        }
      }
      else {
        String str = "SCF data error: no [Components] section." + NEWLINE;
        throw new OhError(str);
      }
    }
    catch (Exception e) {
      e.printStackTrace();
    }


    // Read [Iterators] section.
    //
    try {
      br.skipWhites();
      header = br.readWord();
      
      if (header.equals("[Iterators]")) {
        for (int i = 0; i < _numOfIterators; i++) {
          int id;
          br.skipWhites();
          id = Integer.parseInt(br.readWord());
          br.skipWhites();
          objectName = br.readWord();
          br.skipWhites();
          objectNumber = Integer.parseInt(br.readWord());
          br.skipWhites();
          className = br.readWord();

          if (id != i) {
            StringWriter buffer = new StringWriter();
            PrintWriter  out    = new PrintWriter(buffer);
            out.print("SCF data error in [Iterators] section. The id number " + i
              + " was expected, but it was " + id + "." + NEWLINE);
            String str = buffer.toString();
            out.close();
            throw new OhError(str);
          }

//;       _iterators[id] = (OhElement)Class.forName("ohymosj." + className).newInstance();
          _iterators[id] = OhElement.create(className);
          _iterators[id].init(objectName, objectNumber, br);
        }
      }
      else {
        String str = "SCF data error: no [Iterators] section." + NEWLINE;
        throw new OhError(str);
      }
    }
    catch (Exception e) {
      e.printStackTrace();
    }


    // Read [SendPorts] section.
    //
    try {
      br.skipWhites();
      header = br.readWord();
      
      if (header.equals("[SendPorts]")) {
        for (int i = 0; i < _numOfSendPorts; i++) {
          int id;
          br.skipWhites();
          id = Integer.parseInt(br.readWord());
          br.skipWhites();
          objectName = br.readWord();
          br.skipWhites();
          objectNumber = Integer.parseInt(br.readWord());
          br.skipWhites();
          className = br.readWord();
          br.skipWhites();
          portType = br.readWord();

          if (id != i) {
            StringWriter buffer = new StringWriter();
            PrintWriter  out    = new PrintWriter(buffer);
            out.print("SCF data error in [SendPorts] section. The id number " + i
              + " was expected, but it was " + id + "." + NEWLINE);
            String str = buffer.toString();
            out.close();
            throw new OhError(str);
          }

//;       _sendPorts[i] = (OhPort)Class.forName("ohymosj." + className).newInstance();
          _sendPorts[i] = OhPort.create(className);
          _sendPorts[i].init(objectName, objectNumber);
        }
      }
      else {
        String str = "SCF data error: no [SendPorts] section." + NEWLINE;
        throw new OhError(str);
      }
    }
    catch (Exception e) {
      e.printStackTrace();
    }

    return true;
  }    
    
  // @end {initialization} -----------------------------------------------

  // @begin {constructors} -----------------------------------------------

  /**
   * <japanese>
   * ftHgRXgN^łB
   * </japanese>
   *
   */
  public OhScfSubSystem() {
    super();
    _receivePorts = null;
    _sendPorts    = null;
    _elements     = null;
    _iterators    = null;
    _scfFile      = null;
    _className    = "OhScfSubSystem";
  }
  
  // @end {constructors} -------------------------------------------------

  // @begin {methods} ----------------------------------------------------

  /**
   * <japanese>
   * Mp[qo^܂B
   * hNXŒ`܂B
   * <a href="OhSubSystem.html#construct()">OhSubSystem.construct()</a>
   * Ă΂܂B
   * o^ĂȂMp[q͐ڑł܂B
   * </japanese>
   *
   * @exception OhError if an error occurs
   */
  public void registerReceivePorts() throws OhError {  //; (*1)
    for (int i = 0; i < _numOfReceivePorts; i++) {
      register((Object)_receivePorts[i]);
    }
  }


  /**
   * <japanese>
   * Mp[qo^܂B
   * hNXŒ`܂B
   * <a href="OhSubSystem.html#construct()">OhSubSystem.construct()</a>
   * Ă΂܂B
   * o^ĂȂMp[q͐ڑł܂B
   * </japanese>
   *
   * @exception OhError if an error occurs
   */
  public void registerSendPorts() throws OhError {  //; (*2)
    for (int i = 0; i < _numOfSendPorts; i++) {
      register((Object)_sendPorts[i]);
    }
  }


  /**
   * <japanese>
   * S\vf̂AvZsȂȂ\IuWFNg̃AhX _componentArray ɓo^܂B<p>
   *
   * o^ĂȂ\IuWFNgɂ́AvZs߂𑗂邱Ƃł܂B<p>
   *
   * Ⴆ΁AhNXō\IuWFNg abc oϐƂĒ`ꍇA
   * ̂悤 abc o^܂B
   *
   * <pre>
   *   public void registerComponents() {
   *     register((Object)abc);
   *   }
   * </pre>
   *
   * ̎AK abc  Object ^ɃLXgĉB<p>
   *
   * ɂ abc ́AꎞI _objectList Ɏ^ꂽA
   * ŏII Object ^z _componentArray ɋL܂B
   * i<a href="OhSubSystem.html#construct()">OhSubSystem.construct()</a> QƁj
   *
   * </japanese>
   *
   * @exception OhError if an error occurs
   */
  public void registerComponents() throws OhError {  //; (*3)
    for (int i = 0; i < _numOfElements; i++) {
      register((Object)_elements[i]);
    }
  }


  /**
   * <japanese>
   * ̃\bh́AS\vf̂AvZsȂvf̃AhX _iteratorArray ɓo^܂B
   * vZsȂnfVɍ쐬vO}Ã\bh`Ȃ΂Ȃ܂B<p>
   *
   * </japanese>
   *
   * @exception OhError if an error occurs
   */
  public void registerIterators() throws OhError {  //; (*4)
    if (_numberOfIteratorSets <= 0)
      return;

    // Open the SCF. Comment strings are removed.
    //
    FileReader       fr = null;
    OhBufferedReader br = null;

    try {
      fr = new FileReader(_scfFile);
      br = new OhBufferedReader(fr, true);
    }
    catch (FileNotFoundException e) {
      e.printStackTrace();
    }

    String  strLine = null;
    boolean foundP  = false;

    try {
      br.skipWhites();
      while ((strLine = br.readWord()) != null) {
        if (strLine.equals("[RegisterIterators]")) {
          foundP = true;
          break;
        }
      }
    }
    catch (IOException e) {
      e.printStackTrace();
    }

    if (foundP == false) {
      throw new OhError("");
    }


    try {
      for (int m = 0; m < _numberOfIteratorSets; m++) {
        int num;
        br.skipWhites();
        num = Integer.parseInt(br.readWord());
        
        //; _iteratorArray[i] == null ƂȂĂ i TB exit B
        //;
        int i;
        boolean flag = false;
        
        for (i = 0; i < _numberOfIteratorSets; i++) {
          if (_iteratorArray[i] == null) {
            flag = true;
            break;
          }
        }
        
        if (flag == false) {
          StringWriter buffer = new StringWriter();
          PrintWriter  out    = new PrintWriter(buffer);
          oPrint(out, "    ");
          String str = buffer.toString();
          out.close();
          throw new OhError(str);
        }
        
        for (int k = 0; k < num; k++) {
          int s;
          br.skipWhites();
          s = Integer.parseInt(br.readWord());
          register(_iterators[s]);
        }
        
        int[] numberOfIteratorsArray = {_numberOfIterators[i]};
        
        _iteratorArray[i]     = makeObjectArray(numberOfIteratorsArray);    //; OhElement (*4)
        _numberOfIterators[i] = numberOfIteratorsArray[0];
        
        _objectList.clear();
      }
    }
    catch (IOException e) {
      e.printStackTrace();
    }
  }


  /**
   * <japanese>
   * nf̒[q̐ڑ
   * <a href="OhSubSystem.html#connect()">OhSubSystem.connect()</a>
   * pĒ`܂B
   * </japanese>
   *
   * @exception OhError if an error occurs
   */
  public void connectPorts() throws OhError {  //; (*5)
    // Open the SCF. Comment strings are removed.
    //
    FileReader       fr = null;
    OhBufferedReader br = null;

    try {
      fr = new FileReader(_scfFile);
      br = new OhBufferedReader(fr, true);
    }
    catch (FileNotFoundException e) {
      e.printStackTrace();
    }


    String  strLine = null;
    boolean foundP  = false;

    try {
      br.skipWhites();
      while ((strLine = br.readWord()) != null) {
        if (strLine.equals("[RegisterIterators]")) {
          foundP = true;
          break;
        }
      }
    }
    catch (IOException e) {
      e.printStackTrace();
    }

    if (foundP == false) {
      throw new OhError("");
    }

    String objNameS = null;
    String objNameR = null;
    int objNumS = 0;
    int objNumR = 0;

    try {
      while (true) {  // ------------------------------------------------- starts while(true) loop.
        String str;
        br.skipWhites();
        str = br.readWord();

        if (str.charAt(0) == 'z') {
          break;
        }

        // -- There are nine cases. ---

        if (str.charAt(0) == 'r') {    // The sender is a receive port.
          int i;
          br.skipWhites();
          i = Integer.parseInt(br.readWord());    // Receive port ID.

          if (i >= _numOfReceivePorts) {
            StringWriter buffer = new StringWriter();
            PrintWriter  out    = new PrintWriter(buffer);
            out.print(_scfFile + ": Illegal receive port number: r " + i);
            String errMsg = buffer.toString();
            out.close();
            throw new OhError(errMsg);
          }

          // The receiver.
          //
          String strR;
          br.skipWhites();
          strR = br.readWord();

          int j;

          char r = strR.charAt(0);

          switch (r) {

          case 'c':    // The receiver is a usual component.
            br.skipWhites();
            j = Integer.parseInt(br.readWord());
            br.skipWhites();
            objNameR = br.readWord();
            br.skipWhites();
            objNumR = Integer.parseInt(br.readWord());

            if (j >= _numOfElements) {
              StringWriter buffer = new StringWriter();
              PrintWriter  out    = new PrintWriter(buffer);
              out.print(_scfFile + ": Illegal element number: c " + j);
              String errMsg = buffer.toString();
              out.close();
              throw new OhError(errMsg);
            }

            connect((Object)_receivePorts[i], (Object)_elements[j], objNameR, objNumR);

            break;

          case 'i':    // The receiver is a iterator component.
            br.skipWhites();
            j = Integer.parseInt(br.readWord());
            br.skipWhites();
            objNameR = br.readWord();
            br.skipWhites();
            objNumR = Integer.parseInt(br.readWord());

            if (j >= _numOfIterators) {
              StringWriter buffer = new StringWriter();
              PrintWriter  out    = new PrintWriter(buffer);
              out.print(_scfFile + ": Illegal iterator element number: i " + j);
              String errMsg = buffer.toString();
              out.close();
              throw new OhError(errMsg);
            }

            connect((Object)_receivePorts[i], (Object)_iterators[j], objNameR, objNumR);

            break;

          case 's':    // The receiver is a send port.
            br.skipWhites();
            j = Integer.parseInt(br.readWord());
            
            if (j >= _numOfSendPorts) {
              StringWriter buffer = new StringWriter();
              PrintWriter  out    = new PrintWriter(buffer);
              out.print(_scfFile + ": Illegal output port number: s " + j);
              String errMsg = buffer.toString();
              out.close();
              throw new OhError(errMsg);
            }

            connect(_receivePorts[i], _sendPorts[j]);

            break;

          default:
            throw new OhError("");
          }
        }
        else if (str.charAt(0) == 'c') {    // The sender is a usual component.
          int i;
          br.skipWhites();
          i = Integer.parseInt(br.readWord());
          br.skipWhites();
          objNameS = br.readWord();
          br.skipWhites();
          objNumS = Integer.parseInt(br.readWord());

          if (i >= _numOfElements) {
            StringWriter buffer = new StringWriter();
            PrintWriter  out    = new PrintWriter(buffer);
            out.print(_scfFile + ": Illegal element number: c " + i);
            String errMsg = buffer.toString();
            out.close();
            throw new OhError(errMsg);
          }

          // The receiver.
          //
          String strR;
          br.skipWhites();
          strR = br.readWord();

          int j;

          char r = strR.charAt(0);

          switch (r) {

          case 'c':    // The receiver is a usual component.
            br.skipWhites();
            j = Integer.parseInt(br.readWord());
            br.skipWhites();
            objNameR = br.readWord();
            br.skipWhites();
            objNumR = Integer.parseInt(br.readWord());

            if (j >= _numOfElements) {
              StringWriter buffer = new StringWriter();
              PrintWriter  out    = new PrintWriter(buffer);
              out.print(_scfFile + ": Illegal element number: c " + i);    //; j ̊ԈႢł́HHH
              String errMsg = buffer.toString();
              out.close();
              throw new OhError(errMsg);
            }

            connect((Object)_elements[i], objNameS, objNumS, (Object)_elements[j], objNameR, objNumR);

            break;

          case 'i':    // The receiver is a iterator component.
            br.skipWhites();
            j = Integer.parseInt(br.readWord());
            br.skipWhites();
            objNameR = br.readWord();
            br.skipWhites();
            objNumR = Integer.parseInt(br.readWord());

            if (j >= _numOfIterators) {
              StringWriter buffer = new StringWriter();
              PrintWriter  out    = new PrintWriter(buffer);
              out.print(_scfFile + ": Illegal iterator element number: i " + j);
              String errMsg = buffer.toString();
              out.close();
              throw new OhError(errMsg);
            }

            connect((Object)_elements[i], objNameS, objNumS, (Object)_iterators[j], objNameR, objNumR);

            break;

          case 's':    // The receiver is a send port.
            br.skipWhites();
            j = Integer.parseInt(br.readWord());
            
            if (j >= _numOfSendPorts) {
              StringWriter buffer = new StringWriter();
              PrintWriter  out    = new PrintWriter(buffer);
              out.print(_scfFile + ": Illegal output port number: s " + j);
              String errMsg = buffer.toString();
              out.close();
              throw new OhError(errMsg);
            }

            connect((Object)_elements[i], objNameS, objNumS, (Object)_sendPorts[j]);

            break;

          default:
            throw new OhError("");
          }
        }
        else if (str.charAt(0) == 'i') {    // The sender is a iterator component.
          int i;
          br.skipWhites();
          i = Integer.parseInt(br.readWord());
          br.skipWhites();
          objNameS = br.readWord();
          br.skipWhites();
          objNumS = Integer.parseInt(br.readWord());

          if (i >= _numOfIterators) {
            StringWriter buffer = new StringWriter();
            PrintWriter  out    = new PrintWriter(buffer);
            out.print(_scfFile + ": Illegal iterator element number: i " + i);
            String errMsg = buffer.toString();
            out.close();
            throw new OhError(errMsg);
          }

          // The receiver.
          //
          String strR;
          br.skipWhites();
          strR = br.readWord();

          int j;

          char r = strR.charAt(0);

          switch (r) {

          case 'c':    // The receiver is a usual component.
            br.skipWhites();
            j = Integer.parseInt(br.readWord());
            br.skipWhites();
            objNameR = br.readWord();
            br.skipWhites();
            objNumR = Integer.parseInt(br.readWord());

            if (j >= _numOfElements) {
              StringWriter buffer = new StringWriter();
              PrintWriter  out    = new PrintWriter(buffer);
              out.print(_scfFile + ": Illegal element number: c " + j);
              String errMsg = buffer.toString();
              out.close();
              throw new OhError(errMsg);
            }

            connect((Object)_iterators[i], objNameS, objNumS, (Object)_elements[j], objNameR, objNumR);

            break;

          case 'i':    // The receiver is a iterator component.
            br.skipWhites();
            j = Integer.parseInt(br.readWord());
            br.skipWhites();
            objNameR = br.readWord();
            br.skipWhites();
            objNumR = Integer.parseInt(br.readWord());

            if (j >= _numOfIterators) {
              StringWriter buffer = new StringWriter();
              PrintWriter  out    = new PrintWriter(buffer);
              out.print(_scfFile + ": Illegal iterator element number: i " + j);
              String errMsg = buffer.toString();
              out.close();
              throw new OhError(errMsg);
            }

            connect((Object)_iterators[i], objNameS, objNumS, (Object)_iterators[j], objNameR, objNumR);

            break;

          case 's':    // The receiver is a send port.
            br.skipWhites();
            j = Integer.parseInt(br.readWord());
            
            if (j >= _numOfSendPorts) {
              StringWriter buffer = new StringWriter();
              PrintWriter  out    = new PrintWriter(buffer);
              out.print(_scfFile + ": Illegal output port number: s " + j);
              String errMsg = buffer.toString();
              out.close();
              throw new OhError(errMsg);
            }

            connect((Object)_iterators[i], objNameS, objNumS, (Object)_sendPorts[j]);

            break;

          default:
            throw new OhError("");
          }
        }
        else {
          throw new OhError("");
        }
      }  // -------------------------------------------------------------- ends while(true) loop.
    }
    catch (IOException e) {
      e.printStackTrace();
    }
  }


  /**
   * <japanese>
   * nf̒ڒʐMڑ
   * <a href="OhElement.html#register(Object, int)">OhElement.register(Object, int)</a>
   * pĒ`܂B
   *
   * <pre>
   * [ConnectComponents]
   * c 2 c 3 0 # mElmnt[2]->Register((void*)mElmnt[3], 0)
   * i 0 i 1 2 # mIte[0]->Register((void*)mIte[1], 2);
   * z # end mark
   * </pre>
   * </japanese>
   *
   * @exception OhError if an error occurs
   */
  public void connectComponents() throws OhError {    //; (*6)
    // Open the SCF. Comment strings are removed.
    //
    FileReader       fr = null;
    OhBufferedReader br = null;

    try {
      fr = new FileReader(_scfFile);
      br = new OhBufferedReader(fr, true);
    }
    catch (FileNotFoundException e) {
      e.printStackTrace();
    }


    String  strLine = null;
    boolean foundP  = false;

    try {
      br.skipWhites();
      while ((strLine = br.readWord()) != null) {
        if (strLine.equals("[ConnectComponents]")) {
          foundP = true;
          break;
        }
      }
    }
    catch (IOException e) {
      e.printStackTrace();
    }

    if (foundP == false) {
      throw new OhError("");
    }

    try {
      while (true) {
        String strCom1;
        String strCom2;
        int i1;
        int i2;
        int j;
        
        br.skipWhites();
        strCom1 = br.readWord();
        
        if (strCom1.charAt(0) == 'z') {
          break;
        }
        
        br.skipWhites();
        i1 = Integer.parseInt(br.readWord());
        br.skipWhites();
        strCom2 = br.readWord();
        br.skipWhites();
        i2 = Integer.parseInt(br.readWord());
        br.skipWhites();
        j = Integer.parseInt(br.readWord());
        
        OhElement elm1;
        OhElement elm2;
        
        elm1 = (strCom1.charAt(0) == 'c') ? _elements[i1] : _iterators[i1];
        elm2 = (strCom2.charAt(0) == 'c') ? _elements[i2] : _iterators[i2];
        
        elm1.register((Object)elm2, j);
      }
    }
    catch (IOException e) {
      e.printStackTrace();
    }
  }

  // @end {methods} ------------------------------------------------------

} // OhScfSubSystem
