/**
 * @(#) OhScfTotalSystem; total system created from a structure configuration file.
 *                        Make "OhScfTotalSystem.java" from "ohscftotal.h" and "ohscftotal.cpp" by Satoh.
 */

package ohymosj;

import ohymosj.*;
import java.util.*;
import java.io.*;

/**
 * <japanese>
 * \`t@Cgpč\Šn\NXłB<p>
 *
 * ̃NX́A[q\vf̍\t@C "Structure Configuration File" 
 * ǂ݂Ƃč肠ŠñNXłB
 * </japanese>
 * <p>
 * ScfTotalSystem;
 *
 * Total system class configured from a "structure configuration file".
 *
 * @author <a href="mailto:satoh@wr.kuciv.kyoto-u.ac.jp"></a>
 * @version $Revision: 1.13 $ $Date: 2004/03/08 17:03:02 $
 */
public class OhScfTotalSystem extends OhTotalSystem
{
  // @begin {instance variable or static variable} -----------------------

  /**
   * The array where relay ports for receiveing data are stored.
   *
   */
  protected OhPort[] _receivePorts;

  /**
   * Number of relay ports for receiving data.
   *
   */
  protected int _numOfReceivePorts;

  /**
   * The array where relay ports for sending data are stored.
   *
   */
  protected OhPort[] _sendPorts;

  /**
   * Number of relay ports for sending data.
   *
   */
  protected int _numOfSendPorts;

  /**
   * Usual(Non-iterate) Components.
   *
   */
  protected OhElement[] _elements;

  /**
   * Number of usual components.
   *
   */
  protected int _numOfElements;

  /**
   * Iterator Components.
   *
   */
  protected OhElement[] _iterators;

  /**
   * Number of iterator components.
   *
   */
  protected int _numOfIterators;

  /**
   * "Structure Configuration File" Name.
   *
   */
  protected String _scfFile;

  // @end {instance variable or static variable} -------------------------


  // @begin {constructors} -----------------------------------------------

  /**
   * <japanese>
   * ftHgRXgN^łB
   * </japanese>
   *
   * @param aFileName a <code>String</code> value
   * @exception OhError if an error occurs
   */
  public OhScfTotalSystem(String aFileName) throws OhError {
    super();
    _receivePorts = null;
    _sendPorts    = null;
    _elements     = null;
    _iterators    = null;
    _scfFile      = null;
    _className    = "OhScfTotalSystem";

    // OhObject.init("OhTotalSystem", 0);
    //
    if (_objectName != null) {
      StringWriter buffer = new StringWriter();
      PrintWriter  out    = new PrintWriter(buffer);
      out.print("\t\"OhScfTotalSystem\", aObjNum(arg) = 0" + NEWLINE);
      oPrint(out, "\t");
      String str = buffer.toString();
      out.close();
      throw new OhError(str);
    }

    _objectName   = "totalSystem";
    _objectNumber = 0;


    // Read SCF name from "aBr".
    //
    _scfFile = aFileName;


    // Open the SCF. Comment strings are removed.
    //
    FileReader       fr = null;
    OhBufferedReader br = null;

    try {
      fr = new FileReader(_scfFile);
      br = new OhBufferedReader(fr, true);
    }
    catch (FileNotFoundException e) {
      e.printStackTrace();
    }


    // Read Numbers part.
    //
    try {
      br.skipWhites();
      _numOfReceivePorts = Integer.parseInt(br.readWord());
      if (_numOfReceivePorts > 0) {
        _receivePorts = new OhPort[_numOfReceivePorts];
      }
      
      br.skipWhites();
      _numOfElements = Integer.parseInt(br.readWord());
      if (_numOfElements > 0) {
        _elements = new OhElement[_numOfElements];
      }
      
      br.skipWhites();
      _numOfIterators = Integer.parseInt(br.readWord());
      if (_numOfIterators > 0) {
        _iterators = new OhElement[_numOfIterators];
      }

      br.skipWhites();
      _numberOfIteratorSets = Integer.parseInt(br.readWord());
      
      br.skipWhites();
      _numOfSendPorts = Integer.parseInt(br.readWord());
      if (_numOfSendPorts > 0) {
        _sendPorts = new OhPort[_numOfSendPorts];
      }
    }
    catch (Exception e) {
      e.printStackTrace();
    }


    // Read Memory allocation part.
    //
    String header;
    String objectName;
    String className;
    int    objectNumber;


    // Read [InputPorts] section.
    //
    try {
      br.skipWhites();
      header = br.readWord();
      
      if (header.equals("[InputPorts]")) {
        int id;
        for (int i = 0; i < _numOfReceivePorts; i++) {
          br.skipWhites();
          id = Integer.parseInt(br.readWord());
          br.skipWhites();
          objectName = br.readWord();
          br.skipWhites();
          className = br.readWord();

          if (id != i) {
            StringWriter buffer = new StringWriter();
            PrintWriter  out    = new PrintWriter(buffer);
            out.print("SCF Total data error in [InputPorts] section. The id number " + i
              + " was expected, but it was " + id + "." + NEWLINE);
            String str = buffer.toString();
            out.close();
            throw new OhError(str);
          }

//;       _receivePorts[i] = (OhPort)Class.forName("ohymosj." + className).newInstance();
          _receivePorts[i] = OhPort.create(className);
          _receivePorts[i].init(objectName, 0);
        }
      }
      else {
        String str = "SCF Total data error: no [InputPorts] section." + NEWLINE;
        throw new OhError(str);
      }
    }
    catch (Exception e) {
      e.printStackTrace();
    }


    // Read [Components] section.
    //
    try {
      br.skipWhites();
      header = br.readWord();
      
      if (header.equals("[Components]")) {
        for (int i = 0; i < _numOfElements; i++) {
          int id;
          br.skipWhites();
          id = Integer.parseInt(br.readWord());
          br.skipWhites();
          objectName = br.readWord();
          br.skipWhites();
          objectNumber = Integer.parseInt(br.readWord());
          br.skipWhites();
          className = br.readWord();

          if (id != i) {
            StringWriter buffer = new StringWriter();
            PrintWriter  out    = new PrintWriter(buffer);
            out.print("SCF Total data error in [Components] section. The id number " + i
              + " was expected, but it was " + id + "." + NEWLINE);
            String str = buffer.toString();
            out.close();
            throw new OhError(str);
          }

//;       _elements[id] = (OhElement)Class.forName("ohymosj." + className).newInstance();
          _elements[id] = OhElement.create(className);
          _elements[id].init(objectName, objectNumber, br);
        }
      }
      else {
        String str = "SCF Total data error: no [Components] section." + NEWLINE;
        throw new OhError(str);
      }
    }
    catch (Exception e) {
      e.printStackTrace();
    }


    // Read [Iterators] section.
    //
    try {
      br.skipWhites();
      header = br.readWord();
      
      if (header.equals("[Iterators]")) {
        for (int i = 0; i < _numOfIterators; i++) {
          int id;
          br.skipWhites();
          id = Integer.parseInt(br.readWord());
          br.skipWhites();
          objectName = br.readWord();
          br.skipWhites();
          objectNumber = Integer.parseInt(br.readWord());
          br.skipWhites();
          className = br.readWord();

          if (id != i) {
            StringWriter buffer = new StringWriter();
            PrintWriter  out    = new PrintWriter(buffer);
            out.print("SCF Total data error in [Iterators] section. The id number " + i
              + " was expected, but it was " + id + "." + NEWLINE);
            String str = buffer.toString();
            out.close();
            throw new OhError(str);
          }

//;       _iterators[id] = (OhElement)Class.forName("ohymosj." + className).newInstance();
          _iterators[id] = OhElement.create(className);
          _iterators[id].init(objectName, objectNumber, br);
        }
      }
      else {
        String str = "SCF Total data error: no [Iterators] section." + NEWLINE;
        throw new OhError(str);
      }
    }
    catch (Exception e) {
      e.printStackTrace();
    }


    // Read [OutputPorts] section.
    //
    try {
      br.skipWhites();
      header = br.readWord();
      
      if (header.equals("[OutputPorts]")) {
        for (int i = 0; i < _numOfSendPorts; i++) {
          int id;
          int lineLength;
          br.skipWhites();
          id = Integer.parseInt(br.readWord());
          br.skipWhites();
          objectName = br.readWord();
          br.skipWhites();
          lineLength = Integer.parseInt(br.readWord());
          br.skipWhites();
          className = br.readWord();

          if (id != i) {
            StringWriter buffer = new StringWriter();
            PrintWriter  out    = new PrintWriter(buffer);
            out.print("SCF Total data error in [SendPorts] section. The id number " + i
              + " was expected, but it was " + id + "." + NEWLINE);
            String str = buffer.toString();
            out.close();
            throw new OhError(str);
          }

//;       _sendPorts[i] = (OhPort)Class.forName("ohymosj." + className).newInstance();
          _sendPorts[i] = OhPort.create(className);
          _sendPorts[i].init(objectName, lineLength);
        }
      }
      else {
        String str = "SCF Total data error: no [SendPorts] section." + NEWLINE;
        throw new OhError(str);
      }
    }
    catch (Exception e) {
      e.printStackTrace();
    }

  }
  
  // @end {constructors} -------------------------------------------------

  // @begin {methods} ----------------------------------------------------

  /**
   * <japanese>
   * Mp[qo^܂B
   * <a href="OhSubSystem.html#construct()">OhSubSystem.construct()</a>
   * Ă΂܂B
   * o^ĂȂMp[q͐ڑł܂B
   * </japanese>
   *
   * @exception OhError if an error occurs
   */
  public void registerInputPorts() throws OhError {  //; (*7)
    for (int i = 0; i < _numOfReceivePorts; i++) {
      register((Object)_receivePorts[i]);
    }
  }


  /**
   * <japanese>
   * <a href="OhScfTotalSystem.html#registerOutputPorts()">OhScfTotalSystem.registerOutputPorts()</a>
   * s܂B
   * </japanese>
   *
   * @exception OhError if an error occurs
   */
  public void registerReceivePorts() throws OhError {  //; (*1)
    registerOutputPorts();
  }



  /**
   * <japanese>
   * Mp[qo^܂B
   * <a href="OhSubSystem.html#construct()">OhSubSystem.construct()</a>
   * Ă΂܂B
   * o^ĂȂMp[q͐ڑł܂B
   * </japanese>
   *
   * @exception OhError if an error occurs
   */
  public void registerOutputPorts() throws OhError {  //; (*8)
    for (int i = 0; i < _numOfSendPorts; i++) {
      register((Object)_sendPorts[i]);
    }
  }


  /**
   * <japanese>
   * <a href="OhScfTotalSystem.html#registerInputPorts()">OhScfTotalSystem.registerInputPorts()</a>
   * s܂B
   * </japanese>
   *
   * @exception OhError if an error occurs
   */
  public void registerSendPorts() throws OhError {  //; (*2)
    registerInputPorts();
  }


  /**
   * <japanese>
   * S\vf̂AvZsȂȂ\IuWFNg̃AhX _componentArray ɓo^܂B<p>
   *
   * o^ĂȂ\IuWFNgɂ́AvZs߂𑗂邱Ƃł܂B<p>
   *
   * Ⴆ΁AhNXō\IuWFNg abc oϐƂĒ`ꍇA
   * ̂悤 abc o^܂B
   *
   * <pre>
   *   public void registerComponents() {
   *     register((Object)abc);
   *   }
   * </pre>
   *
   * ̎AK abc  Object ^ɃLXgĉB<p>
   *
   * ɂ abc ́AꎞI _objectList Ɏ^ꂽA
   * ŏII Object ^z _componentArray ɋL܂B
   * i<a href="OhSubSystem.html#construct()">OhSubSystem.construct()</a> QƁj
   *
   * </japanese>
   *
   * @exception OhError if an error occurs
   */
  public void registerComponents() throws OhError {  //; (*3)
    for (int i = 0; i < _numOfElements; i++) {
      register((Object)_elements[i]);
    }
  }


  /**
   * <japanese>
   * ̃\bh́AS\vf̂AvZsȂvf̃AhX _iteratorArray ɓo^܂B
   * vZsȂnfVɍ쐬vO}Ã\bh`Ȃ΂Ȃ܂B<p>
   * </japanese>
   *
   * @exception OhError if an error occurs
   */
  public void registerIterators() throws OhError {  //; (*4)
    if (_numberOfIteratorSets <= 0)
      return;

    // Open the SCF. Comment strings are removed.
    //
    FileReader       fr = null;
    OhBufferedReader br = null;

    try {
      fr = new FileReader(_scfFile);
      br = new OhBufferedReader(fr, true);
    }
    catch (FileNotFoundException e) {
      e.printStackTrace();
    }

    String  strLine = null;
    boolean foundP  = false;

    try {
      br.skipWhites();
      while ((strLine = br.readWord()) != null) {
        if (strLine.equals("[RegisterIterators]")) {
          foundP = true;
          break;
        }
      }
    }
    catch (IOException e) {
      e.printStackTrace();
    }

    if (foundP == false) {
      throw new OhError("");
    }


    try {
      for (int m = 0; m < _numberOfIteratorSets; m++) {
        int num;
        br.skipWhites();
        num = Integer.parseInt(br.readWord());
        
        //; _iteratorArray[i] == null ƂȂĂ i TB exit B
        //;
        int i;
        boolean flag = false;
        
        for (i = 0; i < _numberOfIteratorSets; i++) {
          if (_iteratorArray[i] == null) {
            flag = true;
            break;
          }
        }
        
        if (flag == false) {
          StringWriter buffer = new StringWriter();
          PrintWriter  out    = new PrintWriter(buffer);
          oPrint(out, "    ");
          String str = buffer.toString();
          out.close();
          throw new OhError(str);
        }
        
        for (int k = 0; k < num; k++) {
          int s;
          br.skipWhites();
          s = Integer.parseInt(br.readWord());
          register(_iterators[s]);
        }
        
        int[] numberOfIteratorsArray = {_numberOfIterators[i]};
        
        _iteratorArray[i]     = makeObjectArray(numberOfIteratorsArray);    //; OhElement (*4)
        _numberOfIterators[i] = numberOfIteratorsArray[0];
        
        _objectList.clear();
      }
    }
    catch (IOException e) {
      e.printStackTrace();
    }
  }


  /**
   * <japanese>
   * nf̒[q̐ڑ
   * <a href="OhSubSystem.html#connect()">OhSubSystem.connect()</a>
   * pĒ`܂B
   * </japanese>
   *
   * @exception OhError if an error occurs
   */
  public void connectPorts() throws OhError {  //; (*5)
    // Open the SCF. Comment strings are removed.
    //
    FileReader       fr = null;
    OhBufferedReader br = null;

    try {
      fr = new FileReader(_scfFile);
      br = new OhBufferedReader(fr, true);
    }
    catch (FileNotFoundException e) {
      e.printStackTrace();
    }


    String  strLine = null;
    boolean foundP  = false;

    try {
      br.skipWhites();
      while ((strLine = br.readWord()) != null) {
        if (strLine.equals("[ConnectPorts]")) {
          foundP = true;
          break;
        }
      }
    }
    catch (IOException e) {
      e.printStackTrace();
    }

    if (foundP == false) {
      throw new OhError("No [ConnectPorts] section.");
    }

    String objNameS = null;
    String objNameR = null;
    int objNumS = 0;
    int objNumR = 0;

    try {
      while (true) {  // ------------------------------------------------- starts while(true) loop.
        String str;
        br.skipWhites();
        str = br.readWord();

        if (str.charAt(0) == 'z') {
          break;
        }

        // -- There are nine cases. ---

        if (str.charAt(0) == 'r') {    // The sender is a receive port.
          int i;
          br.skipWhites();
          i = Integer.parseInt(br.readWord());    // Receive port ID.

          if (i >= _numOfReceivePorts) {
            StringWriter buffer = new StringWriter();
            PrintWriter  out    = new PrintWriter(buffer);
            out.print(_scfFile + ": Illegal receive port number: r " + i);
            String errMsg = buffer.toString();
            out.close();
            throw new OhError(errMsg);
          }

          // The receiver.
          //
          String strR;
          br.skipWhites();
          strR = br.readWord();

          int j;

          char r = strR.charAt(0);

          switch (r) {

          case 'c':    // The receiver is a usual component.
            br.skipWhites();
            j = Integer.parseInt(br.readWord());
            br.skipWhites();
            objNameR = br.readWord();
            br.skipWhites();
            objNumR = Integer.parseInt(br.readWord());

            if (j >= _numOfElements) {
              StringWriter buffer = new StringWriter();
              PrintWriter  out    = new PrintWriter(buffer);
              out.print(_scfFile + ": Illegal element number: c " + j);
              String errMsg = buffer.toString();
              out.close();
              throw new OhError(errMsg);
            }

            connect((Object)_receivePorts[i], (Object)_elements[j], objNameR, objNumR);

            break;

          case 'i':    // The receiver is a iterator component.
            br.skipWhites();
            j = Integer.parseInt(br.readWord());
            br.skipWhites();
            objNameR = br.readWord();
            br.skipWhites();
            objNumR = Integer.parseInt(br.readWord());

            if (j >= _numOfIterators) {
              StringWriter buffer = new StringWriter();
              PrintWriter  out    = new PrintWriter(buffer);
              out.print(_scfFile + ": Illegal iterator element number: i " + j);
              String errMsg = buffer.toString();
              out.close();
              throw new OhError(errMsg);
            }

            connect((Object)_receivePorts[i], (Object)_iterators[j], objNameR, objNumR);

            break;

          case 's':    // The receiver is a send port.
            br.skipWhites();
            j = Integer.parseInt(br.readWord());
            
            if (j >= _numOfSendPorts) {
              StringWriter buffer = new StringWriter();
              PrintWriter  out    = new PrintWriter(buffer);
              out.print(_scfFile + ": Illegal output port number: s " + j);
              String errMsg = buffer.toString();
              out.close();
              throw new OhError(errMsg);
            }

            connect(_receivePorts[i], _sendPorts[j]);

            break;

          default:
            throw new OhError("");
          }
        }
        else if (str.charAt(0) == 'c') {    // The sender is a usual component.
          int i;
          br.skipWhites();
          i = Integer.parseInt(br.readWord());
          br.skipWhites();
          objNameS = br.readWord();
          br.skipWhites();
          objNumS = Integer.parseInt(br.readWord());

          if (i >= _numOfElements) {
            StringWriter buffer = new StringWriter();
            PrintWriter  out    = new PrintWriter(buffer);
            out.print(_scfFile + ": Illegal element number: c " + i);
            String errMsg = buffer.toString();
            out.close();
            throw new OhError(errMsg);
          }

          // The receiver.
          //
          String strR;
          br.skipWhites();
          strR = br.readWord();

          int j;

          char r = strR.charAt(0);

          switch (r) {

          case 'c':    // The receiver is a usual component.
            br.skipWhites();
            j = Integer.parseInt(br.readWord());
            br.skipWhites();
            objNameR = br.readWord();
            br.skipWhites();
            objNumR = Integer.parseInt(br.readWord());

            if (j >= _numOfElements) {
              StringWriter buffer = new StringWriter();
              PrintWriter  out    = new PrintWriter(buffer);
              out.print(_scfFile + ": Illegal element number: c " + i);    //; j ̊ԈႢł́HHH
              String errMsg = buffer.toString();
              out.close();
              throw new OhError(errMsg);
            }

            connect((Object)_elements[i], objNameS, objNumS, (Object)_elements[j], objNameR, objNumR);

            break;

          case 'i':    // The receiver is a iterator component.
            br.skipWhites();
            j = Integer.parseInt(br.readWord());
            br.skipWhites();
            objNameR = br.readWord();
            br.skipWhites();
            objNumR = Integer.parseInt(br.readWord());

            if (j >= _numOfIterators) {
              StringWriter buffer = new StringWriter();
              PrintWriter  out    = new PrintWriter(buffer);
              out.print(_scfFile + ": Illegal iterator element number: i " + j);
              String errMsg = buffer.toString();
              out.close();
              throw new OhError(errMsg);
            }

            connect((Object)_elements[i], objNameS, objNumS, (Object)_iterators[j], objNameR, objNumR);

            break;

          case 's':    // The receiver is a send port.
            br.skipWhites();
            j = Integer.parseInt(br.readWord());
            
            if (j >= _numOfSendPorts) {
              StringWriter buffer = new StringWriter();
              PrintWriter  out    = new PrintWriter(buffer);
              out.print(_scfFile + ": Illegal output port number: s " + j);
              String errMsg = buffer.toString();
              out.close();
              throw new OhError(errMsg);
            }

            connect((Object)_elements[i], objNameS, objNumS, (Object)_sendPorts[j]);

            break;

          default:
            throw new OhError("");
          }
        }
        else if (str.charAt(0) == 'i') {    // The sender is a iterator component.
          int i;
          br.skipWhites();
          i = Integer.parseInt(br.readWord());
          br.skipWhites();
          objNameS = br.readWord();
          br.skipWhites();
          objNumS = Integer.parseInt(br.readWord());

          if (i >= _numOfIterators) {
            StringWriter buffer = new StringWriter();
            PrintWriter  out    = new PrintWriter(buffer);
            out.print(_scfFile + ": Illegal iterator element number: i " + i);
            String errMsg = buffer.toString();
            out.close();
            throw new OhError(errMsg);
          }

          // The receiver.
          //
          String strR;
          br.skipWhites();
          strR = br.readWord();

          int j;

          char r = strR.charAt(0);

          switch (r) {

          case 'c':    // The receiver is a usual component.
            br.skipWhites();
            j = Integer.parseInt(br.readWord());
            br.skipWhites();
            objNameR = br.readWord();
            br.skipWhites();
            objNumR = Integer.parseInt(br.readWord());

            if (j >= _numOfElements) {
              StringWriter buffer = new StringWriter();
              PrintWriter  out    = new PrintWriter(buffer);
              out.print(_scfFile + ": Illegal element number: c " + j);
              String errMsg = buffer.toString();
              out.close();
              throw new OhError(errMsg);
            }

            connect((Object)_iterators[i], objNameS, objNumS, (Object)_elements[j], objNameR, objNumR);

            break;

          case 'i':    // The receiver is a iterator component.
            br.skipWhites();
            j = Integer.parseInt(br.readWord());
            br.skipWhites();
            objNameR = br.readWord();
            br.skipWhites();
            objNumR = Integer.parseInt(br.readWord());

            if (j >= _numOfIterators) {
              StringWriter buffer = new StringWriter();
              PrintWriter  out    = new PrintWriter(buffer);
              out.print(_scfFile + ": Illegal iterator element number: i " + j);
              String errMsg = buffer.toString();
              out.close();
              throw new OhError(errMsg);
            }

            connect((Object)_iterators[i], objNameS, objNumS, (Object)_iterators[j], objNameR, objNumR);

            break;

          case 's':    // The receiver is a send port.
            br.skipWhites();
            j = Integer.parseInt(br.readWord());
            
            if (j >= _numOfSendPorts) {
              StringWriter buffer = new StringWriter();
              PrintWriter  out    = new PrintWriter(buffer);
              out.print(_scfFile + ": Illegal output port number: s " + j);
              String errMsg = buffer.toString();
              out.close();
              throw new OhError(errMsg);
            }

            connect((Object)_iterators[i], objNameS, objNumS, (Object)_sendPorts[j]);

            break;

          default:
            throw new OhError("");
          }
        }
        else {
          throw new OhError("");
        }
      }  // -------------------------------------------------------------- ends while(true) loop.
    }
    catch (IOException e) {
      e.printStackTrace();
    }
  }


  /**
   * <japanese>
   * nf̒ڒʐMڑ
   * <a href="OhElement.html#register(Object, int)">OhElement.register(Object, int)</a>
   * pĒ`܂B
   *
   * <pre>
   * [ConnectComponents]
   * c 2 c 3 0 # mElmnt[2]->Register((void*)mElmnt[3], 0)
   * i 0 i 1 2 # mIte[0]->Register((void*)mIte[1], 2);
   * z # end mark
   * </pre>
   * </japanese>
   *
   * @exception OhError if an error occurs
   */
  public void connectComponents() throws OhError {    //; (*6)
    // Open the SCF. Comment strings are removed.
    //
    FileReader       fr = null;
    OhBufferedReader br = null;

    try {
      fr = new FileReader(_scfFile);
      br = new OhBufferedReader(fr, true);
    }
    catch (FileNotFoundException e) {
      e.printStackTrace();
    }


    String  strLine = null;
    boolean foundP  = false;

    try {
      br.skipWhites();
      while ((strLine = br.readWord()) != null) {
        if (strLine.equals("[ConnectComponents]")) {
          foundP = true;
          break;
        }
      }
    }
    catch (IOException e) {
      e.printStackTrace();
    }

    if (foundP == false) {
      throw new OhError("No [ConnectComponents] section.");
    }

    try {
      while (true) {
        String strCom1;
        String strCom2;
        int i1;
        int i2;
        int j;
        
        br.skipWhites();
        strCom1 = br.readWord();
        
        if (strCom1.charAt(0) == 'z') {
          break;
        }
        
        br.skipWhites();
        i1 = Integer.parseInt(br.readWord());
        br.skipWhites();
        strCom2 = br.readWord();
        br.skipWhites();
        i2 = Integer.parseInt(br.readWord());
        br.skipWhites();
        j = Integer.parseInt(br.readWord());
        
        OhElement elm1;
        OhElement elm2;
        
        elm1 = (strCom1.charAt(0) == 'c') ? _elements[i1] : _iterators[i1];
        elm2 = (strCom2.charAt(0) == 'c') ? _elements[i2] : _iterators[i2];
        
        elm1.register((Object)elm2, j);
      }
    }
    catch (IOException e) {
      e.printStackTrace();
    }
  }


  /**
   * <japanese>
   * ̓Xg[ aBr  _currentTime ǂݍ݂܂B
   * t@C _currentTime ǂݎꍇقƂǂƎv܂B
   * </japanese>
   * <p>
   * Set initial state by reading a file.
   *
   * @param aBr an <code>OhBufferedReader</code> value
   * @exception OhError if an error occurs
   */
  public void setInitialState(OhBufferedReader aBr) throws OhError {    //; (*9)
    int  year = 0;
    long tsec = 0L;
    
    try {
      aBr.skipWhites();
      year = Integer.parseInt(aBr.readWord());
      tsec = Long.parseLong(aBr.readWord());
    }
    catch (IOException e) {
      StringWriter buffer = new StringWriter();
      PrintWriter  out    = new PrintWriter(buffer);
      oPrint(out, "\t");
      String str = buffer.toString();
      out.close();
      throw new OhError("Error in OhScfTotalSystem.setInitialState().", str);
    }
    catch (NumberFormatException e) {
      StringWriter buffer = new StringWriter();
      PrintWriter  out    = new PrintWriter(buffer);
      oPrint(out, "\t");
      String str = buffer.toString();
      out.close();
      throw new OhError("Error in OhScfTotalSystem.setInitialState().", str);
    }

    if (year > 0) {
      _currentTime = new OhTimePoint(year, tsec);
    }
  }

  // @end {methods} ------------------------------------------------------

} // OhScfSubSystem
