/**
 * @(#)  PowerLawKinematicWave; class to express Power Law Kinematic Wave model for Stream Flow.
 *
 */

package ohymosj;

import ohymosj.*;
import java.util.*;
import java.io.*;


/**
 * PowerLawKinematicWave;
 *
 * Power Law Kinematic Wave model for Stream Flow.
 *
 * <japanese>
 * ͓Ԃ̗w@ kinematic wave @ŒǐՂ邽߂̃NXłB<p>
 *
 * ͓Ԃ̗w@ kinematic wave @ŒǐՂ܂B
 * ŁA͓ԂƂ͂̓ɍ_ȂP{͓̉Ԃ̂Ƃ܂B
 *  Q ́Aʐfʐ A ̎w֐ a*A^m ŕ\ł̂Ƃ܂B<p>
 * </japanese>
 *
 * @author <a href="mailto:satoh@wr.kuciv.kyoto-u.ac.jp">satoh</a>
 * @version $Revision: 1.38 $ $Date: 2004/02/10 20:08:11 $
 */
/*
y 1. NX OhElement pBz
*/
public class PowerLawKinematicWave extends OhElement
{
  // @begin {fields} -----------------------------------------------------

/*
y 2. M[qEM[qA܂͎M[qEM[q̔zoϐƂĒ`Bz
*/
  /**
   * Structural variable:
   * number of inflows from the upstream ends.
   * _numOfUpIn >= 0.
   *
   */
  protected int _numOfUpIn;

  /**
   * Structural variable:
   * number of lateral inflows.
   * _numOfLateralIn >= 0.
   *
   */
  protected int _numOfLateralIn;

  /**
   * inflow ports from the upstream end.
   *
   */
  protected OhReceiveDouble[] _rpUpIn;

  /**
   * lateral inflow ports.
   * Lateral inflow rate per unit length of stream will be calculated
   * by multiplying constant _cLateralIn[i] with the value obtained from _rpLateralIn[i]
   *
   */
  protected OhReceiveDouble[] _rpLateralIn;

  /**
   * Send port.
   * Runoff discharge (m^3/sec) at the slopeend or the stream end. 
   *
   * <japanese>
   * <p>
   * M[q͂PłB
   * f[^ <a href="OhDataPackDouble.html">OhDataPackDouble</a> ^łB
   * f[^Ԍ^
   * <a href="DataConnectionType.html#BROKENLINE">DataConnectionType.BROKENLINE</a>
   * łB<p>
   *
   * ͓[̗oʂ܂B
   * vZꂽoʂ̎ [m^3/sec] łB
   * </japanese>
   *
   */
  protected OhSendDouble _sp;

/*
y 3. p^Eԗʂ̋LɗpϐoϐƂĒ`Bz
*/
  // Parameters.

  /**
   * Parameter:
   * kinematic constant <b>a</b> [m/sec] in
   *
   * <pre>
   *   Q = a*A^m
   * </pre>
   *
   */
  protected double _a;

  /**
   * Parameter:
   * kinematic constant <b>m</b> [-] in
   *
   * <pre>
   *   Q = a*A^m
   * </pre>
   *
   */
  protected double _m;

  /**
   * Parameter:
   * length of the stream segment. [m]
   *
   * <japanese>
   * <p>
   * ͓Ԃ̒B[m]
   * </japanese>
   *
   */
  protected double _lengthOfStream;

  /**
   * Parameter:
   * number of divisions of the stream. [-]
   *
   * <japanese>
   * <p>
   * ͓Ԃ𓙕B[-]
   * </japanese>
   *
   */
  protected int _numOfSepStream;

  /**
   * Parameter:
   * time step [sec] for finite difference calculation.
   * Time step is fixed in this calculation model.
   *
   * <japanese>
   * <p>
   * vZԊԊuB[sec] <p>
   *
   * ̃fł͌ŒlƂĈ܂B
   * </japanese>
   *
   */
  protected int _dt;

  /**
   * Parameter:
   * time weighting parameter [-].
   * The standard value of theta is 0.5. 
   * If theta = 0, the finite difference calculation scheme becomes an explicit one.
   *
   */
  protected double _theta;

  /**
   * Parameter:
   * tolerance level of diffrence of two consecutive caluculated discharges to stop the iteration. 
   *
   */
  protected double _epsilon;

  /**
   * Parameter:
   * tolerance level of diffrence of two consecutive caluculated discharges to stop the iteration. 
   *
   */
  protected double _qLevel;

  /**
   * Parameter:
   * _cUpIn[i] is a multiplication factor [-] to the i-th inflow port from upstream end, _rpUpIn[i].
   *
   */
  protected double[] _cUpIn;

  /**
   * Parameter:
   * _cLateralIn[i] is a multiplication factor [-] to the i-th lateral inflow port _rpLateralIn[i].
   *
   */
  protected double[] _cLateralIn;


  // State variable.

  /**
   * State variable:
   * discharge Q [m^3/sec] ( = q * w) at current time.
   *
   */
  protected double[] _qCrnt;


  // Working variables.

  /**
   * Working variable:
   * length of the division. [m]
   *
   * <pre>
   *   _dx = _lengthOfStream / _numOfSepStream
   * </pre>
   *
   */
  protected double _dx;

  /**
   * Working variable:
   * discharge Q [m^3/sec] at next time.
   *
   */
  protected double[] _qNext;

  /**
   * <japanese>
   * ł̗ʂ̕zǂ̂悤ɗ^邩w肷ϐB
   * ONLY_DOWN_STREAM_END ȂA[ŗʂ^āA͓}B
   * EVERY_SECTION ȂAx = _dx, 2*_dx, ..., _numOfSepStream * _dx ̕_ŗʂ^B
   * </japanese>
   * <p>
   * Supplemental variable for setting initial state.
   *
   */
  protected InitialStateDataType _isdt;

  // @end {fields} -------------------------------------------------------


  // @begin {initialization} ---------------------------------------------

  /**
   * Initialization. <p>
   *
   * Uses init(String, int, String, int, int).
   *
   * @param aObjName a <code>String</code> value
   * @param aObjNum an <code>int</code> value
   * @param aBr an <code>OhBufferedReader</code> value
   * @return a <code>boolean</code> value
   * @exception OhError if an error occurs
   */
  public boolean init(String aObjName, int aObjNum, OhBufferedReader aBr) throws OhError {
    int numOfUpIn;
    int numOfLateralIn;

    try {
      aBr.skipWhites();
      numOfUpIn      = Integer.parseInt(aBr.readWord());
      aBr.skipWhites();
      numOfLateralIn = Integer.parseInt(aBr.readWord());
    }
    catch (IOException e) {
      StringWriter sw = new StringWriter();
      PrintWriter  pw = new PrintWriter(sw);
      oPrint(pw, "\t");
      String errStr = sw.toString();
      pw.close();
      throw new OhError(errStr);
    }
    catch (NumberFormatException e) {
      StringWriter sw = new StringWriter();
      PrintWriter  pw = new PrintWriter(sw);
      oPrint(pw, "\t");
      String errStr = sw.toString();
      pw.close();
      throw new OhError(errStr);
    }

    return init(aObjName, aObjNum, null, numOfUpIn, numOfLateralIn);
  }

  /**
   * Initialization. <p>
   *
   * Uses OhElement.init(String, int, int, String).
   *
   * @param aObjName a <code>String</code> value
   * @param aObjNum an <code>int</code> value
   * @param aDataStr a <code>String</code> value
   * @param aNumOfUpIn an <code>int</code> value
   * @param aNumOfLateralIn an <code>int</code> value
   * @return a <code>boolean</code> value
   * @exception OhError if an error occurs
   */
  public boolean init(String aObjName, int aObjNum, String aDataStr,
    int aNumOfUpIn, int aNumOfLateralIn) throws OhError {

    _numOfUpIn      = aNumOfUpIn;
    _numOfLateralIn = aNumOfLateralIn;

    if (_numOfUpIn < 0)
      _numOfUpIn = 0;

    if (_numOfLateralIn < 0)
      _numOfLateralIn = 0;

    _rpUpIn      = new OhReceiveDouble[_numOfUpIn];
    _rpLateralIn = new OhReceiveDouble[_numOfLateralIn];

    _cUpIn      = new double[_numOfUpIn];
    _cLateralIn = new double[_numOfLateralIn];

    if (_numOfUpIn > 0) {
      for (int i = 0; i < _numOfUpIn; i++) {
        _rpUpIn[i] = new OhReceiveDouble();  //; added by Satoh.
        _rpUpIn[i].init("rpUpIn", i);
      }
    }

    if (_numOfLateralIn > 0) {
      for (int i = 0; i < _numOfLateralIn; i++) {
        _rpLateralIn[i] = new OhReceiveDouble();  //; added by Satoh.
        _rpLateralIn[i].init("rpLateralIn", i);
      }
    }

    _sp = new OhSendDouble();  //; added by Satoh.
    _sp.init("sp", 0);

    return super.init(aObjName, aObjNum, 0, aDataStr);
  }
  
  // @end {initialization} -----------------------------------------------

/*
y 4. RXgN^`Bz
*/
  // @begin {constructors} -----------------------------------------------

  /**
   * Creates a new default <code>PowerLawKinematicWave</code> instance.
   *
   * @exception OhError if an error occurs
   */
  public PowerLawKinematicWave() {
    super();

    _cUpIn       = null;
    _cLateralIn  = null;
    _qCrnt       = null;
    _qNext       = null;
    _rpUpIn      = null;
    _rpLateralIn = null;
    
    _className = "PowerLawKinematicWave";
  }

  /**
   * Creates a new <code>SimpleDam</code> instance.
   *
   * Uses init(String, int, String, int, int).
   *
   * @param aObjName a <code>String</code> value
   * @param aObjNum an <code>int</code> value
   * @param aDataStr a <code>String</code> value
   * @param aNumOfUpIn an <code>int</code> value
   * @param aNumOfLateralIn an <code>int</code> value
   * @exception OhError if an error occurs
   */
  public PowerLawKinematicWave(String aObjName, int aObjNum, String aDataStr,
                               int aNumOfUpIn, int aNumOfLateralIn) throws OhError {
    super();

    _cUpIn       = null;
    _cLateralIn  = null;
    _qCrnt       = null;
    _qNext       = null;
    _rpUpIn      = null;
    _rpLateralIn = null;

    _className = "PowerLawKinematicWave";

    init(aObjName, aObjNum, aDataStr, aNumOfUpIn, aNumOfLateralIn);
  }
  
  // @end {constructors} -------------------------------------------------

  // @begin {methods} ----------------------------------------------------

/*
y 5. M[qEM[qo^郁\bh registerReceivePorts(), registerSendPorts() `Bz
*/
  /**
   * <japanese>
   * SM[q̓o^ <p>
   *
   * ̃\bh́A̗vffׂĂ̎M[q
   * <a href="OhElement.html#register(Object)">OhElement.register(Object)</a>
   * <a href="OhElement.html#register(Object, int)">OhElement.register(Object, int)</a>
   * ɂo^܂B<p>
   *
   * o^ĂȂ[q͐ڑłȂ̂őMɗpł܂B
   * </japanese>
   *
   * @exception OhError if an error occurs
   */
  public void registerReceivePorts() throws OhError {   //; (*1)
    for (int i = 0; i < _numOfUpIn; i++) {
      register((Object)_rpUpIn[i]);
    }

    for (int i = 0; i < _numOfLateralIn; i++) {
      register((Object)_rpLateralIn[i]);
    }
  }

  /**
   * <japanese>
   * hNX̑SM[q̓o^ <p>
   *
   * ̃\bh́A̗vffׂĂ̑M[q
   * <a href="OhElement.html#register(Object)">OhElement.register(Object)</a>
   * <a href="OhElement.html#register(Object, int)">OhElement.register(Object, int)</a>
   * ɂo^܂B<p>
   *
   * o^ĂȂ[q͐ڑłȂ̂őMɗpł܂B
   * </japanese>
   *
   * @exception OhError if an error occurs
   */
  public void registerSendPorts() throws OhError {
    register((Object)_sp);
  }

/*
y 6. p^Eԗʂǂݍރ\bh setParameter(), setInitialState() `Bz
*/
  /**
   * <japanese>
   * p^̐ݒ <p>
   *
   * ̓Xg[ aBr Ŏw肳t@Cf[^ǂ݂Ƃăp^ݒ肵܂B
   * </japanese>
   *
   * @param aBr an <code>OhBufferedReader</code> value
   * @exception OhError if an error occurs
   */
  public void setParameter(OhBufferedReader aBr) throws OhError {  //; (*2)
    try {
      aBr.skipWhites();
      _a = Double.parseDouble(aBr.readWord());
      _m = Double.parseDouble(aBr.readWord());
      _lengthOfStream = Double.parseDouble(aBr.readWord());
      _numOfSepStream = Integer.parseInt(aBr.readWord());
      _dt = Integer.parseInt(aBr.readWord());
      _theta = Double.parseDouble(aBr.readWord());
      _qLevel = Double.parseDouble(aBr.readWord());
      _epsilon = Double.parseDouble(aBr.readWord());
    }
    catch (IOException e) {
      StringWriter sw = new StringWriter();
      PrintWriter  pw = new PrintWriter(sw);
      oPrint(pw, "\t");
      String errStr = sw.toString();
      pw.close();
      throw new OhError(errStr);
    }
    catch (NumberFormatException e) {
      StringWriter sw = new StringWriter();
      PrintWriter  pw = new PrintWriter(sw);
      oPrint(pw, "\t");
      String errStr = sw.toString();
      pw.close();
      throw new OhError(errStr);
    }

    _qCrnt = new double[_numOfSepStream + 1];
    _qNext = new double[_numOfSepStream + 1];

    try {
      if (_numOfUpIn > 0) {  //; 㗬[̗ꍇAf[^n̏d݂ǂݎB
        for (int i = 0; i < _numOfUpIn; i++) {
          aBr.skipWhites();
          _cUpIn[i] = Double.parseDouble(aBr.readWord());
        }
      }

      if (_numOfLateralIn > 0) {  //; ̗ꍇAf[^n̏d݂ǂݎB
        for (int i = 0; i < _numOfLateralIn; i++) {
          aBr.skipWhites();
          _cLateralIn[i] = Double.parseDouble(aBr.readWord());
        }
      }
    }
    catch (IOException e) {
      StringWriter sw = new StringWriter();
      PrintWriter  pw = new PrintWriter(sw);
      oPrint(pw, "\t");
      String errStr = sw.toString();
      pw.close();
      throw new OhError(errStr);
    }
    catch (NumberFormatException e) {
      StringWriter sw = new StringWriter();
      PrintWriter  pw = new PrintWriter(sw);
      oPrint(pw, "\t");
      String errStr = sw.toString();
      pw.close();
      throw new OhError(errStr);
    }

    _dx = _lengthOfStream / _numOfSepStream;
  }

  /**
   * <japanese>
   * ԗʂ̐ݒ <p>
   *
   * ̓Xg[ aBr Ŏw肳t@Cf[^ǂ݂Ƃďԗʂݒ肵܂B
   * </japanese>
   *
   * @param aBr an <code>OhBufferedReader</code> value
   * @exception OhError if an error occurs
   */
  public void setInitialState(OhBufferedReader aBr) throws OhError {  //;(*3)
    /* set initial state */

    int  year = 0;
    long tsec = 0L;
    
    try {
      aBr.skipWhites();
      year = Integer.parseInt(aBr.readWord());
      tsec = Long.parseLong(aBr.readWord());
    }
    catch (IOException e) {
      StringWriter buffer = new StringWriter();
      PrintWriter  out    = new PrintWriter(buffer);
      oPrint(out, "\t");
      String errStr = buffer.toString();
      out.close();
      throw new OhError("Error in setInitialState().", errStr);
    }
    catch (NumberFormatException e) {
      StringWriter buffer = new StringWriter();
      PrintWriter  out    = new PrintWriter(buffer);
      oPrint(out, "\t");
      String errStr = buffer.toString();
      out.close();
      throw new OhError("Error in setInitialState().", errStr);
    }

    if (year > 0) {
      _currentTime = new OhTimePoint(year, tsec);
    }

    int dataType;  // 0:ONLY_DOWN_STREAM_END 1:EVERY_SECTION

    try {
      aBr.skipWhites();
      dataType = Integer.parseInt(aBr.readWord());
    }
    catch (IOException e) {
      StringWriter buffer = new StringWriter();
      PrintWriter  out    = new PrintWriter(buffer);
      oPrint(out, "\t");
      String errStr = buffer.toString();
      out.close();
      throw new OhError("Error in setInitialState().", errStr);
    }
    catch (NumberFormatException e) {
      StringWriter buffer = new StringWriter();
      PrintWriter  out    = new PrintWriter(buffer);
      oPrint(out, "\t");
      String errStr = buffer.toString();
      out.close();
      throw new OhError("Error in setInitialState().", errStr);
    }

    if (dataType == 0)
      _isdt = InitialStateDataType.ONLY_DOWN_STREAM_END;
    else
      _isdt = InitialStateDataType.EVERY_SECTION;

    try {
      if (_isdt == InitialStateDataType.ONLY_DOWN_STREAM_END) {
        aBr.skipWhites();
        _qCrnt[_numOfSepStream] = Double.parseDouble(aBr.readWord());
      }
      else {
        for (int i = 1; i <= _numOfSepStream; i++) {
          _qCrnt[_numOfSepStream] = Double.parseDouble(aBr.readWord());
        }
      }
    }
    catch (IOException e) {
      StringWriter buffer = new StringWriter();
      PrintWriter  out    = new PrintWriter(buffer);
      oPrint(out, "\t");
      String errStr = buffer.toString();
      out.close();
      throw new OhError("Error in setInitialState().", errStr);
    }
    catch (NumberFormatException e) {
      StringWriter buffer = new StringWriter();
      PrintWriter  out    = new PrintWriter(buffer);
      oPrint(out, "\t");
      String errStr = buffer.toString();
      out.close();
      throw new OhError("Error in setInitialState().", errStr);
    }
  }

/*
y 7. o̓\bh initialOutput() `Bz
*/
  public boolean initialOutput() throws OhError {    //; (*5)
    /* initial output */

    if (_numOfUpIn == 0) {
      _qCrnt[0] = 0.;
      if (_isdt == InitialStateDataType.ONLY_DOWN_STREAM_END) {
        int i = 0;
        for (i = 1; i < _numOfSepStream; i++) {
          _qCrnt[i] = _qCrnt[_numOfSepStream] * i / (double)_numOfSepStream;
        }
      }
    }
    else {
      double qIn = 0.;
      int j;
      for (j = 0; j < _numOfUpIn; j++) {
        if (_rpUpIn[j].canYouGetData(_currentTime) == false) {
          return false;
        }
        qIn += _cUpIn[j] * _rpUpIn[j].getData(_currentTime);
      }

      if (_isdt == InitialStateDataType.ONLY_DOWN_STREAM_END) {
        int i;
        for (i = 0; i < _numOfSepStream; i++) {
          _qCrnt[i] = ( _qCrnt[_numOfSepStream] * i + qIn * (_numOfSepStream - 1))
                      / (double)_numOfSepStream;
        }
      }
    }

    _sp.setCommentString("OhTimePoint.Year OhTimePoint.Tsec(sec) discharge(m**3/sec)");

    _sp.sendData(_currentTime, _qCrnt[_numOfSepStream]);

    return true;
  }

/*
y 8. ŏIԗʂރ\bh saveTerminalState() `Bz
*/
  /**
   * <japanese>
   * ŏIԂۑ郁\bh
   * </japanese>
   *
   * @param aPw a <code>PrintWriter</code> value
   */
  public void saveTerminalState(PrintWriter aPw) {    //; (*4)
    /* output terminal state into a file */

    aPw.print(_currentTime + " # _currentTime");
    aPw.print(NEWLINE);
    aPw.print("1 # EVERY_SECTION");
    aPw.print(NEWLINE);

    for (int i = 1; i <= _numOfSepStream; i++) {
      aPw.print(_qCrnt[i] + " # Section " + i);
      aPw.print(NEWLINE);
    }
  }

/*
y 9. vZԊԊuvZ郁\bh calculateTimeStep() `Bz
*/
  /**
   * <japanese>
   * vZԊԊu͌Œ肵lƂ܂B
   * </japanese>
   *
   * @return a <code>long</code> value
   * @exception OhError if an error occurs
   */
  public long calculateTimeStep() throws OhError {
    return (long)_dt;    // "_dt" is defined in pmt file.
  }

/*
y 10. vZ邱Ƃł邩𔻒f郁\bh canYouCalculate() `Bz
*/
  /**
   * <japanese>
   * vZ\̔f <p>
   *
   * <a href="OhElement.html">OhElement</a> NX
   * \bh <a href="OhElement.html#getNecessaryTimeFrom()">OhElement.getNecessaryTimeFrom()</a>,
   * <a href="OhElement.html#getNecessaryTimeTo()">OhElement.getNecessaryTimeTo()</a> 
   * `鎞Ԃ̊Ԃ̃f[^擾łΌvZ\Ɣf܂B
   * <a href="OhElement.html#getNecessaryTimeFrom()">OhElement.getNecessaryTimeFrom()</a> \bh
   * _currentTime ԂA
   * <a href="OhElement.html#getNecessaryTimeTo()">OhElement.getNecessaryTimeTo()</a> \bh
   * _currentTime + _timeStep ̒lԂ܂B<p>
   *
   * f[^擾s\łƕԎĂM[qɑ΂ČvZvo܂B
   * ̓Iɂ́AM[q calReq() bZ[W𑗂܂B
   * </japanese>
   *
   * @return a <code>boolean</code> value
   * @exception OhCannotGetDataError if an error occurs
   * @exception OhError if an error occurs
   */
  public boolean canYouCalculate() throws OhCannotGetDataError, OhError {
    return canYouCalculate0();
  }

/*
y 11. 1Xebv̌vZ郁\bh calculate() `Bz
*/
  /**
   * The solution of power law kimematic wave model by Keith Beven's method.
   *
   * <japanese>
   * <p>
   * Beven @ɂw@ kinematic wave f̉@
   * </japanese>
   *
   * @return a <code>boolean</code> value
   * @exception OhError if an error occurs
   */
  public boolean calculate() throws OhError {  //;(*6)
    double dta;
    dta = _timeStep;    //; "_timeStep" equals "_dt";

    /* Lateral inflow */

    double rLin = 0.;
    int i;

    OhTimePoint crnTime = new OhTimePoint();  //; For work.
    crnTime.copy(_currentTime);
    OhTimePoint nextTime = crnTime.add(_timeStep);

    for (i = 0; i < _numOfLateralIn; i++) {
      rLin += _cLateralIn[i] * _rpLateralIn[i].getMeanData(_currentTime, nextTime) / _lengthOfStream;
    }

    /* upstream end */

    _qNext[0] = 0.;

    for (i = 0; i < _numOfUpIn; i++) {
      _qNext[0] += _cUpIn[i] * _rpUpIn[i].getData(nextTime);
    }

    double qest;
    double cold;  //; `dx
    double cnew;  //; XVꂽ`dx

    double m1 = (_m - 1.) / _m;

    for (i = 1; i <= _numOfSepStream; i++) {
      double qbase = (_qCrnt[i-1] + _qCrnt[i] + _qNext[i-1]) / 3.;
      qest = _qNext[i-1] + rLin * _dx;
      if (_qCrnt[i] > qest) {
        qest = _qCrnt[i];
      }
      double qmean = 0.5 * (_qCrnt[i-1] + _qCrnt[i]);
      cold = _a * _m * Math.pow(qmean / _a, m1);


      while (true) {
        double qmean1 = 0.5 * (_qNext[i-1] + qest);
        cnew = _a * _m * Math.pow(qmean1 / _a, m1);

        _qNext[i] = (_qCrnt[i] + _theta * cnew * dta * (_qNext[i-1] / _dx + rLin)
          - (1. - _theta) * cold * dta *
          ((_qCrnt[i] - _qCrnt[i-1]) / _dx - rLin)) / (1. + _theta * cnew * dta / _dx);


        if (_qNext[i] < 0.)
          _qNext[i] = 0.;


        if (Math.abs(_qNext[i] - qest) < (_epsilon * (_qLevel + qbase)))
          break;

        qest = _qNext[i];
      }

      _qNext[i] = qest;
    }

  /* output */

    OhTimePoint crntTime = new OhTimePoint();  //; For work.
    crntTime.copy(_currentTime);
    OhTimePoint nextT = crntTime.add(_timeStep);

    _sp.sendData(nextT, _qNext[_numOfSepStream]);

  /* update state variable */

    for (i = 0; i <= _numOfSepStream; i++) {
      _qCrnt[i] = _qNext[i];
    }

    return true;
  }

/*
y 12. ̈ǍvZEo͍Ƃ郁\bh work() `Bz
*/
  /**
   * <japanese>
   * <p>
   * _timeStep 𓾂Ă calculate() s work0() ̗p܂B
   * </japanese>
   *
   * @return a <code>boolean</code> value
   * @exception OhCannotGetDataError if an error occurs
   * @exception OhError if an error occurs
   */
  public boolean work() throws OhCannotGetDataError, OhError {
    return work0();
  }  

} // PowerLawKinematicWave
