/**
 * @(#)  SimpleDam.java
 *
 */

package ohymosj;

import ohymosj.*;
import java.util.*;
import java.io.*;


/**
 * SimpleDam.
 *
 * Simple dam model.
 *
 * @author <a href="mailto:satoh@wr.kuciv.kyoto-u.ac.jp">satoh</a>
 * @version $Revision: 1.29 $ $Date: 2004/03/08 17:03:02 $
 */
public class SimpleDam extends OhElement
{
  // @begin {instance variable or static variable} -----------------------

  // 1. Receive ports and send ports.
  
  protected OhReceiveDouble[] _rP;

  protected OhSendDouble _sP0;
  protected OhSendDouble _sP1;

  // 2. Structural parameters.
  
  protected int _numberOfRp;

  // 3. Parameters.
  
  protected double _qPass;

  protected double _ratio;

  protected int _dt;

  protected double[] _cRp;

  // 4. State variables.

  protected double _storageCrnt;

  protected double _qCrnt;

  // 5. Working variable.

  protected double _storageNext;

  protected double _qNext;

  protected double _qInCrnt;

  protected double _qInNext;


  // @end {instance variable or static variable} -------------------------

  // @begin {initialization} ---------------------------------------------

  public boolean init(String aObjName, int aObjNum, String aDataStr, int aNrp) throws OhError {
    _numberOfRp = aNrp;

    if (_numberOfRp < 1)
      _numberOfRp = 1;

    _rP  = new OhReceiveDouble[_numberOfRp];
    _cRp = new double[_numberOfRp];

    for (int i = 0; i < _numberOfRp; i++) {
      _rP[i] = new OhReceiveDouble();     //; Added by Satoh. (2003.12.10)
      _rP[i].init("rp", i);
    }

    return super.init(aObjName, aObjNum, 0, aDataStr);
  }

  
  public boolean init(String aObjName, int aObjNum, OhBufferedReader aBr) throws OhError {
    int nrp;
    try {
      nrp = Integer.parseInt(aBr.readWord());
    }
    catch (IOException e) {
      StringWriter sw = new StringWriter();
      PrintWriter  pw = new PrintWriter(sw);
      oPrint(pw, "\t");
      String str = sw.toString();
      pw.close();
      throw new OhError(str);
    }
    catch (NumberFormatException e) {
      StringWriter sw = new StringWriter();
      PrintWriter  pw = new PrintWriter(sw);
      oPrint(pw, "\t");
      String str = sw.toString();
      pw.close();
      throw new OhError(str);
    }

    return init(aObjName, aObjNum, null, nrp);
  }
  
  // @end {initialization} -----------------------------------------------
  
  // @begin {constructors} -----------------------------------------------

  public SimpleDam() throws OhError {
    super();

    _rP  = null;
    _sP0 = new OhSendDouble("sp", 0);
    _sP1 = new OhSendDouble("sp", 1);
    _numberOfRp = 0;
    _cRp = null;

    _className = "SimpleDam";
  }


  public SimpleDam(String aObjName, int aObjNum, String aDataStr, int aNrp) throws OhError {
    super();

    _rP  = null;
    _sP0 = new OhSendDouble("sp", 0);
    _sP1 = new OhSendDouble("sp", 1);
    _cRp = null;

    _className = "SimpleDam";

    init(aObjName, aObjNum, aDataStr, aNrp);
  }
  
  // @end {constructors} -------------------------------------------------

  // @begin {methods} ----------------------------------------------------

  // 1. Methods about receive ports and send ports.

  public void registerReceivePorts() throws OhError {   //(*1)
    for (int i = 0; i < _numberOfRp; i++) {
      register((Object)_rP[i]);      //; Throws "OhError".
    }
  }

  public void registerSendPorts() throws OhError {
    register((Object)_sP0);    //; Throws "OhError".
    register((Object)_sP1);    //; Throws "OhError".
  }


  // 2. Parameter setting and initialization of states.

  public void setParameter(OhBufferedReader aBr) throws OhError {  //(*2)

    try {
      _qPass = Double.parseDouble(aBr.readWord());
      _ratio = Double.parseDouble(aBr.readWord());
      _dt = Integer.parseInt(aBr.readWord());
    }
    catch (IOException e) {
      StringWriter sw = new StringWriter();
      PrintWriter  pw = new PrintWriter(sw);
      oPrint(pw, "\t");
      String str = sw.toString();
      pw.close();
      throw new OhError(str);
    }
    catch (NumberFormatException e) {
      StringWriter sw = new StringWriter();
      PrintWriter  pw = new PrintWriter(sw);
      oPrint(pw, "\t");
      String str = sw.toString();
      pw.close();
      throw new OhError(str);
    }

    if (_qPass < 0.0) {
      StringWriter sw = new StringWriter();
      PrintWriter  pw = new PrintWriter(sw);
      pw.print("_qPass should be non-negative but is " + _qPass);
      String str = sw.toString();
      pw.close();
      throw new OhError(str);
    }

    if (_ratio < 0.0 || _ratio > 1.0) {
      StringWriter sw = new StringWriter();
      PrintWriter  pw = new PrintWriter(sw);
      pw.print("_ratio should be in [0.0, 1.0] but it is " + _qPass);
      String str = sw.toString();
      pw.close();
      throw new OhError(str);
    }

    if (_dt < 0) {
      StringWriter sw = new StringWriter();
      PrintWriter  pw = new PrintWriter(sw);
      pw.print("_dt should be positive but it is " + _dt);
      String str = sw.toString();
      pw.close();
      throw new OhError(str);
    }

    if (_numberOfRp > 0) {  //; Weights for inflow receive ports.
      for (int i = 0; i < _numberOfRp; i++) {
        try {
          _cRp[i] = Double.parseDouble(aBr.readWord());
        }
        catch (IOException e) {
          StringWriter sw = new StringWriter();
          PrintWriter  pw = new PrintWriter(sw);
          oPrint(pw, "\t");
          String str = sw.toString();
          pw.close();
          throw new OhError(str);
        }
        catch (NumberFormatException e) {
          StringWriter sw = new StringWriter();
          PrintWriter  pw = new PrintWriter(sw);
          oPrint(pw, "\t");
          String str = sw.toString();
          pw.close();
          throw new NumberFormatException(str);
        }
      }
    }
  }


  public void saveParameter (PrintWriter aPw) {
    aPw.print(_qPass + " " + _ratio + " " + _dt + NEWLINE);
    for (int i = 0; i < _numberOfRp; i++) {
      aPw.print(_cRp[i] + " ");
    }
    aPw.print(NEWLINE);
  }

  public void setInitialState(OhBufferedReader aBr) throws OhError {  //(*3)
    int  year = 0;
    long tsec = 0L;
    
    try {
      year = Integer.parseInt(aBr.readWord());
      tsec = Long.parseLong(aBr.readWord());
    }
    catch (IOException e) {
      StringWriter buffer = new StringWriter();
      PrintWriter  out    = new PrintWriter(buffer);
      oPrint(out, "\t");
      String str = buffer.toString();
      out.close();
      throw new OhError("Error in setInitialState().", str);
    }
    catch (NumberFormatException e) {
      StringWriter buffer = new StringWriter();
      PrintWriter  out    = new PrintWriter(buffer);
      oPrint(out, "\t");
      String str = buffer.toString();
      out.close();
      throw new OhError("Error in setInitialState().", str);
    }
    
    
    if (year > 0) {
      _currentTime = new OhTimePoint(year, tsec);
    }

    try {
      _storageCrnt = Double.parseDouble(aBr.readWord());
      //;
      //; _qCrnt will be calculated in initialOutput().
    }
    catch (IOException e) {
      StringWriter sw = new StringWriter();
      PrintWriter  pw = new PrintWriter(sw);
      oPrint(pw, "\t");
      String str = sw.toString();
      pw.close();
      throw new OhError(str);
    }
    catch (NumberFormatException e) {
      StringWriter sw = new StringWriter();
      PrintWriter  pw = new PrintWriter(sw);
      oPrint(pw, "\t");
      String str = sw.toString();
      pw.close();
      throw new OhError(str);
    }
  }
  

  public void saveTerminalState(PrintWriter aPw) {    //(*4)
    aPw.print(_currentTime + " # _currentTime");
    aPw.print(NEWLINE);
    aPw.print(_storageCrnt + " # _storageCrnt (m^3)");
    aPw.print(NEWLINE);
  }


  // 3. Calculation.

  public boolean initialOutput() throws OhError {    //(*5)
    _qInCrnt = 0.0;

    if (_numberOfRp > 0) {
      int j = 0;

      for(j = 0; j < _numberOfRp; j++) {
        if (_rP[j].canYouGetData(_currentTime) == false) {
          return false;
        }

        _qInCrnt += _cRp[j] * (_rP[j].getData(_currentTime));
      }
    }

    if (_qInCrnt <= _qPass) {
      _qCrnt = _qInCrnt;
    }
    else {
      _qCrnt = _qPass + _ratio * (_qInCrnt - _qPass);
    }


    OhTimePoint crntT = new OhTimePoint();
    crntT.copy(_currentTime);
    
    _sP0.setCommentString("OhTimePoint.Year OhTimePoint.Tsec(sec) discharge(m**3/sec)");
    _sP0.sendData(crntT, _qCrnt);

    _sP1.setCommentString("OhTimePoint.Year OhTimePoint.Tsec(sec) storage(m**3)");
    _sP1.sendData(crntT, _storageCrnt);

    return true;
  }


  public long calculateTimeStep() throws OhError {
    return (long)_dt;  //; "_dt" is defined in pmt file.
  }

  public boolean canYouCalculate() throws OhCannotGetDataError, OhError {
    return canYouCalculate0();
  }

  public boolean work() throws OhCannotGetDataError, OhError {
    return work0();
  }



  
  public boolean calculate() throws OhError {  //(*6)
    double dta;
    dta = _timeStep;    //; "_timeStep" equals "_dt";

    _qInNext = 0.0;

    OhTimePoint crnTime = new OhTimePoint();
    crnTime.copy(_currentTime);
    OhTimePoint nextTime = crnTime.add(_timeStep);

    if (_numberOfRp > 0) {
      int j = 0;
      for (j = 0; j < _numberOfRp; j++) {
        if (_rP[j].canYouGetData(nextTime) == false) {
          return false;
        }
        _qInNext += _cRp[j] * (_rP[j].getData(nextTime));
      }
    }

    if (_qInNext <= _qPass) {
      _qNext = _qInNext;
    }
    else {
      _qNext = _qPass + _ratio * (_qInNext - _qPass);
    }
    
    _storageNext = _storageCrnt + dta * (_qInCrnt + _qInNext) * 0.5 - dta * (_qCrnt + _qNext) * 0.5;


    //; Output.
    //;
    _sP0.sendData(nextTime, _qNext);
    _sP1.sendData(nextTime, _storageNext);


    //; Standard output. (This is not especially required.)
    //;
//  System.out.println("time = " + nextTime + ", Q = " + _qNext + ", Storage = " + _storageNext);
    
    
    //; Update state variables.
    //;
    _storageCrnt = _storageNext;
    _qCrnt       = _qNext;
    _qInCrnt     = _qInNext;

    return true;
  }


  // @begin {methods} ----------------------------------------------------

} // SimpleDam
